@@ -17,20 +17,29 @@ import copy
 import h5py
 import numpy as np
 
-from yt.funcs import *
-
 from yt.analysis_modules.cosmological_observation.cosmology_splice import \
-     CosmologySplice
-from yt.convenience import load
+    CosmologySplice
+from yt.convenience import \
+    load
+from yt.funcs import \
+    mylog
+from yt.units.yt_array import \
+    YTArray
+from yt.utilities.cosmology import \
+    Cosmology
 from yt.utilities.parallel_tools.parallel_analysis_interface import \
     parallel_objects, \
     parallel_root_only
-from yt.utilities.physical_constants import \
-     speed_of_light_cgs, \
-     cm_per_km
 
 class LightRay(CosmologySplice):
     """
+    LightRay(parameter_filename, simulation_type=None,
+             near_redshift=None, far_redshift=None,
+             use_minimum_datasets=True, deltaz_min=0.0,
+             minimum_coherent_box_fraction=0.0,
+             time_data=True, redshift_data=True,
+             find_outputs=False, load_kwargs=None):
+
     Create a LightRay object.  A light ray is much like a light cone,
     in that it stacks together multiple datasets in order to extend a
     redshift interval.  Unlike a light cone, which does randomly
@@ -38,6 +47,8 @@ class LightRay(CosmologySplice):
     randomly oriented single rays.  The purpose of these is to create
     synthetic QSO lines of sight.
 
+    Light rays can also be made from single datasets.
+    
     Once the LightRay object is set up, use LightRay.make_light_ray to
     begin making rays.  Different randomizations can be created with a
     single object by providing different random seeds to make_light_ray.
@@ -45,26 +56,32 @@ class LightRay(CosmologySplice):
     Parameters
     ----------
     parameter_filename : string
-        The simulation dataset.
-    simulation_type : string
-        The simulation type.
-    near_redshift : float
-        The near (lowest) redshift for the light ray.
-    far_redshift : float
-        The far (highest) redshift for the light ray.  NOTE: in order 
-        to use only a single dataset in a light ray, set the 
-        near_redshift and far_redshift to be the same.
-    use_minimum_datasets : bool
+        The path to the simulation parameter file or dataset.
+    simulation_type : optional, string
+        The simulation type.  If None, the first argument is assumed to 
+        refer to a single dataset.
+        Default: None
+    near_redshift : optional, float
+        The near (lowest) redshift for a light ray containing multiple 
+        datasets.  Do not use is making a light ray from a single 
+        dataset.
+        Default: None
+    far_redshift : optional, float
+        The far (highest) redshift for a light ray containing multiple 
+        datasets.  Do not use is making a light ray from a single 
+        dataset.
+        Default: None
+    use_minimum_datasets : optional, bool
         If True, the minimum number of datasets is used to connect the
         initial and final redshift.  If false, the light ray solution
         will contain as many entries as possible within the redshift
         interval.
         Default: True.
-    deltaz_min : float
+    deltaz_min : optional, float
         Specifies the minimum :math:`\Delta z` between consecutive
         datasets in the returned list.
         Default: 0.0.
-    minimum_coherent_box_fraction : float
+    minimum_coherent_box_fraction : optional, float
         Used with use_minimum_datasets set to False, this parameter
         specifies the fraction of the total box size to be traversed
         before rerandomizing the projection axis and center.  This
@@ -72,45 +89,72 @@ class LightRay(CosmologySplice):
         coherent large scale structure, but in practice does not work
         so well.  Try setting this parameter to 1 and see what happens.
         Default: 0.0.
-    time_data : bool
+    time_data : optional, bool
         Whether or not to include time outputs when gathering
         datasets for time series.
         Default: True.
-    redshift_data : bool
+    redshift_data : optional, bool
         Whether or not to include redshift outputs when gathering
         datasets for time series.
         Default: True.
-    find_outputs : bool
+    find_outputs : optional, bool
         Whether or not to search for datasets in the current 
         directory.
         Default: False.
+    load_kwargs : optional, dict
+        Optional dictionary of kwargs to be passed to the "load" 
+        function, appropriate for use of certain frontends.  E.g.
+        Tipsy using "bounding_box"
+        Gadget using "unit_base", etc.
+        Default : None
 
     """
-    def __init__(self, parameter_filename, simulation_type,
-                 near_redshift, far_redshift,
+    def __init__(self, parameter_filename, simulation_type=None,
+                 near_redshift=None, far_redshift=None,
                  use_minimum_datasets=True, deltaz_min=0.0,
                  minimum_coherent_box_fraction=0.0,
                  time_data=True, redshift_data=True,
-                 find_outputs=False):
+                 find_outputs=False, load_kwargs=None):
 
         self.near_redshift = near_redshift
         self.far_redshift = far_redshift
         self.use_minimum_datasets = use_minimum_datasets
         self.deltaz_min = deltaz_min
         self.minimum_coherent_box_fraction = minimum_coherent_box_fraction
-
+        self.parameter_filename = parameter_filename
+        if load_kwargs is None:
+            self.load_kwargs = {}
+        else:
+            self.load_kwargs = load_kwargs
         self.light_ray_solution = []
         self._data = {}
 
-        # Get list of datasets for light ray solution.
-        CosmologySplice.__init__(self, parameter_filename, simulation_type,
-                                 find_outputs=find_outputs)
-        self.light_ray_solution = \
-          self.create_cosmology_splice(self.near_redshift, self.far_redshift,
-                                       minimal=self.use_minimum_datasets,
-                                       deltaz_min=self.deltaz_min,
-                                       time_data=time_data,
-                                       redshift_data=redshift_data)
+        # Make a light ray from a single, given dataset.        
+        if simulation_type is None:
+            ds = load(parameter_filename, **self.load_kwargs)
+            if ds.cosmological_simulation:
+                redshift = ds.current_redshift
+                self.cosmology = Cosmology(
+                    hubble_constant=ds.hubble_constant,
+                    omega_matter=ds.omega_matter,
+                    omega_lambda=ds.omega_lambda,
+                    unit_registry=ds.unit_registry)
+            else:
+                redshift = 0.
+            self.light_ray_solution.append({"filename": parameter_filename,
+                                            "redshift": redshift})
+
+        # Make a light ray from a simulation time-series.
+        else:
+            # Get list of datasets for light ray solution.
+            CosmologySplice.__init__(self, parameter_filename, simulation_type,
+                                     find_outputs=find_outputs)
+            self.light_ray_solution = \
+              self.create_cosmology_splice(self.near_redshift, self.far_redshift,
+                                           minimal=self.use_minimum_datasets,
+                                           deltaz_min=self.deltaz_min,
+                                           time_data=time_data,
+                                           redshift_data=redshift_data)
 
     def _calculate_light_ray_solution(self, seed=None, 
                                       start_position=None, end_position=None,
@@ -134,7 +178,7 @@ class LightRay(CosmologySplice):
             else:
                 # assume trajectory given as r, theta, phi
                 if len(trajectory) != 3:
-                    raise RuntimeError("LightRay Error: trajectory must have lenght 3.")
+                    raise RuntimeError("LightRay Error: trajectory must have length 3.")
                 r, theta, phi = trajectory
                 self.light_ray_solution[0]['end'] = self.light_ray_solution[0]['start'] + \
                   r * np.array([np.cos(phi) * np.sin(theta),
@@ -169,7 +213,7 @@ class LightRay(CosmologySplice):
                                 (self.light_ray_solution[q]['redshift'], z_next,
                                  self.light_ray_solution[q]['traversal_box_fraction']))
                     mylog.error("Full box delta z is %f, but it is %f to the next data dump." %
-                                (self.light_ray_solution[q]['deltazMax'],
+                                (self.light_ray_solution[q]['dz_max'],
                                  self.light_ray_solution[q]['redshift']-z_next))
 
                 # Get dataset axis and center.
@@ -210,11 +254,17 @@ class LightRay(CosmologySplice):
     def make_light_ray(self, seed=None,
                        start_position=None, end_position=None,
                        trajectory=None,
-                       fields=None,
+                       fields=None, setup_function=None,
                        solution_filename=None, data_filename=None,
-                       get_los_velocity=True,
+                       get_los_velocity=True, redshift=None,
                        njobs=-1):
         """
+        make_light_ray(seed=None, start_position=None, end_position=None,
+                       trajectory=None, fields=None, setup_function=None,
+                       solution_filename=None, data_filename=None,
+                       get_los_velocity=True, redshift=None,
+                       njobs=-1)
+
         Create a light ray and get field values for each lixel.  A light
         ray consists of a list of field values for cells intersected by
         the ray and the path length of the ray through those cells.
@@ -222,37 +272,48 @@ class LightRay(CosmologySplice):
 
         Parameters
         ----------
-        seed : int
+        seed : optional, int
             Seed for the random number generator.
             Default: None.
-        start_position : list of floats
+        start_position : optional, list of floats
             Used only if creating a light ray from a single dataset.
             The coordinates of the starting position of the ray.
             Default: None.
-        end_position : list of floats
+        end_position : optional, list of floats
             Used only if creating a light ray from a single dataset.
             The coordinates of the ending position of the ray.
             Default: None.
-        trajectory : list of floats
+        trajectory : optional, list of floats
             Used only if creating a light ray from a single dataset.
             The (r, theta, phi) direction of the light ray.  Use either 
             end_position or trajectory, not both.
             Default: None.
-        fields : list
+        fields : optional, list
             A list of fields for which to get data.
             Default: None.
-        solution_filename : string
+        setup_function : optional, callable, accepts a ds
+            This function will be called on each dataset that is loaded 
+            to create the light ray.  For, example, this can be used to 
+            add new derived fields.
+            Default: None.
+        solution_filename : optional, string
             Path to a text file where the trajectories of each
             subray is written out.
             Default: None.
-        data_filename : string
+        data_filename : optional, string
             Path to output file for ray data.
             Default: None.
-        get_los_velocity : bool
+        get_los_velocity : optional, bool
             If True, the line of sight velocity is calculated for
             each point in the ray.
             Default: True.
-        njobs : int
+        redshift : optional, float
+            Used with light rays made from single datasets to specify a 
+            starting redshift for the ray.  If not used, the starting 
+            redshift will be 0 for a non-cosmological dataset and 
+            the dataset redshift for a cosmological dataset.
+            Default: None.
+        njobs : optional, int
             The number of parallel jobs over which the segments will 
             be split.  Choose -1 for one processor per segment.
             Default: -1.
@@ -260,16 +321,32 @@ class LightRay(CosmologySplice):
         Examples
         --------
 
-        >>> from yt.mods import *
-        >>> from yt.analysis_modules.cosmological_analysis.light_ray.api import \
+        Make a light ray from multiple datasets:
+        
+        >>> import yt
+        >>> from yt.analysis_modules.cosmological_observation.light_ray.api import \
         ...     LightRay
-        >>> my_ray = LightRay('enzo_tiny_simulation/32Mpc_32.enzo', 'Enzo',
+        >>> my_ray = LightRay("enzo_tiny_cosmology/32Mpc_32.enzo", "Enzo",
         ...                   0., 0.1, time_data=False)
         ...
         >>> my_ray.make_light_ray(seed=12345,
-        ...                       solution_filename='solution.txt',
-        ...                       data_filename='my_ray.h5',
-        ...                       fields=['temperature', 'density'],
+        ...                       solution_filename="solution.txt",
+        ...                       data_filename="my_ray.h5",
+        ...                       fields=["temperature", "density"],
+        ...                       get_los_velocity=True)
+
+        Make a light ray from a single dataset:
+
+        >>> import yt
+        >>> from yt.analysis_modules.cosmological_observation.light_ray.api import \
+        ...     LightRay
+        >>> my_ray = LightRay("IsolatedGalaxy/galaxy0030/galaxy0030")
+        ...
+        >>> my_ray.make_light_ray(start_position=[0., 0., 0.],
+        ...                       end_position=[1., 1., 1.],
+        ...                       solution_filename="solution.txt",
+        ...                       data_filename="my_ray.h5",
+        ...                       fields=["temperature", "density"],
         ...                       get_los_velocity=True)
         
         """
@@ -288,9 +365,9 @@ class LightRay(CosmologySplice):
         all_fields = fields[:]
         all_fields.extend(['dl', 'dredshift', 'redshift'])
         if get_los_velocity:
-            all_fields.extend(['x-velocity', 'y-velocity',
-                               'z-velocity', 'los_velocity'])
-            data_fields.extend(['x-velocity', 'y-velocity', 'z-velocity'])
+            all_fields.extend(['velocity_x', 'velocity_y',
+                               'velocity_z', 'velocity_los'])
+            data_fields.extend(['velocity_x', 'velocity_y', 'velocity_z'])
 
         all_ray_storage = {}
         for my_storage, my_segment in parallel_objects(self.light_ray_solution,
@@ -298,18 +375,35 @@ class LightRay(CosmologySplice):
                                                        njobs=njobs):
 
             # Load dataset for segment.
-            ds = load(my_segment['filename'])
-            my_segment["start"] = ds.domain_width * my_segment["start"] + \
-                ds.domain_left_edge
-            my_segment["end"] = ds.domain_width * my_segment["end"] + \
-                ds.domain_left_edge
+            ds = load(my_segment['filename'], **self.load_kwargs)
 
-            if self.near_redshift == self.far_redshift:
+            my_segment['unique_identifier'] = ds.unique_identifier
+            if redshift is not None:
+                if ds.cosmological_simulation and redshift != ds.current_redshift:
+                    mylog.warn("Generating light ray with different redshift than " +
+                               "the dataset itself.")
+                my_segment["redshift"] = redshift
+
+            if setup_function is not None:
+                setup_function(ds)
+
+            if start_position is not None:
+                my_segment["start"] = ds.arr(my_segment["start"], "code_length")
+                my_segment["end"] = ds.arr(my_segment["end"], "code_length")
+            else:
+                my_segment["start"] = ds.domain_width * my_segment["start"] + \
+                  ds.domain_left_edge
+                my_segment["end"] = ds.domain_width * my_segment["end"] + \
+                  ds.domain_left_edge
+
+            if not ds.cosmological_simulation:
+                next_redshift = my_segment["redshift"]
+            elif self.near_redshift == self.far_redshift:
                 next_redshift = my_segment["redshift"] - \
                   self._deltaz_forward(my_segment["redshift"], 
                                        ds.domain_width[0].in_units("Mpccm / h") *
                                        my_segment["traversal_box_fraction"])
-            elif my_segment['next'] is None:
+            elif my_segment.get("next", None) is None:
                 next_redshift = self.near_redshift
             else:
                 next_redshift = my_segment['next']['redshift']
@@ -344,10 +438,10 @@ class LightRay(CosmologySplice):
                 if get_los_velocity:
                     line_of_sight = sub_segment[1] - sub_segment[0]
                     line_of_sight /= ((line_of_sight**2).sum())**0.5
-                    sub_vel = ds.arr([sub_ray['x-velocity'],
-                                      sub_ray['y-velocity'],
-                                      sub_ray['z-velocity']])
-                    sub_data['los_velocity'].extend((np.rollaxis(sub_vel, 1) *
+                    sub_vel = ds.arr([sub_ray['velocity_x'],
+                                      sub_ray['velocity_y'],
+                                      sub_ray['velocity_z']])
+                    sub_data['velocity_los'].extend((np.rollaxis(sub_vel, 1) *
                                                      line_of_sight).sum(axis=1)[asort])
                     del sub_vel
 
@@ -355,7 +449,6 @@ class LightRay(CosmologySplice):
                 del sub_ray, asort
 
             for key in sub_data:
-                if key in "xyz": continue
                 sub_data[key] = ds.arr(sub_data[key]).in_cgs()
 
             # Get redshift for each lixel.  Assume linear relation between l and z.
@@ -393,18 +486,32 @@ class LightRay(CosmologySplice):
 
     @parallel_root_only
     def _write_light_ray(self, filename, data):
-        "Write light ray data to hdf5 file."
+        """
+        _write_light_ray(filename, data)
+
+        Write light ray data to hdf5 file.
+        """
 
         mylog.info("Saving light ray data to %s." % filename)
         output = h5py.File(filename, 'w')
         for field in data.keys():
-            output.create_dataset(field, data=data[field])
-            output[field].attrs["units"] = str(data[field].units)
+            # if the field is a tuple, only use the second part of the tuple
+            # in the hdf5 output (i.e. ('gas', 'density') -> 'density')
+            if isinstance(field, tuple):
+                fieldname = field[1]
+            else:
+                fieldname = field
+            output.create_dataset(fieldname, data=data[field])
+            output[fieldname].attrs["units"] = str(data[field].units)
         output.close()
 
     @parallel_root_only
     def _write_light_ray_solution(self, filename, extra_info=None):
-        "Write light ray solution to a file."
+        """
+        _write_light_ray_solution(filename, extra_info=None)
+
+        Write light ray solution to a file.
+        """
 
         mylog.info("Writing light ray solution to %s." % filename)
         f = open(filename, 'w')
@@ -422,7 +529,11 @@ class LightRay(CosmologySplice):
         f.close()
 
 def _flatten_dict_list(data, exceptions=None):
-    "Flatten the list of dicts into one dict."
+    """
+    _flatten_dict_list(data, exceptions=None)
+
+    Flatten the list of dicts into one dict.
+    """
 
     if exceptions is None: exceptions = []
     new_data = {}
@@ -437,12 +548,20 @@ def _flatten_dict_list(data, exceptions=None):
     return new_data
 
 def vector_length(start, end):
-    "Calculate vector length."
+    """
+    vector_length(start, end)
+    
+    Calculate vector length.
+    """
 
     return np.sqrt(np.power((end - start), 2).sum())
 
 def periodic_distance(coord1, coord2):
-    "Calculate length of shortest vector between to points in periodic domain."
+    """
+    periodic_distance(coord1, coord2)
+
+    Calculate length of shortest vector between to points in periodic domain.
+    """
     dif = coord1 - coord2
 
     dim = np.ones(coord1.shape,dtype=int)
@@ -455,7 +574,21 @@ def periodic_distance(coord1, coord2):
     return np.sqrt((dif * dif).sum(axis=-1))
 
 def periodic_ray(start, end, left=None, right=None):
-    "Break up periodic ray into non-periodic segments."
+    """
+    periodic_ray(start, end, left=None, right=None)
+
+    Break up periodic ray into non-periodic segments. 
+    Accepts start and end points of periodic ray as YTArrays.
+    Accepts optional left and right edges of periodic volume as YTArrays.
+    Returns a list of lists of coordinates, where each element of the 
+    top-most list is a 2-list of start coords and end coords of the 
+    non-periodic ray: 
+
+    [[[x0start,y0start,z0start], [x0end, y0end, z0end]], 
+     [[x1start,y1start,z1start], [x1end, y1end, z1end]], 
+     ...,]
+
+    """
 
     if left is None:
         left = np.zeros(start.shape)
