@@ -14,20 +14,21 @@ API for level_sets
 #-----------------------------------------------------------------------------
 
 from .contour_finder import \
-    coalesce_join_tree, \
     identify_contours
 
 from .clump_handling import \
     Clump, \
     find_clumps, \
     get_lowest_clumps, \
-    write_clump_hierarchy, \
-    write_clumps, \
-    write_old_clump_hierarchy, \
-    write_old_clumps, \
-    write_old_clump_info, \
-    _DistanceToMainClump
+    write_clump_index, \
+    write_clumps
 
+from .clump_info_items import \
+    add_clump_info
+
+from .clump_validators import \
+    add_validator
+    
 from .clump_tools import \
     recursive_all_clumps, \
     return_all_clumps, \
