@@ -1,6 +1,7 @@
 """
 API for particle_trajectories
 """
+from __future__ import absolute_import
 #-----------------------------------------------------------------------------
 # Copyright (c) 2013, yt Development Team.
 #
@@ -9,4 +10,4 @@ API for particle_trajectories
 # The full license is in the file COPYING.txt, distributed with this software.
 #-----------------------------------------------------------------------------
 
-from particle_trajectories import ParticleTrajectories
+from .particle_trajectories import ParticleTrajectories
