@@ -22,18 +22,23 @@ http://adsabs.harvard.edu/abs/2013MNRAS.428.1395B
 # The full license is in the file COPYING.txt, distributed with this software.
 #-----------------------------------------------------------------------------
 
+from yt.extern.six import string_types
 import numpy as np
 from yt.funcs import *
-from yt.utilities.physical_constants import mp, kboltz
+from yt.utilities.physical_constants import mp
 from yt.utilities.parallel_tools.parallel_analysis_interface import \
-     communication_system, parallel_objects
+     parallel_objects
 from yt.units.yt_array import uconcatenate
 
 n_kT = 10000
 kT_min = 8.08e-2
 kT_max = 50.
 
-comm = communication_system.communicators[-1]
+photon_units = {"Energy":"keV",
+                "dx":"kpc"}
+for ax in "xyz":
+    photon_units[ax] = "kpc"
+    photon_units["v"+ax] = "km/s"
 
 class PhotonModel(object):
 
@@ -47,7 +52,7 @@ class PhotonModel(object):
 class ThermalPhotonModel(PhotonModel):
     r"""
     Initialize a ThermalPhotonModel from a thermal spectrum. 
-    
+
     Parameters
     ----------
 
@@ -62,34 +67,39 @@ class ThermalPhotonModel(PhotonModel):
     photons_per_chunk : integer
         The maximum number of photons that are allocated per chunk. Increase or decrease
         as needed.
+    method : string, optional
+        The method used to generate the photon energies from the spectrum:
+        "invert_cdf": Invert the cumulative distribution function of the spectrum.
+        "accept_reject": Acceptance-rejection method using the spectrum. 
+        The first method should be sufficient for most cases. 
     """
-    def __init__(self, spectral_model, X_H=0.75, Zmet=0.3, photons_per_chunk=10000000):
+    def __init__(self, spectral_model, X_H=0.75, Zmet=0.3, 
+                 photons_per_chunk=10000000, method="invert_cdf"):
         self.X_H = X_H
         self.Zmet = Zmet
         self.spectral_model = spectral_model
         self.photons_per_chunk = photons_per_chunk
+        self.method = method
 
     def __call__(self, data_source, parameters):
-        
+
         ds = data_source.ds
 
         exp_time = parameters["FiducialExposureTime"]
         area = parameters["FiducialArea"]
         redshift = parameters["FiducialRedshift"]
         D_A = parameters["FiducialAngularDiameterDistance"].in_cgs()
-        dist_fac = 1.0/(4.*np.pi*D_A.value*D_A.value*(1.+redshift)**3)
+        dist_fac = 1.0/(4.*np.pi*D_A.value*D_A.value*(1.+redshift)**2)
         src_ctr = parameters["center"]
 
-        vol_scale = 1.0/np.prod(ds.domain_width.in_cgs().to_ndarray())
-
         my_kT_min, my_kT_max = data_source.quantities.extrema("kT")
 
-        self.spectral_model.prepare()
-        energy = self.spectral_model.ebins
+        self.spectral_model.prepare_spectrum(redshift)
+        emid = self.spectral_model.emid
+        ebins = self.spectral_model.ebins
+        nchan = len(emid)
 
-        citer = data_source.chunks(["kT","cell_volume","density",
-                                    "x","y","z","dx","velocity_x",
-                                    "velocity_y","velocity_z"], "io")
+        citer = data_source.chunks([], "io")
 
         photons = {}
         photons["x"] = []
@@ -102,7 +112,13 @@ class ThermalPhotonModel(PhotonModel):
         photons["Energy"] = []
         photons["NumberOfPhotons"] = []
 
-        spectral_norm = area.v*exp_time.v*dist_fac/vol_scale
+        spectral_norm = area.v*exp_time.v*dist_fac
+
+        tot_num_cells = data_source.ires.shape[0]
+
+        pbar = get_pbar("Generating photons ", tot_num_cells)
+
+        cell_counter = 0
 
         for chunk in parallel_objects(citer):
 
@@ -114,14 +130,14 @@ class ThermalPhotonModel(PhotonModel):
             EM = (chunk["density"]/mp).v**2
             EM *= 0.5*(1.+self.X_H)*self.X_H*vol
 
-            if isinstance(self.Zmet, basestring):
+            if isinstance(self.Zmet, string_types):
                 metalZ = chunk[self.Zmet].v
             else:
-                metalZ = self.Zmet
+                metalZ = self.Zmet*np.ones(num_cells)
 
             idxs = np.argsort(kT)
 
-            kT_bins = np.linspace(kT_min, max(my_kT_max, kT_max), num=n_kT+1)
+            kT_bins = np.linspace(kT_min, max(my_kT_max.v, kT_max), num=n_kT+1)
             dkT = kT_bins[1]-kT_bins[0]
             kT_idxs = np.digitize(kT[idxs], kT_bins)
             kT_idxs = np.minimum(np.maximum(1, kT_idxs), n_kT) - 1
@@ -136,7 +152,7 @@ class ThermalPhotonModel(PhotonModel):
                 n += bcount
             kT_idxs = np.unique(kT_idxs)
 
-            cell_em = EM[idxs]*vol_scale
+            cell_em = EM[idxs]*spectral_norm
 
             number_of_photons = np.zeros(num_cells, dtype="uint64")
             energies = np.zeros(self.photons_per_chunk)
@@ -144,8 +160,6 @@ class ThermalPhotonModel(PhotonModel):
             start_e = 0
             end_e = 0
 
-            pbar = get_pbar("Generating photons for chunk ", num_cells)
-
             for ibegin, iend, ikT in zip(bcell, ecell, kT_idxs):
 
                 kT = kT_bins[ikT] + 0.5*dkT
@@ -154,58 +168,57 @@ class ThermalPhotonModel(PhotonModel):
 
                 cem = cell_em[ibegin:iend]
 
-                em_sum_c = cem.sum()
-                if isinstance(self.Zmet, basestring):
-                    em_sum_m = (metalZ[ibegin:iend]*cem).sum()
-                else:
-                    em_sum_m = metalZ*em_sum_c
-
                 cspec, mspec = self.spectral_model.get_spectrum(kT)
 
-                cumspec_c = np.cumsum(cspec.d)
-                tot_ph_c = cumspec_c[-1]*spectral_norm*em_sum_c
-                cumspec_c /= cumspec_c[-1]
-                cumspec_c = np.insert(cumspec_c, 0, 0.0)
-
-                cumspec_m = np.cumsum(mspec.d)
-                tot_ph_m = cumspec_m[-1]*spectral_norm*em_sum_m
-                cumspec_m /= cumspec_m[-1]
-                cumspec_m = np.insert(cumspec_m, 0, 0.0)
+                tot_ph_c = cspec.d.sum()
+                tot_ph_m = mspec.d.sum()
 
                 u = np.random.random(size=n_current)
 
-                cell_norm_c = tot_ph_c*cem/em_sum_c
-                cell_n_c = np.uint64(cell_norm_c) + np.uint64(np.modf(cell_norm_c)[0] >= u)
-            
-                if isinstance(self.Zmet, basestring):
-                    cell_norm_m = tot_ph_m*metalZ[ibegin:iend]*cem/em_sum_m
-                else:
-                    cell_norm_m = tot_ph_m*metalZ*cem/em_sum_m
-                cell_n_m = np.uint64(cell_norm_m) + np.uint64(np.modf(cell_norm_m)[0] >= u)
-            
-                number_of_photons[ibegin:iend] = cell_n_c + cell_n_m
+                cell_norm_c = tot_ph_c*cem
+                cell_norm_m = tot_ph_m*metalZ[ibegin:iend]*cem
+                cell_norm = np.modf(cell_norm_c + cell_norm_m)
+                cell_n = np.uint64(cell_norm[1]) + np.uint64(cell_norm[0] >= u)
 
-                end_e += int((cell_n_c+cell_n_m).sum())
+                number_of_photons[ibegin:iend] = cell_n
+
+                end_e += int(cell_n.sum())
 
                 if end_e > self.photons_per_chunk:
                     raise RuntimeError("Number of photons generated for this chunk "+
                                        "exceeds photons_per_chunk (%d)! " % self.photons_per_chunk +
                                        "Increase photons_per_chunk!")
 
-                energies[start_e:end_e] = _generate_energies(cell_n_c, cell_n_m, cumspec_c, cumspec_m, energy)
-            
+                if self.method == "invert_cdf":
+                    cumspec_c = np.cumsum(cspec.d)
+                    cumspec_m = np.cumsum(mspec.d)
+                    cumspec_c = np.insert(cumspec_c, 0, 0.0)
+                    cumspec_m = np.insert(cumspec_m, 0, 0.0)
+
+                ei = start_e
+                for cn, Z in zip(number_of_photons[ibegin:iend], metalZ[ibegin:iend]):
+                    if cn == 0: continue
+                    if self.method == "invert_cdf":
+                        cumspec = cumspec_c + Z*cumspec_m
+                        cumspec /= cumspec[-1]
+                        randvec = np.random.uniform(size=cn)
+                        randvec.sort()
+                        cell_e = np.interp(randvec, cumspec, ebins)
+                    elif self.method == "accept_reject":
+                        tot_spec = cspec.d+Z*mspec.d
+                        tot_spec /= tot_spec.sum()
+                        eidxs = np.random.choice(nchan, size=cn, p=tot_spec)
+                        cell_e = emid[eidxs]
+                    energies[ei:ei+cn] = cell_e
+                    cell_counter += 1
+                    pbar.update(cell_counter)
+                    ei += cn
+
                 start_e = end_e
 
-                pbar.update(iend)
-
-            pbar.finish()
-
             active_cells = number_of_photons > 0
             idxs = idxs[active_cells]
 
-            mylog.info("Number of photons generated for this chunk: %d" % int(number_of_photons.sum()))
-            mylog.info("Number of cells with photons: %d" % int(active_cells.sum()))
-
             photons["NumberOfPhotons"].append(number_of_photons[active_cells])
             photons["Energy"].append(ds.arr(energies[:end_e].copy(), "keV"))
             photons["x"].append((chunk["x"][idxs]-src_ctr[0]).in_units("kpc"))
@@ -216,23 +229,17 @@ class ThermalPhotonModel(PhotonModel):
             photons["vz"].append(chunk["velocity_z"][idxs].in_units("km/s"))
             photons["dx"].append(chunk["dx"][idxs].in_units("kpc"))
 
+        pbar.finish()
+
         for key in photons:
             if len(photons[key]) > 0:
                 photons[key] = uconcatenate(photons[key])
+            elif key == "NumberOfPhotons":
+                photons[key] = np.array([])
+            else:
+                photons[key] = YTArray([], photon_units[key])
+
+        mylog.info("Number of photons generated: %d" % int(np.sum(photons["NumberOfPhotons"])))
+        mylog.info("Number of cells with photons: %d" % len(photons["x"]))
 
         return photons
-
-def _generate_energies(cell_n_c, cell_n_m, counts_c, counts_m, energy):
-    energies = np.array([])
-    for cn_c, cn_m in zip(cell_n_c, cell_n_m):
-        if cn_c > 0:
-            randvec_c = np.random.uniform(size=cn_c)
-            randvec_c.sort()
-            cell_e_c = np.interp(randvec_c, counts_c, energy)
-            energies = np.append(energies, cell_e_c)
-        if cn_m > 0: 
-            randvec_m = np.random.uniform(size=cn_m)
-            randvec_m.sort()
-            cell_e_m = np.interp(randvec_m, counts_m, energy)
-            energies = np.append(energies, cell_e_m)
-    return energies
