@@ -32,17 +32,17 @@ class SpectralModel(object):
         self.ebins = np.linspace(self.emin, self.emax, nchan+1)
         self.de = np.diff(self.ebins)
         self.emid = 0.5*(self.ebins[1:]+self.ebins[:-1])
-        
-    def prepare(self):
+
+    def prepare_spectrum(self):
         pass
-    
+
     def get_spectrum(self):
         pass
-                                                        
+
 class XSpecThermalModel(SpectralModel):
     r"""
     Initialize a thermal gas emission model from PyXspec.
-    
+
     Parameters
     ----------
 
@@ -54,19 +54,25 @@ class XSpecThermalModel(SpectralModel):
         The maximum energy for the spectral model.
     nchan : integer
         The number of channels in the spectral model.
+    settings : dictionary, optional
+        A dictionary of key, value pairs (must both be strings)
+        that can be used to set various options in XSPEC.
 
     Examples
     --------
     >>> mekal_model = XSpecThermalModel("mekal", 0.05, 50.0, 1000)
     """
-    def __init__(self, model_name, emin, emax, nchan, thermal_broad=False):
+    def __init__(self, model_name, emin, emax, nchan, 
+                 thermal_broad=False, settings=None):
         self.model_name = model_name
         self.thermal_broad = thermal_broad
-        SpectralModel.__init__(self, emin, emax, nchan)
-        
-    def prepare(self):
+        if settings is None: settings = {}
+        self.settings = settings
+        super(XSpecThermalModel, self).__init__(emin, emax, nchan)
+
+    def prepare_spectrum(self, zobs):
         """
-        Prepare the thermal model for execution.
+        Prepare the thermal model for execution given a redshift *zobs* for the spectrum.
         """
         import xspec
         xspec.Xset.chatter = 0
@@ -79,9 +85,11 @@ class XSpecThermalModel(SpectralModel):
         else:
             self.norm = 1.0e-14
         self.thermal_comp.norm = 1.0
-        self.thermal_comp.Redshift = 0.0
+        self.thermal_comp.Redshift = zobs
         if self.thermal_broad:
             xspec.Xset.addModelString("APECTHERMAL","yes")
+        for k,v in self.settings.items():
+            xspec.Xset.addModelString(k,v)
 
     def get_spectrum(self, kT):
         """
@@ -89,18 +97,20 @@ class XSpecThermalModel(SpectralModel):
         """
         self.thermal_comp.kT = kT
         self.thermal_comp.Abundanc = 0.0
-        cosmic_spec = self.norm*np.array(self.model.values(0))
+        cosmic_spec = np.array(self.model.values(0))
         if self.model_name == "bremss":
             metal_spec = np.zeros(self.nchan)
         else:
             self.thermal_comp.Abundanc = 1.0
-            metal_spec = self.norm*np.array(self.model.values(0)) - cosmic_spec
+            metal_spec = np.array(self.model.values(0)) - cosmic_spec
+        cosmic_spec *= self.norm
+        metal_spec *= self.norm
         return YTArray(cosmic_spec, "cm**3/s"), YTArray(metal_spec, "cm**3/s")
-        
+
 class XSpecAbsorbModel(SpectralModel):
     r"""
     Initialize an absorption model from PyXspec.
-    
+
     Parameters
     ----------
 
@@ -114,19 +124,25 @@ class XSpecAbsorbModel(SpectralModel):
         The maximum energy for the spectral model.
     nchan : integer, optional
         The number of channels in the spectral model.
+    settings : dictionary, optional
+        A dictionary of key, value pairs (must both be strings)
+        that can be used to set various options in XSPEC.
 
     Examples
     --------
     >>> abs_model = XSpecAbsorbModel("wabs", 0.1)
     """
-    def __init__(self, model_name, nH, emin=0.01, emax=50.0, nchan=100000):
+    def __init__(self, model_name, nH, emin=0.01, emax=50.0, 
+                 nchan=100000, settings=None):
         self.model_name = model_name
         self.nH = nH
-        SpectralModel.__init__(self, emin, emax, nchan)
-        
-    def prepare(self):
+        if settings is None: settings = {}
+        self.settings = settings
+        super(XSpecAbsorbModel, self).__init__(emin, emax, nchan)
+
+    def prepare_spectrum(self):
         """
-        Prepare the absorption model for execution.
+        Prepare the absorption model for execution given a redshift *zobs* for the spectrum.
         """
         import xspec
         xspec.Xset.chatter = 0
@@ -135,6 +151,8 @@ class XSpecAbsorbModel(SpectralModel):
         self.model = xspec.Model(self.model_name+"*powerlaw")
         self.model.powerlaw.norm = self.nchan/(self.emax.value-self.emin.value)
         self.model.powerlaw.PhoIndex = 0.0
+        for k,v in self.settings.items():
+            xspec.Xset.addModelString(k,v)
 
     def get_spectrum(self):
         """
@@ -150,7 +168,7 @@ class TableApecModel(SpectralModel):
     available at http://www.atomdb.org. This code borrows heavily from Python
     routines used to read the APEC tables developed by Adam Foster at the
     CfA (afoster@cfa.harvard.edu). 
-    
+
     Parameters
     ----------
 
@@ -183,7 +201,7 @@ class TableApecModel(SpectralModel):
                                      self.apec_prefix+"_coco.fits")
         self.linefile = os.path.join(self.apec_root,
                                      self.apec_prefix+"_line.fits")
-        SpectralModel.__init__(self, emin, emax, nchan)
+        super(TableApecModel, self).__init__(emin, emax, nchan)  
         self.wvbins = hc/self.ebins[::-1].d
         # H, He, and trace elements
         self.cosmic_elem = [1,2,3,4,5,9,11,15,17,19,21,22,23,24,25,27,29,30]
@@ -196,8 +214,8 @@ class TableApecModel(SpectralModel):
                            32.0650,35.4530,39.9480,39.0983,40.0780,
                            44.9559,47.8670,50.9415,51.9961,54.9380,
                            55.8450,58.9332,58.6934,63.5460,65.3800])
-        
-    def prepare(self):
+
+    def prepare_spectrum(self, zobs):
         """
         Prepare the thermal model for execution.
         """
@@ -216,17 +234,18 @@ class TableApecModel(SpectralModel):
         self.dTvals = np.diff(self.Tvals)
         self.minlam = self.wvbins.min()
         self.maxlam = self.wvbins.max()
-    
+        self.scale_factor = 1.0/(1.+zobs)
+
     def _make_spectrum(self, element, tindex):
-        
+
         tmpspec = np.zeros(self.nchan)
-        
+
         i = np.where((self.line_handle[tindex].data.field('element') == element) &
                      (self.line_handle[tindex].data.field('lambda') > self.minlam) &
                      (self.line_handle[tindex].data.field('lambda') < self.maxlam))[0]
 
         vec = np.zeros(self.nchan)
-        E0 = hc/self.line_handle[tindex].data.field('lambda')[i]
+        E0 = hc/self.line_handle[tindex].data.field('lambda')[i]*self.scale_factor
         amp = self.line_handle[tindex].data.field('epsilon')[i]
         ebins = self.ebins.d
         if self.thermal_broad:
@@ -246,19 +265,19 @@ class TableApecModel(SpectralModel):
             return tmpspec
         else:
             ind = ind[0]
-                                                    
+
         n_cont = self.coco_handle[tindex].data.field('N_Cont')[ind]
-        e_cont = self.coco_handle[tindex].data.field('E_Cont')[ind][:n_cont]
+        e_cont = self.coco_handle[tindex].data.field('E_Cont')[ind][:n_cont]*self.scale_factor
         continuum = self.coco_handle[tindex].data.field('Continuum')[ind][:n_cont]
 
         tmpspec += np.interp(self.emid.d, e_cont, continuum)*self.de.d
-        
+
         n_pseudo = self.coco_handle[tindex].data.field('N_Pseudo')[ind]
-        e_pseudo = self.coco_handle[tindex].data.field('E_Pseudo')[ind][:n_pseudo]
+        e_pseudo = self.coco_handle[tindex].data.field('E_Pseudo')[ind][:n_pseudo]*self.scale_factor
         pseudo = self.coco_handle[tindex].data.field('Pseudo')[ind][:n_pseudo]
-        
+
         tmpspec += np.interp(self.emid.d, e_pseudo, pseudo)*self.de.d
-        
+
         return tmpspec
 
     def get_spectrum(self, kT):
@@ -288,7 +307,7 @@ class TableApecModel(SpectralModel):
 class TableAbsorbModel(SpectralModel):
     r"""
     Initialize an absorption model from a table stored in an HDF5 file.
-    
+
     Parameters
     ----------
 
@@ -299,7 +318,7 @@ class TableAbsorbModel(SpectralModel):
 
     Examples
     --------
-    >>> abs_model = XSpecAbsorbModel("wabs", 0.1)
+    >>> abs_model = XSpecAbsorbModel("abs_table.h5", 0.1)
     """
 
     def __init__(self, filename, nH):
@@ -312,15 +331,15 @@ class TableAbsorbModel(SpectralModel):
         self.sigma = YTArray(f["cross_section"][:], "cm**2")
         nchan = self.sigma.shape[0]
         f.close()
-        SpectralModel.__init__(self, emin, emax, nchan)
+        super(TableAbsorbModel, self).__init__(emin, emax, nchan)
         self.nH = YTQuantity(nH*1.0e22, "cm**-2")
-        
-    def prepare(self):
+
+    def prepare_spectrum(self):
         """
         Prepare the absorption model for execution.
         """
         pass
-        
+
     def get_spectrum(self):
         """
         Get the absorption spectrum.
