@@ -1,11 +1,12 @@
 #!/usr/bin/env python
-
+import sys
 def configuration(parent_package='', top_path=None):
     from numpy.distutils.misc_util import Configuration
     config = Configuration('analysis_modules', parent_package, top_path)
     config.make_config_py()  # installs __config__.py
     config.add_subpackage("absorption_spectrum")
     config.add_subpackage("cosmological_observation")
+    config.add_subpackage("halo_analysis")
     config.add_subpackage("halo_finding")
     config.add_subpackage("halo_mass_function")
     config.add_subpackage("level_sets")
