@@ -16,6 +16,7 @@ Various non-grid data containers.
 import itertools
 import types
 import uuid
+from yt.extern.six import string_types
 
 data_object_registry = {}
 
@@ -27,6 +28,12 @@ from contextlib import contextmanager
 from yt.funcs import *
 
 from yt.data_objects.particle_io import particle_handler_registry
+from yt.units.unit_object import UnitParseError
+from yt.utilities.exceptions import \
+    YTUnitConversionError, \
+    YTFieldUnitError, \
+    YTFieldUnitParseError, \
+    YTSpatialFieldUnitError
 from yt.utilities.lib.marching_cubes import \
     march_cubes_grid, march_cubes_grid_flux
 from yt.utilities.parallel_tools.parallel_analysis_interface import \
@@ -38,10 +45,10 @@ from yt.utilities.amr_kdtree.api import \
 from .derived_quantities import DerivedQuantityCollection
 from yt.fields.field_exceptions import \
     NeedsGridType
-from yt.fields.derived_field import \
-    ValidateSpatial
 import yt.geometry.selection_routines
-from yt.extern.six import add_metaclass
+from yt.geometry.selection_routines import \
+    compose_selector
+from yt.extern.six import add_metaclass, string_types
 
 def force_array(item, shape):
     try:
@@ -101,8 +108,15 @@ class YTDataContainer(object):
         sets its initial set of fields, and the remainder of the arguments
         are passed as field_parameters.
         """
-        if ds != None:
+        # ds is typically set in the new object type created in Dataset._add_object_class
+        # but it can also be passed as a parameter to the constructor, in which case it will
+        # override the default. This code ensures it is never not set.
+        if ds is not None:
             self.ds = ds
+        else:
+            if not hasattr(self, "ds"):
+                raise RuntimeError("Error: ds must be set either through class type or parameter to the constructor")
+
         self._current_particle_type = "all"
         self._current_fluid_type = self.ds.default_fluid_type
         self.ds.objects.append(weakref.proxy(self))
@@ -155,18 +169,17 @@ class YTDataContainer(object):
     def _set_center(self, center):
         if center is None:
             self.center = None
-            self.set_field_parameter('center', self.center)
             return
         elif isinstance(center, YTArray):
             self.center = self.ds.arr(center.in_cgs())
             self.center.convert_to_units('code_length')
-        elif isinstance(center, (types.ListType, types.TupleType, np.ndarray)):
+        elif isinstance(center, (list, tuple, np.ndarray)):
             if isinstance(center[0], YTQuantity):
                 self.center = self.ds.arr([c.in_cgs() for c in center])
                 self.center.convert_to_units('code_length')
             else:
                 self.center = self.ds.arr(center, 'code_length')
-        elif isinstance(center, basestring):
+        elif isinstance(center, string_types):
             if center.lower() in ("c", "center"):
                 self.center = self.ds.domain_center
              # is this dangerous for race conditions?
@@ -183,7 +196,7 @@ class YTDataContainer(object):
         This is typically only used by derived field functions, but
         it returns parameters used to generate fields.
         """
-        if self.field_parameters.has_key(name):
+        if name in self.field_parameters:
             return self.field_parameters[name]
         else:
             return default
@@ -199,7 +212,7 @@ class YTDataContainer(object):
         """
         Checks if a field parameter is set.
         """
-        return self.field_parameters.has_key(name)
+        return name in self.field_parameters
 
     def convert(self, datatype):
         """
@@ -218,7 +231,7 @@ class YTDataContainer(object):
         """
         Checks if a data field already exists.
         """
-        return self.field_data.has_key(key)
+        return key in self.field_data
 
     def keys(self):
         return self.field_data.keys()
@@ -244,9 +257,9 @@ class YTDataContainer(object):
         # when there are, for example, no elements in the object.
         rv = self.field_data.get(f, None)
         if rv is None:
-            if isinstance(f, types.TupleType):
+            if isinstance(f, tuple):
                 fi = self.ds._get_field_info(*f)
-            elif isinstance(f, types.StringType):
+            elif isinstance(f, bytes):
                 fi = self.ds._get_field_info("unknown", f)
             rv = self.ds.arr(self.field_data[key], fi.units)
         return rv
@@ -298,7 +311,11 @@ class YTDataContainer(object):
         return rv
 
     def _generate_spatial_fluid(self, field, ngz):
-        rv = np.empty(self.ires.size, dtype="float64")
+        finfo = self.ds._get_field_info(*field)
+        if finfo.units is None:
+            raise YTSpatialFieldUnitError(field)
+        units = finfo.units
+        rv = self.ds.arr(np.empty(self.ires.size, dtype="float64"), units)
         ind = 0
         if ngz == 0:
             deps = self._identify_dependencies([field], spatial = True)
@@ -336,7 +353,7 @@ class YTDataContainer(object):
             if ngt_exception.ghost_zones != 0:
                 raise NotImplementedError
             size = self._count_particles(ftype)
-            rv = np.empty(size, dtype="float64")
+            rv = self.ds.arr(np.empty(size, dtype="float64"), finfo.units)
             ind = 0
             for io_chunk in self.chunks([], "io", cache = False):
                 for i, chunk in enumerate(self.chunks(field, "spatial")):
@@ -419,7 +436,7 @@ class YTDataContainer(object):
         """
         from glue.core import DataCollection, Data
         from glue.qt.glue_application import GlueApplication
-        
+
         gdata = Data(label=label)
         for component_name in fields:
             gdata.add_component(self[component_name], component_name)
@@ -430,7 +447,16 @@ class YTDataContainer(object):
             app.start()
         else:
             data_collection.append(gdata)
-        
+
+    @property
+    def _hash(self):
+        s = "%s" % self
+        try:
+            import hashlib
+            return hashlib.md5(s.encode('utf-8')).hexdigest()
+        except ImportError:
+            return s
+
     def __reduce__(self):
         args = tuple([self.ds._hash(), self._type_name] +
                      [getattr(self, n) for n in self._con_args] +
@@ -440,8 +466,11 @@ class YTDataContainer(object):
     def __repr__(self):
         # We'll do this the slow way to be clear what's going on
         s = "%s (%s): " % (self.__class__.__name__, self.ds)
-        s += ", ".join(["%s=%s" % (i, getattr(self,i))
-                       for i in self._con_args])
+        for i in self._con_args:
+            try:
+                s += ", %s=%s" % (i, getattr(self, i).in_cgs())
+            except AttributeError:
+                s += ", %s=%s" % (i, getattr(self, i))
         return s
 
     @contextmanager
@@ -477,10 +506,10 @@ class YTDataContainer(object):
             if field in self._container_fields:
                 explicit_fields.append(field)
                 continue
-            if isinstance(field, types.TupleType):
+            if isinstance(field, tuple):
                 if len(field) != 2 or \
-                   not isinstance(field[0], types.StringTypes) or \
-                   not isinstance(field[1], types.StringTypes):
+                   not isinstance(field[0], string_types) or \
+                   not isinstance(field[1], string_types):
                     raise YTFieldNotParseable(field)
                 ftype, fname = field
                 finfo = self.ds._get_field_info(ftype, fname)
@@ -542,9 +571,22 @@ class YTSelectionContainer(YTDataContainer, ParallelAnalysisInterface):
     _sort_by = None
     _selector = None
     _current_chunk = None
+    _data_source = None
+    _dimensionality = None
 
-    def __init__(self, *args, **kwargs):
-        super(YTSelectionContainer, self).__init__(*args, **kwargs)
+    def __init__(self, ds, field_parameters, data_source=None):
+        ParallelAnalysisInterface.__init__(self)
+        super(YTSelectionContainer, self).__init__(ds, field_parameters)
+        self._data_source = data_source
+        if data_source is not None:
+            if data_source.ds is not self.ds:
+                raise RuntimeError("Attempted to construct a DataContainer with a data_source "
+                                   "from a different DataSet", ds, data_source.ds)
+            if data_source._dimensionality < self._dimensionality:
+                raise RuntimeError("Attempted to construct a DataContainer with a data_source "
+                                   "of lower dimensionality (%u vs %u)" %
+                                    (data_source._dimensionality, self._dimensionality))
+            self.field_parameters.update(data_source.field_parameters)
 
     @property
     def selector(self):
@@ -555,7 +597,11 @@ class YTSelectionContainer(YTDataContainer, ParallelAnalysisInterface):
                          "%s_selector" % self._type_name, None)
         if sclass is None:
             raise YTDataSelectorNotImplemented(self._type_name)
-        self._selector = sclass(self)
+
+        if self._data_source is not None:
+            self._selector = compose_selector(self, self._data_source.selector, sclass(self))
+        else:
+            self._selector = sclass(self)
         return self._selector
 
     def chunks(self, fields, chunking_style, **kwargs):
@@ -575,11 +621,6 @@ class YTSelectionContainer(YTDataContainer, ParallelAnalysisInterface):
             if inspected >= len(fields_to_get): break
             inspected += 1
             fi = self.ds._get_field_info(*field)
-            if not spatial and any(
-                    isinstance(v, ValidateSpatial) for v in fi.validators):
-                # We don't want to pre-fetch anything that's spatial, as that
-                # will be done later.
-                continue
             fd = self.ds.field_dependencies.get(field, None) or \
                  self.ds.field_dependencies.get(field[1], None)
             # This is long overdue.  Any time we *can't* find a field
@@ -623,7 +664,7 @@ class YTSelectionContainer(YTDataContainer, ParallelAnalysisInterface):
         # need to be used in spatial fields later on.
         fields_to_get = []
         # This will be pre-populated with spatial fields
-        fields_to_generate = [] 
+        fields_to_generate = []
         for field in self._determine_fields(fields):
             if field in self.field_data: continue
             finfo = self.ds._get_field_info(*field)
@@ -638,7 +679,7 @@ class YTSelectionContainer(YTDataContainer, ParallelAnalysisInterface):
         elif self._locked == True:
             raise GenerationInProgress(fields)
         # Track which ones we want in the end
-        ofields = set(self.field_data.keys()
+        ofields = set(list(self.field_data.keys())
                     + fields_to_get
                     + fields_to_generate)
         # At this point, we want to figure out *all* our dependencies.
@@ -671,7 +712,7 @@ class YTSelectionContainer(YTDataContainer, ParallelAnalysisInterface):
 
         fields_to_generate += gen_fluids + gen_particles
         self._generate_fields(fields_to_generate)
-        for field in self.field_data.keys():
+        for field in list(self.field_data.keys()):
             if field not in ofields:
                 self.field_data.pop(field)
 
@@ -691,12 +732,34 @@ class YTSelectionContainer(YTDataContainer, ParallelAnalysisInterface):
                 fi = self.ds._get_field_info(*field)
                 try:
                     fd = self._generate_field(field)
-                    if type(fd) == np.ndarray:
-                        fd = self.ds.arr(fd, fi.units)
                     if fd is None:
                         raise RuntimeError
+                    if fi.units is None:
+                        # first time calling a field with units='auto', so we
+                        # infer the units from the units of the data we get back
+                        # from the field function and use these units for future
+                        # field accesses
+                        units = str(getattr(fd, 'units', ''))
+                        fi.units = units
+                        self.field_data[field] = self.ds.arr(fd, units)
+                        msg = ("Field %s was added without specifying units, "
+                               "assuming units are %s")
+                        mylog.warn(msg % (fi.name, units))
+                        continue
+                    try:
+                        fd.convert_to_units(fi.units)
+                    except AttributeError:
+                        # If the field returns an ndarray, coerce to a
+                        # dimensionless YTArray and verify that field is
+                        # supposed to be unitless
+                        fd = self.ds.arr(fd, '')
+                        if fi.units != '':
+                            raise YTFieldUnitError(fi, fd.units)
+                    except YTUnitConversionError:
+                        raise YTFieldUnitError(fi, fd.units)
+                    except UnitParseError:
+                        raise YTFieldUnitParseError(fi)
                     self.field_data[field] = fd
-                    fd.convert_to_units(fi.units)
                 except GenerationInProgress as gip:
                     for f in gip.fields:
                         if f not in fields_to_generate:
@@ -765,30 +828,32 @@ class YTSelectionContainer(YTDataContainer, ParallelAnalysisInterface):
 
 class YTSelectionContainer0D(YTSelectionContainer):
     _spatial = False
-    def __init__(self, ds, field_parameters):
+    _dimensionality = 0
+    def __init__(self, ds, field_parameters = None, data_source = None):
         super(YTSelectionContainer0D, self).__init__(
-            ds, field_parameters)
+            ds, field_parameters, data_source)
 
 class YTSelectionContainer1D(YTSelectionContainer):
     _spatial = False
-    def __init__(self, ds, field_parameters):
+    _dimensionality = 1
+    def __init__(self, ds, field_parameters = None, data_source = None):
         super(YTSelectionContainer1D, self).__init__(
-            ds, field_parameters)
+            ds, field_parameters, data_source)
         self._grids = None
         self._sortkey = None
         self._sorted = {}
 
 class YTSelectionContainer2D(YTSelectionContainer):
     _key_fields = ['px','py','pdx','pdy']
+    _dimensionality = 2
     """
     Prepares the YTSelectionContainer2D, normal to *axis*.  If *axis* is 4, we are not
     aligned with any axis.
     """
     _spatial = False
-    def __init__(self, axis, ds, field_parameters):
-        ParallelAnalysisInterface.__init__(self)
+    def __init__(self, axis, ds, field_parameters = None, data_source = None):
         super(YTSelectionContainer2D, self).__init__(
-            ds, field_parameters)
+            ds, field_parameters, data_source)
         # We need the ds, which will exist by now, for fix_axis.
         self.axis = fix_axis(axis, self.ds)
         self.set_field_parameter("axis", axis)
@@ -807,6 +872,8 @@ class YTSelectionContainer2D(YTSelectionContainer):
         self.fields = [k for k in self.field_data if k not in skip]
         if fields is not None:
             self.fields = ensure_list(fields) + self.fields
+        if len(self.fields) == 0:
+            raise ValueError("No fields found to plot in get_pw")
         (bounds, center, display_center) = \
             get_window_parameters(axis, center, width, self.ds)
         pw = PWViewerMPL(self, bounds, fields=self.fields, origin=origin,
@@ -874,7 +941,7 @@ class YTSelectionContainer2D(YTSelectionContainer):
             return frb
 
         if center is None:
-            center = self.get_field_parameter("center")
+            center = self.center
             if center is None:
                 center = (self.ds.domain_right_edge
                         + self.ds.domain_left_edge)/2.0
@@ -883,6 +950,8 @@ class YTSelectionContainer2D(YTSelectionContainer):
         if iterable(width):
             w, u = width
             width = self.ds.quan(w, input_units = u)
+        elif not isinstance(width, YTArray):
+            width = self.ds.quan(width, 'code_length')
         if height is None:
             height = width
         elif iterable(height):
@@ -908,33 +977,45 @@ class YTSelectionContainer3D(YTSelectionContainer):
     _key_fields = ['x','y','z','dx','dy','dz']
     _spatial = False
     _num_ghost_zones = 0
-    def __init__(self, center, ds = None, field_parameters = None):
-        ParallelAnalysisInterface.__init__(self)
-        super(YTSelectionContainer3D, self).__init__(ds, field_parameters)
+    _dimensionality = 3
+    def __init__(self, center, ds, field_parameters = None, data_source = None):
+        super(YTSelectionContainer3D, self).__init__(ds, field_parameters, data_source)
         self._set_center(center)
         self.coords = None
         self._grids = None
         self.quantities = DerivedQuantityCollection(self)
 
-    def cut_region(self, field_cuts, field_parameters = None):
+    def cut_region(self, field_cuts, field_parameters=None):
         """
-        Return an InLineExtractedRegion, where the object cells are cut on the
-        fly with a set of field_cuts.  It is very useful for applying
-        conditions to the fields in your data object.  Note that in previous
-        versions of yt, this accepted 'grid' as a variable, but presently it
-        requires 'obj'.
-        
+        Return an YTCutRegionBase, where the a cell is identified as being inside
+        the cut region based on the value of one or more fields.  Note that in
+        previous versions of yt the name 'grid' was used to represent the data
+        object used to construct the field cut, as of yt 3.0, this has been
+        changed to 'obj'.
+
+        Parameters
+        ----------
+        field_cuts : list of strings
+           A list of conditionals that will be evaluated. In the namespace
+           available, these conditionals will have access to 'obj' which is a
+           data object of unknown shape, and they must generate a boolean array.
+           For instance, conditionals = ["obj['temperature'] < 1e3"]
+        field_parameters : dictionary
+           A dictionary of field parameters to be used when applying the field
+           cuts.
+
         Examples
         --------
-        To find the total mass of gas above 10^6 K in your volume:
+        To find the total mass of hot gas with temperature greater than 10^6 K
+        in your volume:
 
-        >>> ds = load("RedshiftOutput0005")
+        >>> ds = yt.load("RedshiftOutput0005")
         >>> ad = ds.all_data()
-        >>> cr = ad.cut_region(["obj['Temperature'] > 1e6"])
-        >>> print cr.quantities["TotalQuantity"]("CellMassMsun")
+        >>> cr = ad.cut_region(["obj['temperature'] > 1e6"])
+        >>> print cr.quantities.total_quantity("cell_mass").in_units('Msun')
         """
         cr = self.ds.cut_region(self, field_cuts,
-                                  field_parameters = field_parameters)
+                                field_parameters=field_parameters)
         return cr
 
     def extract_isocontours(self, field, value, filename = None,
@@ -944,9 +1025,10 @@ class YTSelectionContainer3D(YTSelectionContainer):
         Triangles in that isocontour.
 
         This function simply returns the vertices of all the triangles
-        calculated by the marching cubes algorithm; for more complex
-        operations, such as identifying connected sets of cells above a given
-        threshold, see the extract_connected_sets function.  This is more
+        calculated by the `marching cubes
+        <http://en.wikipedia.org/wiki/Marching_cubes>`_ algorithm; for more
+        complex operations, such as identifying connected sets of cells above a
+        given threshold, see the extract_connected_sets function.  This is more
         useful for calculating, for instance, total isocontour area, or
         visualizing in an external program (such as `MeshLab
         <http://meshlab.sf.net>`_.)
@@ -977,11 +1059,6 @@ class YTSelectionContainer3D(YTSelectionContainer):
             contain the values of the field specified at the center of each
             triangle.
 
-        References
-        ----------
-
-        .. [1] Marching Cubes: http://en.wikipedia.org/wiki/Marching_cubes
-
         Examples
         --------
         This will create a data object, find a nice value in the center, and
@@ -995,16 +1072,13 @@ class YTSelectionContainer3D(YTSelectionContainer):
         """
         verts = []
         samples = []
-        pb = get_pbar("Extracting ", len(list(self._get_grid_objs())))
-        for i, g in enumerate(self._get_grid_objs()):
-            pb.update(i)
+        for block, mask in self.blocks:
             my_verts = self._extract_isocontours_from_grid(
-                            g, field, value, sample_values)
+                block, mask, field, value, sample_values)
             if sample_values is not None:
                 my_verts, svals = my_verts
                 samples.append(svals)
             verts.append(my_verts)
-        pb.finish()
         verts = np.concatenate(verts).transpose()
         verts = self.comm.par_combine_object(verts, op='cat', datatype='array')
         verts = verts.transpose()
@@ -1028,11 +1102,9 @@ class YTSelectionContainer3D(YTSelectionContainer):
             return verts, samples
         return verts
 
-
-    def _extract_isocontours_from_grid(self, grid, field, value,
-                                       sample_values = None):
-        mask = self._get_cut_mask(grid) * grid.child_mask
-        vals = grid.get_vertex_centered_data(field, no_ghost = False)
+    def _extract_isocontours_from_grid(self, grid, mask, field, value,
+                                       sample_values=None):
+        vals = grid.get_vertex_centered_data(field, no_ghost=False)
         if sample_values is not None:
             svals = grid.get_vertex_centered_data(sample_values)
         else:
@@ -1047,11 +1119,12 @@ class YTSelectionContainer3D(YTSelectionContainer):
         consideration of global connectedness, and calculates the flux over
         those contours.
 
-        This function will conduct marching cubes on all the cells in a given
-        data container (grid-by-grid), and then for each identified triangular
-        segment of an isocontour in a given cell, calculate the gradient (i.e.,
-        normal) in the isocontoured field, interpolate the local value of the
-        "fluxing" field, the area of the triangle, and then return:
+        This function will conduct `marching cubes
+        <http://en.wikipedia.org/wiki/Marching_cubes>`_ on all the cells in a
+        given data container (grid-by-grid), and then for each identified
+        triangular segment of an isocontour in a given cell, calculate the
+        gradient (i.e., normal) in the isocontoured field, interpolate the local
+        value of the "fluxing" field, the area of the triangle, and then return:
 
         area * local_flux_value * (n dot v)
 
@@ -1089,11 +1162,6 @@ class YTSelectionContainer3D(YTSelectionContainer):
             The summed flux.  Note that it is not currently scaled; this is
             simply the code-unit area times the fields.
 
-        References
-        ----------
-
-        .. [1] Marching Cubes: http://en.wikipedia.org/wiki/Marching_cubes
-
         Examples
         --------
         This will create a data object, find a nice value in the center, and
@@ -1106,15 +1174,14 @@ class YTSelectionContainer3D(YTSelectionContainer):
         ...     "velocity_x", "velocity_y", "velocity_z", "Metal_Density")
         """
         flux = 0.0
-        for g in self._get_grid_objs():
-            flux += self._calculate_flux_in_grid(g, field, value,
-                    field_x, field_y, field_z, fluxing_field)
+        for block, mask in self.blocks:
+            flux += self._calculate_flux_in_grid(block, mask, field, value, field_x,
+                                                 field_y, field_z, fluxing_field)
         flux = self.comm.mpi_allreduce(flux, op="sum")
         return flux
 
-    def _calculate_flux_in_grid(self, grid, field, value,
+    def _calculate_flux_in_grid(self, grid, mask, field, value,
                     field_x, field_y, field_z, fluxing_field = None):
-        mask = self._get_cut_mask(grid) * grid.child_mask
         vals = grid.get_vertex_centered_data(field)
         if fluxing_field is None:
             ff = np.ones(vals.shape, dtype="float64")
@@ -1250,14 +1317,14 @@ class YTBooleanRegionBase(YTSelectionContainer3D):
     """
     This will build a hybrid region based on the boolean logic
     of the regions.
-    
+
     Parameters
     ----------
     regions : list
         A list of region objects and strings describing the boolean logic
         to use when building the hybrid region. The boolean logic can be
         nested using parentheses.
-    
+
     Examples
     --------
     >>> re1 = ds.region([0.5, 0.5, 0.5], [0.4, 0.4, 0.4],
@@ -1271,9 +1338,9 @@ class YTBooleanRegionBase(YTSelectionContainer3D):
     """
     _type_name = "boolean"
     _con_args = ("regions",)
-    def __init__(self, regions, fields = None, ds = None, **kwargs):
+    def __init__(self, regions, fields = None, ds = None, field_parameters = None, data_source = None):
         # Center is meaningless, but we'll define it all the same.
-        YTSelectionContainer3D.__init__(self, [0.5]*3, fields, ds, **kwargs)
+        YTSelectionContainer3D.__init__(self, [0.5]*3, fields, ds, field_parameters, data_source)
         self.regions = regions
         self._all_regions = []
         self._some_overlap = []
@@ -1287,7 +1354,7 @@ class YTBooleanRegionBase(YTSelectionContainer3D):
         # Before anything, we simply find out which regions are involved in all
         # of this process, uniquely.
         for item in self.regions:
-            if isinstance(item, types.StringType): continue
+            if isinstance(item, bytes): continue
             self._all_regions.append(item)
             # So cut_masks don't get messed up.
             item._boolean_touched = True
@@ -1299,7 +1366,7 @@ class YTBooleanRegionBase(YTSelectionContainer3D):
         pbar = get_pbar("Building boolean", len(self._all_regions))
         for i, region in enumerate(self._all_regions):
             try:
-                region._get_list_of_grids() # This is no longer supported. 
+                region._get_list_of_grids() # This is no longer supported.
                 alias = region
             except AttributeError:
                 alias = region.data         # This is no longer supported.
