@@ -29,18 +29,30 @@ from yt.utilities.physical_constants import \
     gravitational_constant_cgs, \
     HUGE
 from yt.utilities.math_utils import prec_accum
+from yt.extern.six import add_metaclass
 
 derived_quantity_registry = {}
 
+def get_position_fields(field, data):
+    axis_names = [data.ds.coordinates.axis_name[num] for num in [0, 1, 2]]
+    if field[0] in data.ds.particle_types:
+        position_fields = [(field[0], 'particle_position_%s' % d)
+                           for d in axis_names]
+    else:
+        position_fields = axis_names
+
+    return position_fields
+
+class RegisteredDerivedQuantity(type):
+    def __init__(cls, name, b, d):
+        type.__init__(cls, name, b, d)
+        if name != "DerivedQuantity":
+            derived_quantity_registry[name] = cls
+
+@add_metaclass(RegisteredDerivedQuantity)
 class DerivedQuantity(ParallelAnalysisInterface):
     num_vals = -1
 
-    class __metaclass__(type):
-        def __init__(cls, name, b, d):
-            type.__init__(cls, name, b, d)
-            if name != "DerivedQuantity":
-                derived_quantity_registry[name] = cls
-
     def __init__(self, data_source):
         self.data_source = data_source
 
@@ -94,8 +106,12 @@ class WeightedAverageQuantity(DerivedQuantity):
     r"""
     Calculates the weight average of a field or fields.
 
-    Where f is the field and w is the weight, the weighted average is 
-    Sum_i(f_i * w_i) / Sum_i(w_i).
+    Returns a YTQuantity for each field requested; if one,
+    it returns a single YTQuantity, if many, it returns a list of YTQuantities
+    in order of the listed fields.  
+
+    Where f is the field and w is the weight, the weighted average is
+    Sum_i(f_i \* w_i) / Sum_i(w_i).
 
     Parameters
     ----------
@@ -112,7 +128,7 @@ class WeightedAverageQuantity(DerivedQuantity):
     >>> print ad.quantities.weighted_average_quantity([("gas", "density"),
     ...                                                ("gas", "temperature")],
     ...                                               ("gas", "cell_mass"))
-    
+
     """
     def count_values(self, fields, weight):
         # This is a list now
@@ -149,7 +165,7 @@ class TotalQuantity(DerivedQuantity):
     >>> ds = load("IsolatedGalaxy/galaxy0030/galaxy0030")
     >>> ad = ds.all_data()
     >>> print ad.quantities.total_quantity([("gas", "cell_mass")])
-    
+
     """
     def count_values(self, fields):
         # This is a list now
@@ -171,8 +187,9 @@ class TotalQuantity(DerivedQuantity):
 
 class TotalMass(TotalQuantity):
     r"""
-    Calculates the total mass in gas and particles. Returns a tuple where the
-    first part is total gas mass and the second part is total particle mass.
+    Calculates the total mass of the object. Returns a YTArray where the
+    first element is total gas mass and the second element is total particle 
+    mass.
 
     Examples
     --------
@@ -180,18 +197,21 @@ class TotalMass(TotalQuantity):
     >>> ds = load("IsolatedGalaxy/galaxy0030/galaxy0030")
     >>> ad = ds.all_data()
     >>> print ad.quantities.total_mass()
-    
+
     """
     def __call__(self):
         self.data_source.ds.index
         fi = self.data_source.ds.field_info
         fields = []
         if ("gas", "cell_mass") in fi:
-            fields.append(("gas", "cell_mass"))
+            gas = super(TotalMass, self).__call__([('gas', 'cell_mass')])
+        else:
+            gas = self.data_source.ds.arr([0], 'g')
         if ("all", "particle_mass") in fi:
-            fields.append(("all", "particle_mass"))
-        rv = super(TotalMass, self).__call__(fields)
-        return rv
+            part = super(TotalMass, self).__call__([('all', 'particle_mass')])
+        else:
+            part = self.data_source.ds.arr([0], 'g')
+        return self.data_source.ds.arr([gas, part])
 
 class CenterOfMass(DerivedQuantity):
     r"""
@@ -202,11 +222,11 @@ class CenterOfMass(DerivedQuantity):
     Parameters
     ----------
     use_gas : bool
-        Flag to include gas in the calculation.  Gas is ignored if not 
+        Flag to include gas in the calculation.  Gas is ignored if not
         present.
         Default: True
     use_particles : bool
-        Flag to include particles in the calculation.  Particles are ignored 
+        Flag to include particles in the calculation.  Particles are ignored
         if not present.
         Default: False
 
@@ -216,7 +236,7 @@ class CenterOfMass(DerivedQuantity):
     >>> ds = load("IsolatedGalaxy/galaxy0030/galaxy0030")
     >>> ad = ds.all_data()
     >>> print ad.quantities.center_of_mass()
-    
+
     """
     def count_values(self, use_gas = True, use_particles = False):
         use_gas &= \
@@ -271,11 +291,11 @@ class BulkVelocity(DerivedQuantity):
     Parameters
     ----------
     use_gas : bool
-        Flag to include gas in the calculation.  Gas is ignored if not 
+        Flag to include gas in the calculation.  Gas is ignored if not
         present.
         Default: True
     use_particles : bool
-        Flag to include particles in the calculation.  Particles are ignored 
+        Flag to include particles in the calculation.  Particles are ignored
         if not present.
         Default: True
 
@@ -285,7 +305,7 @@ class BulkVelocity(DerivedQuantity):
     >>> ds = load("IsolatedGalaxy/galaxy0030/galaxy0030")
     >>> ad = ds.all_data()
     >>> print ad.quantities.bulk_velocity()
-    
+
     """
     def count_values(self, use_gas = True, use_particles = False):
         # This is a list now
@@ -298,7 +318,7 @@ class BulkVelocity(DerivedQuantity):
     def process_chunk(self, data, use_gas = True, use_particles = False):
         vals = []
         if use_gas:
-            vals += [(data["gas", "velocity_%s" % ax] * 
+            vals += [(data["gas", "velocity_%s" % ax] *
                       data["gas", "cell_mass"]).sum(dtype=np.float64)
                      for ax in 'xyz']
             vals.append(data["gas", "cell_mass"].sum(dtype=np.float64))
@@ -327,17 +347,20 @@ class BulkVelocity(DerivedQuantity):
 
 class WeightedVariance(DerivedQuantity):
     r"""
-    Calculates the weighted variance and weighted mean for a field 
-    or list of fields.
+    Calculates the weighted variance and weighted mean for a field
+    or list of fields. Returns a YTArray for each field requested; if one,
+    it returns a single YTArray, if many, it returns a list of YTArrays
+    in order of the listed fields.  The first element of each YTArray is
+    the weighted variance, and the second element is the weighted mean.
 
-    Where f is the field, w is the weight, and <f_w> is the weighted mean, 
-    the weighted variance is 
-    Sum_i( (f_i - <f_w>)^2 * w_i ) / Sum_i(w_i).
+    Where f is the field, w is the weight, and <f_w> is the weighted mean,
+    the weighted variance is
+    Sum_i( (f_i - <f_w>)^2 \* w_i ) / Sum_i(w_i).
 
     Parameters
     ----------
     fields : field or list of fields
-        The field or fields of which the variance and mean values are 
+        The field or fields of which the variance and mean values are
         to be calculated.
     weight : field
         The weight field.
@@ -350,7 +373,7 @@ class WeightedVariance(DerivedQuantity):
     >>> print ad.quantities.weighted_variance([("gas", "density"),
     ...                                        ("gas", "temperature")],
     ...                                       ("gas", "cell_mass"))
-    
+
     """
     def count_values(self, fields, weight):
         # This is a list now
@@ -378,30 +401,31 @@ class WeightedVariance(DerivedQuantity):
         my_weight = values.pop(-1)
         all_weight = my_weight.sum(dtype=np.float64)
         rvals = []
-        for i in range(len(values) / 2):
+        for i in range(int(len(values) / 2)):
             my_mean = values[i]
-            my_var2 = values[i + len(values) / 2]
+            my_var2 = values[i + int(len(values) / 2)]
             all_mean = (my_weight * my_mean).sum(dtype=np.float64) / all_weight
-            rvals.append(np.sqrt((my_weight * (my_var2 + 
-                                               (my_mean - all_mean)**2)).sum(dtype=np.float64) / 
-                                               all_weight))
-            rvals.append(all_mean)
+            rvals.append(self.data_source.ds.arr([(np.sqrt((my_weight * 
+                                                 (my_var2 + (my_mean - 
+                                                  all_mean)**2)).sum(dtype=np.float64) 
+                                                  / all_weight)), all_mean]))
         return rvals
-    
+
 class AngularMomentumVector(DerivedQuantity):
     r"""
     Calculates the angular momentum vector, using gas and/or particles.
 
     The angular momentum vector is the mass-weighted mean specific angular momentum.
+    Returns a YTArray of the vector.
 
     Parameters
     ----------
     use_gas : bool
-        Flag to include gas in the calculation.  Gas is ignored if not 
+        Flag to include gas in the calculation.  Gas is ignored if not
         present.
         Default: True
     use_particles : bool
-        Flag to include particles in the calculation.  Particles are ignored 
+        Flag to include particles in the calculation.  Particles are ignored
         if not present.
         Default: True
 
@@ -411,13 +435,9 @@ class AngularMomentumVector(DerivedQuantity):
     >>> ds = load("IsolatedGalaxy/galaxy0030/galaxy0030")
     >>> ad = ds.all_data()
     >>> print ad.quantities.angular_momentum_vector()
-    
+
     """
     def count_values(self, use_gas=True, use_particles=True):
-        use_gas &= \
-          (("gas", "cell_mass") in self.data_source.ds.field_info)
-        use_particles &= \
-          (("all", "particle_mass") in self.data_source.ds.field_info)
         num_vals = 0
         if use_gas: num_vals += 4
         if use_particles: num_vals += 4
@@ -449,13 +469,17 @@ class AngularMomentumVector(DerivedQuantity):
         if values:
             jx += values.pop(0).sum(dtype=np.float64)
             jy += values.pop(0).sum(dtype=np.float64)
-            jz += values.pop(0).sum(dtype=np.float64)            
+            jz += values.pop(0).sum(dtype=np.float64)
             m  += values.pop(0).sum(dtype=np.float64)
-        return (jx / m, jy / m, jz / m)
+        return self.data_source.ds.arr([jx / m, jy / m, jz / m])
 
 class Extrema(DerivedQuantity):
     r"""
     Calculates the min and max value of a field or list of fields.
+    Returns a YTArray for each field requested.  If one, a single YTArray
+    is returned, if many, a list of YTArrays in order of field list is 
+    returned.  The first element of each YTArray is the minimum of the
+    field and the second is the maximum of the field.
 
     Parameters
     ----------
@@ -472,7 +496,7 @@ class Extrema(DerivedQuantity):
     >>> ad = ds.all_data()
     >>> print ad.quantities.extrema([("gas", "density"),
     ...                              ("gas", "temperature")])
-    
+
     """
     def count_values(self, fields, non_zero):
         self.num_vals = len(fields) * 2
@@ -498,12 +522,12 @@ class Extrema(DerivedQuantity):
 
     def reduce_intermediate(self, values):
         # The values get turned into arrays here.
-        return [(mis.min(), mas.max() )
+        return [self.data_source.ds.arr([mis.min(), mas.max()])
                 for mis, mas in zip(values[::2], values[1::2])]
 
 class MaxLocation(DerivedQuantity):
     r"""
-    Calculates the maximum value plus the index, x, y, and z position 
+    Calculates the maximum value plus the index, x, y, and z position
     of the maximum.
 
     Parameters
@@ -517,7 +541,7 @@ class MaxLocation(DerivedQuantity):
     >>> ds = load("IsolatedGalaxy/galaxy0030/galaxy0030")
     >>> ad = ds.all_data()
     >>> print ad.quantities.max_location(("gas", "density"))
-    
+
     """
     def count_values(self, *args, **kwargs):
         self.num_vals = 5
@@ -528,16 +552,18 @@ class MaxLocation(DerivedQuantity):
         return rv
 
     def process_chunk(self, data, field):
+        axis_names = data.ds.coordinates.axis_name
         field = data._determine_fields(field)[0]
         ma = array_like_field(data, -HUGE, field)
-        mx = array_like_field(data, -1, "x")
-        my = array_like_field(data, -1, "y")
-        mz = array_like_field(data, -1, "z")
+        position_fields = get_position_fields(field, data)
+        mx = array_like_field(data, -1, position_fields[0])
+        my = array_like_field(data, -1, position_fields[1])
+        mz = array_like_field(data, -1, position_fields[2])
         maxi = -1
         if data[field].size > 0:
             maxi = np.argmax(data[field])
             ma = data[field][maxi]
-            mx, my, mz = [data[ax][maxi] for ax in 'xyz']
+            mx, my, mz = [data[ax][maxi] for ax in position_fields]
         return (ma, maxi, mx, my, mz)
 
     def reduce_intermediate(self, values):
@@ -546,7 +572,7 @@ class MaxLocation(DerivedQuantity):
 
 class MinLocation(DerivedQuantity):
     r"""
-    Calculates the minimum value plus the index, x, y, and z position 
+    Calculates the minimum value plus the index, x, y, and z position
     of the minimum.
 
     Parameters
@@ -560,7 +586,7 @@ class MinLocation(DerivedQuantity):
     >>> ds = load("IsolatedGalaxy/galaxy0030/galaxy0030")
     >>> ad = ds.all_data()
     >>> print ad.quantities.min_location(("gas", "density"))
-    
+
     """
     def count_values(self, *args, **kwargs):
         self.num_vals = 5
@@ -573,14 +599,15 @@ class MinLocation(DerivedQuantity):
     def process_chunk(self, data, field):
         field = data._determine_fields(field)[0]
         ma = array_like_field(data, HUGE, field)
-        mx = array_like_field(data, -1, "x")
-        my = array_like_field(data, -1, "y")
-        mz = array_like_field(data, -1, "z")
+        position_fields = get_position_fields(field, data)
+        mx = array_like_field(data, -1, position_fields[0])
+        my = array_like_field(data, -1, position_fields[1])
+        mz = array_like_field(data, -1, position_fields[2])
         mini = -1
         if data[field].size > 0:
             mini = np.argmin(data[field])
             ma = data[field][mini]
-            mx, my, mz = [data[ax][mini] for ax in 'xyz']
+            mx, my, mz = [data[ax][mini] for ax in position_fields]
         return (ma, mini, mx, my, mz)
 
     def reduce_intermediate(self, values):
@@ -591,22 +618,22 @@ class SpinParameter(DerivedQuantity):
     r"""
     Calculates the dimensionless spin parameter.
 
-    Given by Equation 3 of Peebles (1971, A&A, 11, 377), the spin parameter 
+    Given by Equation 3 of Peebles (1971, A&A, 11, 377), the spin parameter
     is defined as
-    
-    lambda = (L * |E|^(1/2)) / (G * M^5/2),
-    
-    where L is the total angular momentum, E is the total energy (kinetic and 
+
+    lambda = (L \* |E|^(1/2)) / (G \* M^5/2),
+
+    where L is the total angular momentum, E is the total energy (kinetic and
     potential), G is the gravitational constant, and M is the total mass.
 
     Parameters
     ----------
     use_gas : bool
-        Flag to include gas in the calculation.  Gas is ignored if not 
+        Flag to include gas in the calculation.  Gas is ignored if not
         present.
         Default: True
     use_particles : bool
-        Flag to include particles in the calculation.  Particles are ignored 
+        Flag to include particles in the calculation.  Particles are ignored
         if not present.
         Default: True
 
@@ -616,7 +643,7 @@ class SpinParameter(DerivedQuantity):
     >>> ds = load("IsolatedGalaxy/galaxy0030/galaxy0030")
     >>> ad = ds.all_data()
     >>> print ad.quantities.center_of_mass()
-    
+
     """
     def count_values(self, **kwargs):
         self.num_vals = 3
