@@ -16,7 +16,7 @@ Time series analysis functions.
 import inspect, functools, weakref, glob, types, os
 
 from yt.funcs import *
-from yt.extern.six import add_metaclass
+from yt.extern.six import add_metaclass, string_types
 from yt.convenience import load
 from yt.config import ytcfg
 from .data_containers import data_object_registry
@@ -128,9 +128,9 @@ class DatasetSeries(object):
 
     """
     def __new__(cls, outputs, *args, **kwargs):
-        if isinstance(outputs, basestring):
+        if isinstance(outputs, string_types):
             outputs = get_filenames_from_glob_pattern(outputs)
-        ret = super(DatasetSeries, cls).__new__(cls, *args, **kwargs)
+        ret = super(DatasetSeries, cls).__new__(cls)
         try:
             ret._pre_outputs = outputs[:]
         except TypeError:
@@ -140,7 +140,7 @@ class DatasetSeries(object):
     def __init__(self, outputs, parallel = True, setup_function = None,
                  **kwargs):
         # This is needed to properly set _pre_outputs for Simulation subclasses.
-        if iterable(outputs) and not isinstance(outputs, basestring):
+        if iterable(outputs) and not isinstance(outputs, string_types):
             self._pre_outputs = outputs[:]
         self.tasks = AnalysisTaskProxy(self)
         self.params = TimeSeriesParametersContainer(self)
@@ -156,7 +156,7 @@ class DatasetSeries(object):
     def __iter__(self):
         # We can make this fancier, but this works
         for o in self._pre_outputs:
-            if isinstance(o, types.StringTypes):
+            if isinstance(o, str):
                 ds = load(o, **self.kwargs)
                 self._setup_function(ds)
                 yield ds
@@ -164,13 +164,13 @@ class DatasetSeries(object):
                 yield o
 
     def __getitem__(self, key):
-        if isinstance(key, types.SliceType):
-            if isinstance(key.start, types.FloatType):
+        if isinstance(key, slice):
+            if isinstance(key.start, float):
                 return self.get_range(key.start, key.stop)
             # This will return a sliced up object!
             return DatasetSeries(self._pre_outputs[key], self.parallel)
         o = self._pre_outputs[key]
-        if isinstance(o, types.StringTypes):
+        if isinstance(o, str):
             o = load(o, **self.kwargs)
             self._setup_function(o)
         return o
@@ -323,8 +323,16 @@ class DatasetSeries(object):
 
         """
         
-        if isinstance(filenames, types.StringTypes):
+        if isinstance(filenames, str):
             filenames = get_filenames_from_glob_pattern(filenames)
+
+        # This will crash with a less informative error if filenames is not
+        # iterable, but the plural keyword should give users a clue...
+        for fn in filenames:
+            if not isinstance(fn, str):
+                raise YTOutputNotIdentified("DataSeries accepts a list of "
+                                            "strings, but "
+                                            "received {0}".format(fn))
         obj = cls(filenames[:], parallel = parallel,
                   setup_function = setup_function, **kwargs)
         return obj
@@ -398,6 +406,7 @@ class SimulationTimeSeries(DatasetSeries):
         self.basename = os.path.basename(parameter_filename)
         self.directory = os.path.dirname(parameter_filename)
         self.parameters = {}
+        self.key_parameters = []
 
         # Set some parameter defaults.
         self._set_parameter_defaults()
@@ -412,6 +421,21 @@ class SimulationTimeSeries(DatasetSeries):
         
         self.print_key_parameters()
 
+    def _set_parameter_defaults(self):
+        pass
+
+    def _parse_parameter_file(self):
+        pass
+
+    def _set_units(self):
+        pass
+
+    def _calculate_simulation_bounds(self):
+        pass
+
+    def _get_all_outputs(**kwargs):
+        pass
+        
     def __repr__(self):
         return self.parameter_filename
 
@@ -437,23 +461,78 @@ class SimulationTimeSeries(DatasetSeries):
         """
         Print out some key parameters for the simulation.
         """
-        for a in ["domain_dimensions", "domain_left_edge",
-                  "domain_right_edge", "initial_time", "final_time",
-                  "stop_cycle", "cosmological_simulation"]:
-            if not hasattr(self, a):
-                mylog.error("Missing %s in dataset definition!", a)
-                continue
-            v = getattr(self, a)
-            mylog.info("Parameters: %-25s = %s", a, v)
-        if hasattr(self, "cosmological_simulation") and \
-           getattr(self, "cosmological_simulation"):
+        if self.simulation_type == "grid":
+            for a in ["domain_dimensions", "domain_left_edge",
+                      "domain_right_edge"]:
+                self._print_attr(a)
+        for a in ["initial_time", "final_time",
+                  "cosmological_simulation"]:
+            self._print_attr(a)
+        if getattr(self, "cosmological_simulation", False):
             for a in ["box_size", "omega_lambda",
                       "omega_matter", "hubble_constant",
                       "initial_redshift", "final_redshift"]:
-                if not hasattr(self, a):
-                    mylog.error("Missing %s in dataset definition!", a)
-                    continue
-                v = getattr(self, a)
-                mylog.info("Parameters: %-25s = %s", a, v)
+                self._print_attr(a)
+        for a in self.key_parameters:
+            self._print_attr(a)
         mylog.info("Total datasets: %d." % len(self.all_outputs))
 
+    def _print_attr(self, a):
+        """
+        Print the attribute or warn about it missing.
+        """
+        if not hasattr(self, a):
+            mylog.error("Missing %s in dataset definition!", a)
+            return
+        v = getattr(self, a)
+        mylog.info("Parameters: %-25s = %s", a, v)
+
+    def _get_outputs_by_key(self, key, values, tolerance=None, outputs=None):
+        r"""
+        Get datasets at or near to given values.
+
+        Parameters
+        ----------
+        key: str
+            The key by which to retrieve outputs, usually 'time' or
+            'redshift'.
+        values: array_like
+            A list of values, given as floats.
+        tolerance : float
+            If not None, do not return a dataset unless the value is
+            within the tolerance value.  If None, simply return the
+            nearest dataset.
+            Default: None.
+        outputs : list
+            The list of outputs from which to choose.  If None,
+            self.all_outputs is used.
+            Default: None.
+
+        Examples
+        --------
+        >>> datasets = es.get_outputs_by_key('redshift', [0, 1, 2], tolerance=0.1)
+
+        """
+
+        if not isinstance(values, YTArray):
+            if isinstance(values, tuple) and len(values) == 2:
+                values = self.arr(*values)
+            else:
+                values = self.arr(values)
+        values = values.in_cgs()
+
+        if outputs is None:
+            outputs = self.all_outputs
+        my_outputs = []
+        if not outputs:
+            return my_outputs
+        for value in values:
+            outputs.sort(key=lambda obj:np.abs(value - obj[key]))
+            if (tolerance is None or np.abs(value - outputs[0][key]) <= tolerance) \
+                    and outputs[0] not in my_outputs:
+                my_outputs.append(outputs[0])
+            else:
+                mylog.error("No dataset added for %s = %f.", key, value)
+
+        outputs.sort(key=lambda obj: obj['time'])
+        return my_outputs
