@@ -16,29 +16,26 @@ native.
 #-----------------------------------------------------------------------------
 
 import numpy as np
-import types
 from numbers import Number as numeric_type
 
 from yt.funcs import mylog, only_on_root
 from yt.units.unit_object import Unit
-from yt.units.yt_array import YTArray
 from .derived_field import \
     DerivedField, \
     NullFunc, \
-    TranslationFunc, \
-    ValidateSpatial
+    TranslationFunc
 from yt.utilities.exceptions import \
     YTFieldNotFound
 from .field_plugin_registry import \
     field_plugins
-from yt.units.unit_object import \
-    Unit
 from .particle_fields import \
+    add_union_field, \
     particle_deposition_functions, \
     particle_vector_functions, \
     particle_scalar_functions, \
     standard_particle_fields, \
-    add_volume_weighted_smoothed_field
+    add_volume_weighted_smoothed_field, \
+    sph_whitelist_fields
 
 class FieldInfoContainer(dict):
     """
@@ -51,6 +48,7 @@ class FieldInfoContainer(dict):
     fallback = None
     known_other_fields = ()
     known_particle_fields = ()
+    extra_union_fields = ()
 
     def __init__(self, ds, field_list, slice_info = None):
         self._show_field_errors = []
@@ -121,6 +119,13 @@ class FieldInfoContainer(dict):
                                    num_neighbors=num_neighbors,
                                    ftype=ftype)
 
+    def setup_extra_union_fields(self, ptype="all"):
+        if ptype != "all":
+            raise RuntimeError("setup_extra_union_fields is currently" + 
+                               "only enabled for particle type \"all\".")
+        for units, field in self.extra_union_fields:
+            add_union_field(self, ptype, field, units)
+
     def setup_smoothed_fields(self, ptype, num_neighbors = 64, ftype = "gas"):
         # We can in principle compute this, but it is not yet implemented.
         if (ptype, "density") not in self:
@@ -130,23 +135,17 @@ class FieldInfoContainer(dict):
         else:
             sml_name = None
         new_aliases = []
-        for _, alias_name in self.field_aliases:
-            if alias_name in ("particle_position", "particle_velocity"):
+        for ptype2, alias_name in list(self):
+            if ptype2 != ptype:
                 continue
-            if (ptype, alias_name) not in self: continue
-            fn = add_volume_weighted_smoothed_field(ptype,
-                "particle_position", "particle_mass",
-                sml_name, "density", alias_name, self,
-                num_neighbors)
-            new_aliases.append(((ftype, alias_name), fn[0]))
-        for ptype2, alias_name in self.keys():
-            if ptype2 != ptype: continue
-            if alias_name in ("particle_position", "particle_velocity"):
+            if alias_name not in sph_whitelist_fields:
                 continue
-            fn = add_volume_weighted_smoothed_field(ptype,
-                "particle_position", "particle_mass",
+            fn = add_volume_weighted_smoothed_field(
+                ptype, "particle_position", "particle_mass",
                 sml_name, "density", alias_name, self,
                 num_neighbors)
+            if 'particle_' in alias_name:
+                alias_name = alias_name.replace('particle_', '')
             new_aliases.append(((ftype, alias_name), fn[0]))
         for alias, source in new_aliases:
             #print "Aliasing %s => %s" % (alias, source)
@@ -166,7 +165,7 @@ class FieldInfoContainer(dict):
             # field *name* is in there, then the field *tuple*.
             units = self.ds.field_units.get(field[1], units)
             units = self.ds.field_units.get(field, units)
-            if not isinstance(units, types.StringTypes) and args[0] != "":
+            if not isinstance(units, str) and args[0] != "":
                 units = "((%s)*%s)" % (args[0], units)
             if isinstance(units, (numeric_type, np.number, np.ndarray)) and \
                 args[0] == "" and units != 1.0:
@@ -197,7 +196,8 @@ class FieldInfoContainer(dict):
            arguments (field, data)
         units : str
            A plain text string encoding the unit.  Powers must be in
-           python syntax (** instead of ^).
+           python syntax (** instead of ^). If set to "auto" the units
+           will be inferred from the return value of the field function.
         take_log : bool
            Describes whether the field should be logged
         validators : list
@@ -211,11 +211,24 @@ class FieldInfoContainer(dict):
 
         """
         override = kwargs.pop("force_override", False)
-        if not override and name in self: return
+        # Handle the case where the field has already been added.
+        if not override and name in self:
+            # See below.
+            if function is None:
+                def create_function(f):
+                    return f
+                return create_function
+            return
+        # add_field can be used in two different ways: it can be called
+        # directly, or used as a decorator. If called directly, the
+        # function will be passed in as an argument, and we simply create
+        # the derived field and exit. If used as a decorator, function will
+        # be None. In that case, we return a function that will be applied
+        # to the function that the decorator is applied to.
         if function is None:
-            def create_function(function):
-                self[name] = DerivedField(name, function, **kwargs)
-                return function
+            def create_function(f):
+                self[name] = DerivedField(name, f, **kwargs)
+                return f
             return create_function
         self[name] = DerivedField(name, function, **kwargs)
 
@@ -267,7 +280,7 @@ class FieldInfoContainer(dict):
         # This gets used a lot
         if key in self: return True
         if self.fallback is None: return False
-        return self.fallback.has_key(key)
+        return key in self.fallback
 
     def __missing__(self, key):
         if self.fallback is None:
@@ -295,13 +308,13 @@ class FieldInfoContainer(dict):
     def keys(self):
         keys = dict.keys(self)
         if self.fallback:
-            keys += self.fallback.keys()
+            keys += list(self.fallback.keys())
         return keys
 
     def check_derived_fields(self, fields_to_check = None):
         deps = {}
         unavailable = []
-        fields_to_check = fields_to_check or self.keys()
+        fields_to_check = fields_to_check or list(self.keys())
         for field in fields_to_check:
             mylog.debug("Checking %s", field)
             if field not in self: raise RuntimeError
