@@ -6,7 +6,8 @@ from yt.utilities.definitions import \
     mpc_conversion, sec_conversion
 from yt.frontends.stream.fields import \
     StreamFieldInfo
-from yt.units.yt_array import YTArray
+from yt.units.yt_array import \
+     YTArray, YTQuantity
 
 def setup():
     global base_ds
@@ -16,7 +17,9 @@ def setup():
     for fname, (code_units, aliases, dn) in StreamFieldInfo.known_other_fields:
         fields.append(("gas", fname))
         units.append(code_units)
-    base_ds = fake_random_ds(4, fields = fields, units = units)
+
+    base_ds = fake_random_ds(4, fields=fields, units=units, particles=10)
+
     base_ds.index
     base_ds.cosmological_simulation = 1
     base_ds.cosmology = Cosmology()
@@ -42,6 +45,7 @@ def get_params(ds):
         omega_baryon = 0.04,
         observer_redshift = 0.0,
         source_redshift = 3.0,
+        virial_radius = YTQuantity(1.0, "cm"),
     )
 
 _base_fields = (("gas", "density"),
@@ -49,12 +53,17 @@ _base_fields = (("gas", "density"),
                 ("gas", "velocity_y"),
                 ("gas", "velocity_z"))
 
-def realistic_ds(fields, nprocs):
+def realistic_ds(fields, particle_fields, nprocs):
     np.random.seed(int(0x4d3d3d3))
     units = [base_ds._get_field_info(*f).units for f in fields]
+    punits = [base_ds._get_field_info('io', f).units for f in particle_fields]
     fields = [_strip_ftype(f) for f in fields]
-    ds = fake_random_ds(16, fields = fields, units = units,
-                        nprocs = nprocs)
+
+    ds = fake_random_ds(16, fields=fields, units=units, nprocs=nprocs,
+                        particle_fields=particle_fields,
+                        particle_field_units=punits,
+                        particles=base_ds.stream_handler.particle_count[0][0])
+
     ds.parameters["HydroMethod"] = "streaming"
     ds.parameters["EOSType"] = 1.0
     ds.parameters["EOSSoundSpeed"] = 1.0
@@ -75,7 +84,7 @@ def realistic_ds(fields, nprocs):
 def _strip_ftype(field):
     if not isinstance(field, tuple):
         return field
-    elif field[0] == "all":
+    elif field[0] in ("all", "io"):
         return field
     return field[1]
 
@@ -103,13 +112,23 @@ class TestFieldAccess(object):
         self.nproc = nproc
 
     def __call__(self):
-        if self.field_name in base_ds.field_list:
-            # Don't know how to test this.  We need some way of having fields
-            # that are fallbacks be tested, but we don't have that now.
-            return
+
         field = base_ds._get_field_info(*self.field_name)
         deps = field.get_dependencies(ds = base_ds)
-        fields = deps.requested + list(_base_fields)
+        requested = deps.requested
+        particle_fields = \
+            ['particle_position_x', 'particle_position_y', 'particle_position_z',
+             'particle_velocity_x', 'particle_velocity_y', 'particle_velocity_z',
+             'particle_mass']
+        fields = list(_base_fields)
+
+        for rf in requested:
+            if field.particle_type:
+                if rf not in particle_fields:
+                    particle_fields.append(rf[1])
+            else:
+                fields.append(rf)
+
         skip_grids = False
         needs_spatial = False
         for v in field.validators:
@@ -119,7 +138,9 @@ class TestFieldAccess(object):
                 skip_grids = True
             if hasattr(v, "ghost_zones"):
                 needs_spatial = True
-        ds = realistic_ds(fields, self.nproc)
+
+        ds = realistic_ds(fields, particle_fields, self.nproc)
+
         # This gives unequal sized grids as well as subgrids
         dd1 = ds.all_data()
         dd2 = ds.all_data()
@@ -128,8 +149,7 @@ class TestFieldAccess(object):
         dd2.field_parameters.update(sp)
         v1 = dd1[self.field_name]
         # No more conversion checking
-        if not field.particle_type:
-            assert_equal(v1, dd1[self.field_name])
+        assert_equal(v1, dd1[self.field_name])
         if not needs_spatial:
             with field.unit_registry(dd2):
                 res = field._function(field, dd2)
@@ -141,10 +161,13 @@ class TestFieldAccess(object):
                 v1 = g[self.field_name]
                 g.clear_data()
                 g.field_parameters.update(sp)
-                for ax in 'xyz':
-                    assert_array_equal(g[ax].shape, v1.shape)
                 r1 = field._function(field, g)
-                assert_array_equal(r1.shape, v1.shape)
+                if field.particle_type:
+                    assert_equal(v1.shape[0], g.NumberOfParticles)
+                else:
+                    assert_array_equal(r1.shape, v1.shape)
+                    for ax in 'xyz':
+                        assert_array_equal(g[ax].shape, v1.shape)
                 with field.unit_registry(g):
                     res = field._function(field, g)
                     assert_array_equal(v1.shape, res.shape)
@@ -153,21 +176,90 @@ class TestFieldAccess(object):
 
 def test_all_fields():
     for field in sorted(base_ds.field_info):
-        if not isinstance(field, types.TupleType):
-            field = ("unknown", field)
-        finfo = base_ds._get_field_info(*field)
-        if isinstance(field, types.TupleType):
-            fname = field[0]
-        else:
-            fname = field
-        if field[0] == "deposit": continue
-        if field[1].find("beta_p") > -1: continue
-        if finfo.particle_type: continue
+        if field[1].find("beta_p") > -1:
+            continue
+        if field in base_ds.field_list:
+            # Don't know how to test this.  We need some way of having fields
+            # that are fallbacks be tested, but we don't have that now.
+            continue
+
         for nproc in [1, 4, 8]:
             test_all_fields.__name__ = "%s_%s" % (field, nproc)
             yield TestFieldAccess(field, nproc)
 
+def test_add_deposited_particle_field():
+    fn = base_ds.add_deposited_particle_field(('io', 'particle_ones'), 'count')
+    assert_equal(fn, ('deposit', 'io_count_ones'))
+    ad = base_ds.all_data()
+    ret = ad[fn]
+    assert_equal(ret.sum(), ad['particle_ones'].sum())
+
+def test_add_gradient_fields():
+    gfields = base_ds.add_gradient_fields(("gas","density"))
+    gfields += base_ds.add_gradient_fields(("index", "ones"))
+    field_list = [('gas', 'density_gradient_x'),
+                  ('gas', 'density_gradient_y'),
+                  ('gas', 'density_gradient_z'),
+                  ('gas', 'density_gradient_magnitude'),
+                  ('index', 'ones_gradient_x'),
+                  ('index', 'ones_gradient_y'),
+                  ('index', 'ones_gradient_z'),
+                  ('index', 'ones_gradient_magnitude')]
+    assert_equal(gfields, field_list)
+    ad = base_ds.all_data()
+    for field in field_list:
+        ret = ad[field]
+        if field[0] == 'gas':
+            assert str(ret.units) == "g/cm**4"
+        else:
+            assert str(ret.units) == "1/cm"
+
+def get_data(ds, field_name):
+    # Need to create a new data object otherwise the errors we are
+    # intentionally raising lead to spurious GenerationInProgress errors
+    ad = ds.all_data()
+    return ad[field_name]
+
+def test_add_field_unit_semantics():
+    ds = fake_random_ds(16)
+    ad = ds.all_data()
+
+    def density_alias(field, data):
+        return data['density'].in_cgs()
+
+    def unitless_data(field, data):
+            return np.ones(data['density'].shape)
+
+    ds.add_field(('gas','density_alias_no_units'), function=density_alias)
+    ds.add_field(('gas','density_alias_auto'), function=density_alias,
+                 units='auto')
+    ds.add_field(('gas','density_alias_wrong_units'), function=density_alias,
+                 units='m/s')
+    ds.add_field(('gas','density_alias_unparseable_units'), function=density_alias,
+                 units='dragons')
+
+    assert_raises(YTFieldUnitError, get_data, ds, 'density_alias_no_units')
+    assert_raises(YTFieldUnitError, get_data, ds, 'density_alias_wrong_units')
+    assert_raises(YTFieldUnitParseError, get_data, ds,
+                  'density_alias_unparseable_units')
+
+    dens = ad['density_alias_auto']
+    assert_equal(str(dens.units), 'g/cm**3')
+
+    ds.add_field(('gas','dimensionless'), function=unitless_data)
+    ds.add_field(('gas','dimensionless_auto'), function=unitless_data,
+                 units='auto')
+    ds.add_field(('gas','dimensionless_explicit'), function=unitless_data, units='')
+    ds.add_field(('gas','dimensionful'), function=unitless_data, units='g/cm**3')
+
+    assert_equal(str(ad['dimensionless'].units), 'dimensionless')
+    assert_equal(str(ad['dimensionless_auto'].units), 'dimensionless')
+    assert_equal(str(ad['dimensionless_explicit'].units), 'dimensionless')
+    assert_raises(YTFieldUnitError, get_data, ds, 'dimensionful')
+
 if __name__ == "__main__":
     setup()
     for t in test_all_fields():
         t()
+    test_add_deposited_particle_field()
+    test_add_field_unit_semantics()
