@@ -13,7 +13,7 @@ import numpy as np
 import os
 import stat
 import weakref
-import cStringIO
+from yt.extern.six.moves import cStringIO
 import difflib
 import glob
 
@@ -28,10 +28,7 @@ from yt.data_objects.octree_subset import \
     OctreeSubset
 from yt.geometry.oct_container import \
     ARTOctreeContainer
-from .fields import \
-    ARTFieldInfo
-from yt.utilities.definitions import \
-    mpc_conversion
+from .fields import ARTFieldInfo
 from yt.utilities.io_handler import \
     io_registry
 from yt.utilities.lib.misc_utilities import \
@@ -50,14 +47,10 @@ from .io import _read_root_level
 from .io import b2t
 from .io import a2b
 
-from yt.utilities.definitions import \
-    mpc_conversion, sec_conversion
 from yt.utilities.io_handler import \
     io_registry
 from yt.fields.field_info_container import \
     FieldInfoContainer, NullFunc
-from yt.utilities.physical_constants import \
-    mass_hydrogen_cgs, sec_per_Gyr
 
 
 class ARTIndex(OctreeIndex):
@@ -202,7 +195,7 @@ class ARTDataset(Dataset):
         base_prefix, base_suffix = filename_pattern['amr']
         aexpstr = 'a'+file_amr.rsplit('a',1)[1].replace(base_suffix,'')
         possibles = glob.glob(os.path.dirname(os.path.abspath(file_amr))+"/*")
-        for filetype, (prefix, suffix) in filename_pattern.iteritems():
+        for filetype, (prefix, suffix) in filename_pattern.items():
             # if this attribute is already set skip it
             if getattr(self, "_file_"+filetype, None) is not None:
                 continue
@@ -255,7 +248,6 @@ class ARTDataset(Dataset):
         self.velocity_unit = self.quan(velocity, "cm/s")
         self.time_unit = self.length_unit / self.velocity_unit
 
-
     def _parse_parameter_file(self):
         """
         Get the various simulation parameters & constants.
@@ -301,6 +293,7 @@ class ARTDataset(Dataset):
             self.iOctFree, self.nOct = fpu.read_vector(f, 'i', '>')
             self.child_grid_offset = f.tell()
             self.parameters.update(amr_header_vals)
+            amr_header_vals = None
             # estimate the root level
             float_center, fl, iocts, nocts, root_level = _read_art_level_info(
                 f,
@@ -348,18 +341,18 @@ class ARTDataset(Dataset):
 
         # setup standard simulation params yt expects to see
         self.current_redshift = self.parameters["aexpn"]**-1.0 - 1.0
-        self.omega_lambda = amr_header_vals['Oml0']
-        self.omega_matter = amr_header_vals['Om0']
-        self.hubble_constant = amr_header_vals['hubble']
-        self.min_level = amr_header_vals['min_level']
-        self.max_level = amr_header_vals['max_level']
+        self.omega_lambda = self.parameters['Oml0']
+        self.omega_matter = self.parameters['Om0']
+        self.hubble_constant = self.parameters['hubble']
+        self.min_level = self.parameters['min_level']
+        self.max_level = self.parameters['max_level']
         if self.limit_level is not None:
             self.max_level = min(
-                self.limit_level, amr_header_vals['max_level'])
+                self.limit_level, self.parameters['max_level'])
         if self.force_max_level is not None:
             self.max_level = self.force_max_level
         self.hubble_time = 1.0/(self.hubble_constant*100/3.08568025e19)
-        self.current_time = b2t(self.parameters['t']) * sec_per_Gyr
+        self.current_time = self.quan(b2t(self.parameters['t']), 'Gyr')
         self.gamma = self.parameters["gamma"]
         mylog.info("Max level is %02i", self.max_level)
 
@@ -410,7 +403,7 @@ class DarkMatterARTDataset(ARTDataset):
     _file_class = ARTParticleFile
     filter_bbox = False
 
-    def __init__(self, filename, dataset_type='art',
+    def __init__(self, filename, dataset_type='dm_art',
                           fields=None, storage_filename=None,
                           skip_particles=False, skip_stars=False,
                  limit_level=None, spread_age=True,
@@ -601,7 +594,7 @@ class DarkMatterARTDataset(ARTDataset):
 #            self.max_level = self.force_max_level
         self.hubble_time = 1.0/(self.hubble_constant*100/3.08568025e19)
         self.parameters['t'] = a2b(self.parameters['aexpn'])
-        self.current_time = b2t(self.parameters['t']) * sec_per_Gyr
+        self.current_time = self.quan(b2t(self.parameters['t']), 'Gyr')
         self.gamma = self.parameters["gamma"]
         mylog.info("Max level is %02i", self.max_level)
 
@@ -784,7 +777,7 @@ class ARTDomainFile(object):
         Level = np.zeros(MaxLevelNow+1 - MinLev, dtype='int64')
         iNOLL = np.zeros(MaxLevelNow+1 - MinLev, dtype='int64')
         iHOLL = np.zeros(MaxLevelNow+1 - MinLev, dtype='int64')
-        for Lev in xrange(MinLev + 1, MaxLevelNow+1):
+        for Lev in range(MinLev + 1, MaxLevelNow+1):
             level_oct_offsets.append(f.tell())
 
             # Get the info for this level, skip the rest
