@@ -18,7 +18,10 @@ import numpy as np
 import struct
 import os
 import os.path
-
+import sys
+if sys.version_info >= (3,0,0):
+    long = int
+    
 from yt.funcs import *
 from yt.utilities.io_handler import \
     BaseIOHandler
@@ -26,6 +29,9 @@ from yt.utilities.fortran_utils import *
 from yt.utilities.logger import ytLogger as mylog
 from yt.frontends.art.definitions import *
 from yt.utilities.physical_constants import sec_per_year
+from yt.utilities.lib.geometry_utils import compute_morton
+from yt.geometry.oct_container import _ORDER_MAX
+from yt.units.yt_array import YTQuantity
 
 
 class IOHandlerART(BaseIOHandler):
@@ -85,6 +91,7 @@ class IOHandlerART(BaseIOHandler):
             return mask
 
     def _read_particle_coords(self, chunks, ptf):
+        chunks = list(chunks)
         for chunk in chunks:
             for ptype, field_list in sorted(ptf.items()):
                 x = self._get_field((ptype, "particle_position_x"))
@@ -93,6 +100,7 @@ class IOHandlerART(BaseIOHandler):
                 yield ptype, (x, y, z)
 
     def _read_particle_fields(self, chunks, ptf, selector):
+        chunks = list(chunks)
         for chunk in chunks:
             for ptype, field_list in sorted(ptf.items()):
                 x = self._get_field((ptype, "particle_position_x"))
@@ -126,7 +134,7 @@ class IOHandlerART(BaseIOHandler):
                 tr[field] = rp([ax])[0]/dd - off
             if fname.startswith("particle_velocity_%s" % ax):
                 tr[field], = rp(['v'+ax])
-        if fname == "particle_mass":
+        if fname.startswith("particle_mass"):
             a = 0
             data = np.zeros(npa, dtype='f8')
             for ptb, size, m in zip(pbool, sizes, self.ws):
@@ -163,7 +171,7 @@ class IOHandlerART(BaseIOHandler):
             tr[field] = temp
             del data
         # We check again, after it's been filled
-        if fname == "particle_mass":
+        if fname.startswith("particle_mass"):
             # We now divide by NGrid in order to make this match up.  Note that
             # this means that even when requested in *code units*, we are
             # giving them as modified by the ng value.  This only works for
@@ -177,6 +185,106 @@ class IOHandlerART(BaseIOHandler):
         else:
             return tr[field]
 
+class IOHandlerDarkMatterART(IOHandlerART):
+    _dataset_type = "dm_art"
+    def _count_particles(self, data_file):
+        return self.ds.parameters['lspecies'][-1]
+
+    def _initialize_index(self, data_file, regions):
+        totcount = 4096**2 #file is always this size
+        count = data_file.ds.parameters['lspecies'][-1]
+        DLE = data_file.ds.domain_left_edge
+        DRE = data_file.ds.domain_right_edge
+        dx = (DRE - DLE) / 2**_ORDER_MAX
+        with open(data_file.filename, "rb") as f:
+            # The first total_particles * 3 values are positions
+            pp = np.fromfile(f, dtype = '>f4', count = totcount*3)
+            pp.shape = (3, totcount)
+            pp = pp[:,:count] #remove zeros
+            pp = np.transpose(pp).astype(np.float32) #cast as float32 for compute_morton
+            pp = (pp - 1.)/data_file.ds.parameters['ng'] #correct the dm particle units
+        regions.add_data_file(pp, data_file.file_id)
+        morton = compute_morton(pp[:,0], pp[:,1], pp[:,2], DLE, DRE)
+        return morton
+
+    def _identify_fields(self, domain):
+        field_list = []
+        tp = domain.total_particles
+        self.particle_field_list = [f for f in particle_fields]
+        for ptype in self.ds.particle_types_raw:
+            for pfield in self.particle_field_list:
+                pfn = (ptype, pfield)
+                field_list.append(pfn)
+        return field_list, {}
+
+    def _get_field(self,  field):
+        if field in self.cache.keys() and self.caching:
+            mylog.debug("Cached %s", str(field))
+            return self.cache[field]
+        mylog.debug("Reading %s", str(field))
+        tr = {}
+        ftype, fname = field
+        ptmax = self.ws[-1]
+        pbool, idxa, idxb = _determine_field_size(self.ds, ftype, 
+                                                  self.ls, ptmax)
+        npa = idxb - idxa
+        sizes = np.diff(np.concatenate(([0], self.ls)))
+        rp = lambda ax: read_particles(
+            self.file_particle, self.Nrow, idxa=idxa,
+            idxb=idxb, fields=ax)
+        for i, ax in enumerate('xyz'):
+            if fname.startswith("particle_position_%s" % ax):
+                # This is not the same as domain_dimensions
+                dd = self.ds.parameters['ng']
+                off = 1.0/dd
+                tr[field] = rp([ax])[0]/dd - off
+            if fname.startswith("particle_velocity_%s" % ax):
+                tr[field], = rp(['v'+ax])
+        if fname.startswith("particle_mass"):
+            a = 0
+            data = np.zeros(npa, dtype='f8')
+            for ptb, size, m in zip(pbool, sizes, self.ws):
+                if ptb:
+                    data[a:a+size] = m
+                    a += size
+            tr[field] = data
+        elif fname == "particle_index":
+            tr[field] = np.arange(idxa, idxb)
+        elif fname == "particle_type":
+            a = 0
+            data = np.zeros(npa, dtype='int')
+            for i, (ptb, size) in enumerate(zip(pbool, sizes)):
+                if ptb:
+                    data[a: a + size] = i
+                    a += size
+            tr[field] = data
+        if fname == "particle_creation_time":
+            self.tb, self.ages, data = interpolate_ages(
+                tr[field][-nstars:],
+                self.file_stars,
+                self.tb,
+                self.ages,
+                self.ds.current_time)
+            temp = tr.get(field, np.zeros(npa, 'f8'))
+            temp[-nstars:] = data
+            tr[field] = temp
+            del data
+        # We check again, after it's been filled
+        if fname.startswith("particle_mass"):
+            # We now divide by NGrid in order to make this match up.  Note that
+            # this means that even when requested in *code units*, we are
+            # giving them as modified by the ng value.  This only works for
+            # dark_matter -- stars are regular matter.
+            tr[field] /= self.ds.domain_dimensions.prod()
+        if tr == {}:
+            tr[field] = np.array([])
+        if self.caching:
+            self.cache[field] = tr[field]
+            return self.cache[field]
+        else:
+            return tr[field]
+
+
 def _determine_field_size(pf, field, lspecies, ptmax):
     pbool = np.zeros(len(lspecies), dtype="bool")
     idxas = np.concatenate(([0, ], lspecies[:-1]))
@@ -193,18 +301,19 @@ def _determine_field_size(pf, field, lspecies, ptmax):
 def interpolate_ages(data, file_stars, interp_tb=None, interp_ages=None,
                      current_time=None):
     if interp_tb is None:
-        tdum, adum = read_star_field(file_stars,
-                                     field="tdum")
+        t_stars, a_stars = read_star_field(file_stars,
+                                     field="t_stars")
         # timestamp of file should match amr timestamp
         if current_time:
-            tdiff = b2t(tdum)-current_time/(sec_per_year*1e9)
-            if np.abs(tdiff) < 1e-4:
+            tdiff = YTQuantity(b2t(t_stars), 'Gyr') - current_time.in_units('Gyr')
+            if np.abs(tdiff) > 1e-4:
                 mylog.info("Timestamp mismatch in star " +
-                           "particle header")
+                           "particle header: %s", tdiff)
         mylog.info("Interpolating ages")
         interp_tb, interp_ages = b2t(data)
+        interp_tb = YTArray(interp_tb, 'Gyr')
+        interp_ages = YTArray(interp_ages, 'Gyr')
     temp = np.interp(data, interp_tb, interp_ages)
-    temp *= 1.0e9*sec_per_year
     return interp_tb, interp_ages, temp
 
 
@@ -375,7 +484,8 @@ def read_star_field(file, field=None):
     data = {}
     with open(file, 'rb') as fh:
         for dtype, variables in star_struct:
-            found = field in variables or field == variables
+            found = (isinstance(variables, tuple) and field in variables) or \
+                field == variables
             if found:
                 data[field] = read_vector(fh, dtype[1], dtype[0])
             else:
