@@ -15,7 +15,6 @@ Data structures for BoxLib Codes
 
 import os
 import re
-import itertools
 
 from stat import ST_CTIME
 
@@ -23,6 +22,7 @@ import numpy as np
 
 from yt.funcs import *
 from yt.data_objects.grid_patch import AMRGridPatch
+from yt.extern.six.moves import zip as izip
 from yt.geometry.grid_geometry_handler import GridIndex
 from yt.data_objects.static_output import Dataset
 
@@ -98,15 +98,14 @@ class BoxlibGrid(AMRGridPatch):
         return [self.index.grids[cid - self._id_offset]
                 for cid in self._children_ids]
 
-    def _seek(self, f):
+    def _get_offset(self, f):
         # This will either seek to the _offset or figure out the correct
         # _offset.
         if self._offset == -1:
             f.seek(self._base_offset, os.SEEK_SET)
             f.readline()
             self._offset = f.tell()
-        else:
-            f.seek(self._offset)
+        return self._offset
 
     # We override here because we can have varying refinement levels
     def select_ires(self, dobj):
@@ -159,14 +158,14 @@ class BoxlibHierarchy(GridIndex):
 
         dx = []
         for i in range(self.max_level + 1):
-            dx.append([float(v) for v in header_file.next().split()])
+            dx.append([float(v) for v in next(header_file).split()])
             # account for non-3d data sets
             if self.dimensionality < 2:
                 dx[i].append(DRE[1] - DLE[1])
             if self.dimensionality < 3:
                 dx[i].append(DRE[2] - DLE[1])
         self.level_dds = np.array(dx, dtype="float64")
-        header_file.next()
+        next(header_file)
         if self.ds.geometry == "cartesian":
             default_ybounds = (0.0, 1.0)
             default_zbounds = (0.0, 1.0)
@@ -178,7 +177,7 @@ class BoxlibHierarchy(GridIndex):
             default_zbounds = (0.0, 2*np.pi)
         else:
             raise RuntimeError("yt only supports cartesian and cylindrical coordinates.")
-        if int(header_file.next()) != 0:
+        if int(next(header_file)) != 0:
             raise RuntimeError("INTERNAL ERROR! This should be a zero.")
 
         # each level is one group with ngrids on it.
@@ -186,44 +185,44 @@ class BoxlibHierarchy(GridIndex):
         self.grids = []
         grid_counter = 0
         for level in range(self.max_level + 1):
-            vals = header_file.next().split()
+            vals = next(header_file).split()
             lev, ngrids = int(vals[0]), int(vals[1])
             assert(lev == level)
-            nsteps = int(header_file.next())
+            nsteps = int(next(header_file))
             for gi in range(ngrids):
-                xlo, xhi = [float(v) for v in header_file.next().split()]
+                xlo, xhi = [float(v) for v in next(header_file).split()]
                 if self.dimensionality > 1:
-                    ylo, yhi = [float(v) for v in header_file.next().split()]
+                    ylo, yhi = [float(v) for v in next(header_file).split()]
                 else:
                     ylo, yhi = default_ybounds
                 if self.dimensionality > 2:
-                    zlo, zhi = [float(v) for v in header_file.next().split()]
+                    zlo, zhi = [float(v) for v in next(header_file).split()]
                 else:
                     zlo, zhi = default_zbounds
                 self.grid_left_edge[grid_counter + gi, :] = [xlo, ylo, zlo]
                 self.grid_right_edge[grid_counter + gi, :] = [xhi, yhi, zhi]
             # Now we get to the level header filename, which we open and parse.
             fn = os.path.join(self.dataset.output_dir,
-                              header_file.next().strip())
+                              next(header_file).strip())
             level_header_file = open(fn + "_H")
             level_dir = os.path.dirname(fn)
             # We skip the first two lines, which contain BoxLib header file
             # version and 'how' the data was written
-            level_header_file.next()
-            level_header_file.next()
+            next(level_header_file)
+            next(level_header_file)
             # Now we get the number of components
-            ncomp_this_file = int(level_header_file.next())
+            ncomp_this_file = int(next(level_header_file))
             # Skip the next line, which contains the number of ghost zones
-            level_header_file.next()
+            next(level_header_file)
             # To decipher this next line, we expect something like:
             # (8 0
             # where the first is the number of FABs in this level.
-            ngrids = int(level_header_file.next().split()[0][1:])
+            ngrids = int(next(level_header_file).split()[0][1:])
             # Now we can iterate over each and get the indices.
             for gi in range(ngrids):
                 # components within it
-                start, stop = _our_dim_finder.match(level_header_file.next()).groups()
-                # fix for non-3d data
+                start, stop = _our_dim_finder.match(next(level_header_file)).groups()
+                # fix for non-3d data 
                 # note we append '0' to both ends b/c of the '+1' in dims below
                 start += ',0'*(3-self.dimensionality)
                 stop += ',0'*(3-self.dimensionality)
@@ -234,14 +233,14 @@ class BoxlibHierarchy(GridIndex):
                 self.grid_start_index[grid_counter + gi,:] = start
             # Now we read two more lines.  The first of these is a close
             # parenthesis.
-            level_header_file.next()
+            next(level_header_file)
             # The next is again the number of grids
-            level_header_file.next()
+            next(level_header_file)
             # Now we iterate over grids to find their offsets in each file.
             for gi in range(ngrids):
                 # Now we get the data file, at which point we're ready to
                 # create the grid.
-                dummy, filename, offset = level_header_file.next().split()
+                dummy, filename, offset = next(level_header_file).split()
                 filename = os.path.join(level_dir, filename)
                 go = self.grid(grid_counter + gi, int(offset), filename, self)
                 go.Level = self.grid_levels[grid_counter + gi,:] = level
@@ -260,7 +259,7 @@ class BoxlibHierarchy(GridIndex):
         """
         # open the test file & grab the header
         with open(os.path.expanduser(test_grid.filename), 'rb') as f:
-            header = f.readline()
+            header = f.readline().decode("ascii", "ignore")
 
         bpr, endian, start, stop, centering, nc = \
             _header_pattern[self.dimensionality-1].search(header).groups()
@@ -314,7 +313,7 @@ class BoxlibHierarchy(GridIndex):
         header_file = open(self.header_filename, 'r')
         header_file.seek(self.dataset._header_mesh_start)
         # Skip over the level dxs, geometry and the zero:
-        [header_file.next() for i in range(self.dataset._max_level + 3)]
+        [next(header_file) for i in range(self.dataset._max_level + 3)]
         # Now we need to be very careful, as we've seeked, and now we iterate.
         # Does this work?  We are going to count the number of places that we
         # have a three-item line.  The three items would be level, number of
@@ -500,7 +499,7 @@ class BoxlibDataset(Dataset):
         necessary for orientation of the data in space.
         """
 
-        # Note: Python uses a read-ahead buffer, so using .next(), which would
+        # Note: Python uses a read-ahead buffer, so using next(), which would
         # be my preferred solution, won't work here.  We have to explicitly
         # call readline() if we want to end up with an offset at the very end.
         # Fortunately, elsewhere we don't care about the offset, so we're fine
@@ -693,7 +692,7 @@ class OrionHierarchy(BoxlibHierarchy):
                 # for each particle, determine which grids contain it
                 # copied from object_finding_mixin.py
                 mask = np.ones(self.num_grids)
-                for i in xrange(len(coord)):
+                for i in range(len(coord)):
                     np.choose(np.greater(self.grid_left_edge.d[:,i],coord[i]), (mask,0), mask)
                     np.choose(np.greater(self.grid_right_edge.d[:,i],coord[i]), (0,mask), mask)
                 ind = np.where(mask == 1)
@@ -781,6 +780,41 @@ class CastroDataset(BoxlibDataset):
         if any(line.startswith("Castro   ") for line in lines): return True
         return False
 
+    def _parse_parameter_file(self):
+        super(CastroDataset, self)._parse_parameter_file()
+        jobinfo_filename = os.path.join(self.output_dir, "job_info")
+        line = ""
+        with open(jobinfo_filename, "r") as f:
+            while not line.startswith(" Inputs File Parameters"):
+                # boundary condition info starts with -x:, etc.
+                bcs = ["-x:", "+x:", "-y:", "+y:", "-z:", "+z:"]
+                if any(b in line for b in bcs):
+                    p, v = line.strip().split(":")
+                    self.parameters[p] = v.strip()
+                line = next(f)
+            
+            # runtime parameters that we overrode follow "Inputs File
+            # Parameters"
+            # skip the "====..." line
+            line = next(f)
+            for line in f:
+                p, v = line.strip().split("=")
+                self.parameters[p] = v.strip()
+
+            
+        # hydro method is set by the base class -- override it here
+        self.parameters["HydroMethod"] = "Castro"
+
+        # set the periodicity based on the runtime parameters
+        periodicity = [True, True, True]
+        if not self.parameters['-x'] == "interior": periodicity[0] = False
+        if self.dimensionality >= 2:
+            if not self.parameters['-y'] == "interior": periodicity[1] = False
+        if self.dimensionality == 3:
+            if not self.parameters['-z'] == "interior": periodicity[2] = False
+
+        self.periodicity = ensure_tuple(periodicity)
+    
 
 class MaestroDataset(BoxlibDataset):
 
@@ -815,7 +849,7 @@ class MaestroDataset(BoxlibDataset):
                     # line format: codename git hash:  the-hash
                     fields = line.split(":")
                     self.parameters[fields[0]] = fields[1].strip()
-                line = f.next()
+                line = next(f)
             # get the runtime parameters
             for line in f:
                 p, v = (_.strip() for _ in line[4:].split("=", 1))
@@ -871,7 +905,7 @@ class NyxHierarchy(BoxlibHierarchy):
                                 count=3*self.num_grids).reshape((self.num_grids, 3))
         # we need grid_info in `populate_grid_objects`, so save it to self
 
-        for g, pg in itertools.izip(self.grids, grid_info):
+        for g, pg in izip(self.grids, grid_info):
             g.particle_filename = os.path.join(self.ds.output_dir, "DM",
                                                "Level_%s" % (g.Level),
                                                "DATA_%04i" % pg[0])
