@@ -18,7 +18,6 @@ import re
 import os
 import weakref
 import numpy as np
-import six
 
 from stat import \
     ST_CTIME
@@ -26,6 +25,7 @@ from stat import \
 from yt.funcs import *
 from yt.data_objects.grid_patch import \
     AMRGridPatch
+from yt.extern import six
 from yt.geometry.grid_geometry_handler import \
     GridIndex
 from yt.data_objects.static_output import \
@@ -47,6 +47,7 @@ from .fields import ChomboFieldInfo, Orion2FieldInfo, \
 class ChomboGrid(AMRGridPatch):
     _id_offset = 0
     __slots__ = ["_level_id", "stop_index"]
+
     def __init__(self, id, index, level, start, stop):
         AMRGridPatch.__init__(self, id, filename = index.index_filename,
                               index = index)
@@ -99,12 +100,6 @@ class ChomboHierarchy(GridIndex):
         self.domain_left_edge = ds.domain_left_edge
         self.domain_right_edge = ds.domain_right_edge
         self.dataset_type = dataset_type
-
-        if ds.dimensionality == 1:
-            self.dataset_type = "chombo1d_hdf5"
-        if ds.dimensionality == 2:
-            self.dataset_type = "chombo2d_hdf5"
-
         self.field_indexes = {}
         self.dataset = weakref.proxy(ds)
         # for now, the index file is the dataset!
@@ -113,7 +108,8 @@ class ChomboHierarchy(GridIndex):
         self.directory = ds.fullpath
         self._handle = ds._handle
 
-        self.float_type = self._handle['Chombo_global'].attrs['testReal'].dtype.name
+        tr = self._handle['Chombo_global'].attrs.get("testReal", "float32")
+
         self._levels = [key for key in self._handle.keys() if key.startswith('level')]
         GridIndex.__init__(self, ds, dataset_type)
 
@@ -148,20 +144,26 @@ class ChomboHierarchy(GridIndex):
         output_fields = []
         for key, val in self._handle.attrs.items():
             if key.startswith("component"):
-                output_fields.append(val)
+                output_fields.append(val.decode("ascii"))
         self.field_list = [("chombo", c) for c in output_fields]
 
         # look for particle fields
         particle_fields = []
         for key, val in self._handle.attrs.items():
             if key.startswith("particle"):
-                particle_fields.append(val)
-        self.field_list.extend([("io", c) for c in particle_fields])        
+                particle_fields.append(val.decode("ascii"))
+        self.field_list.extend([("io", c) for c in particle_fields])
 
     def _count_grids(self):
         self.num_grids = 0
         for lev in self._levels:
-            self.num_grids += self._handle[lev]['Processors'].len()
+            d = self._handle[lev]
+            if 'Processors' in d:
+                self.num_grids += d['Processors'].len()
+            elif 'boxes' in d:
+                self.num_grids += d['boxes'].len()
+            else:
+                raise RuntimeError("Uknown file specification")
 
     def _parse_index(self):
         f = self._handle # shortcut
@@ -243,34 +245,17 @@ class ChomboDataset(Dataset):
     _field_info_class = ChomboFieldInfo
 
     def __init__(self, filename, dataset_type='chombo_hdf5',
-                 storage_filename = None, ini_filename = None):
+                 storage_filename = None, ini_filename = None,
+                 units_override=None):
         self.fluid_types += ("chombo",)
         self._handle = HDF5FileHandler(filename)
         self.dataset_type = dataset_type
 
-        # look up the dimensionality of the dataset
-        D = self._handle['Chombo_global/'].attrs['SpaceDim']
-        if D == 1:
-            self.dataset_type = 'chombo1d_hdf5'
-        if D == 2:
-            self.dataset_type = 'chombo2d_hdf5'
-
-        # some datasets will not be time-dependent, and to make
-        # matters worse, the simulation time is not always
-        # stored in the same place in the hdf file! Make
-        # sure we handle that here.
-        try:
-            self.current_time = self._handle.attrs['time']
-        except KeyError:
-            try:
-                self.current_time = self._handle['/level_0'].attrs['time']
-            except KeyError:
-                self.current_time = 0.0
-
         self.geometry = "cartesian"
         self.ini_filename = ini_filename
         self.fullplotdir = os.path.abspath(filename)
-        Dataset.__init__(self,filename, self.dataset_type)
+        Dataset.__init__(self, filename, self.dataset_type,
+                         units_override=units_override)
         self.storage_filename = storage_filename
         self.cosmological_simulation = False
 
@@ -286,6 +271,7 @@ class ChomboDataset(Dataset):
         self.length_unit = self.quan(1.0, "cm")
         self.mass_unit = self.quan(1.0, "g")
         self.time_unit = self.quan(1.0, "s")
+        self.magnetic_unit = self.quan(1.0, "gauss")
         self.velocity_unit = self.length_unit / self.time_unit
 
     def _localize(self, f, default):
@@ -315,6 +301,20 @@ class ChomboDataset(Dataset):
 
         self.refine_by = self._handle['/level_0'].attrs['ref_ratio']
         self._determine_periodic()
+        self._determine_current_time()
+
+    def _determine_current_time(self):
+        # some datasets will not be time-dependent, and to make
+        # matters worse, the simulation time is not always
+        # stored in the same place in the hdf file! Make
+        # sure we handle that here.
+        try:
+            self.current_time = self._handle.attrs['time']
+        except KeyError:
+            try:
+                self.current_time = self._handle['/level_0'].attrs['time']
+            except KeyError:
+                self.current_time = 0.0
 
     def _determine_periodic(self):
         # we default to true unless the HDF5 file says otherwise
@@ -386,7 +386,7 @@ class ChomboDataset(Dataset):
 
 class PlutoHierarchy(ChomboHierarchy):
 
-    def __init__(self, ds, dataset_type="pluto_chombo_native"):
+    def __init__(self, ds, dataset_type="chombo_hdf5"):
         ChomboHierarchy.__init__(self, ds, dataset_type)
 
     def _parse_index(self):
@@ -444,11 +444,13 @@ class PlutoDataset(ChomboDataset):
     _index_class = PlutoHierarchy
     _field_info_class = PlutoFieldInfo
 
-    def __init__(self, filename, dataset_type='pluto_chombo_native',
-                 storage_filename = None, ini_filename = None):
+    def __init__(self, filename, dataset_type='chombo_hdf5',
+                 storage_filename = None, ini_filename = None,
+                 units_override=None):
 
         ChomboDataset.__init__(self, filename, dataset_type, 
-                    storage_filename, ini_filename)
+                               storage_filename, ini_filename,
+                               units_override=units_override)
 
     def _parse_parameter_file(self):
         """
@@ -498,6 +500,7 @@ class PlutoDataset(ChomboDataset):
             self.domain_right_edge = np.concatenate((self.domain_right_edge, [1.0]))
             self.domain_dimensions = np.concatenate((self.domain_dimensions, [1]))
 
+        self._determine_current_time()
 
     @classmethod
     def _is_valid(self, *args, **kwargs):
@@ -526,13 +529,14 @@ class Orion2Hierarchy(ChomboHierarchy):
         output_fields = []
         for key, val in self._handle.attrs.items():
             if key.startswith("component"):
-                output_fields.append(val)
+                output_fields.append(val.decode("ascii"))
         self.field_list = [("chombo", c) for c in output_fields]
 
         # look for particle fields
         self.particle_filename = self.index_filename[:-4] + 'sink'
-        if not os.path.exists(self.particle_filename): return
-        pfield_list = [("io", c) for c in self.io.particle_field_index.keys()]
+        if not os.path.exists(self.particle_filename):
+            return
+        pfield_list = [("io", str(c)) for c in self.io.particle_field_index.keys()]
         self.field_list.extend(pfield_list)
 
     def _read_particles(self):
@@ -548,7 +552,7 @@ class Orion2Hierarchy(ChomboHierarchy):
                 # for each particle, determine which grids contain it
                 # copied from object_finding_mixin.py
                 mask = np.ones(self.num_grids)
-                for i in xrange(len(coord)):
+                for i in range(len(coord)):
                     np.choose(np.greater(self.grid_left_edge.d[:,i],coord[i]), (mask,0), mask)
                     np.choose(np.greater(self.grid_right_edge.d[:,i],coord[i]), (0,mask), mask)
                 ind = np.where(mask == 1)
@@ -575,10 +579,12 @@ class Orion2Dataset(ChomboDataset):
     _field_info_class = Orion2FieldInfo
 
     def __init__(self, filename, dataset_type='orion_chombo_native',
-                 storage_filename = None, ini_filename = None):
+                 storage_filename = None, ini_filename = None,
+                 units_override=None):
 
         ChomboDataset.__init__(self, filename, dataset_type,
-                    storage_filename, ini_filename)
+                               storage_filename, ini_filename,
+                               units_override=units_override)
 
     def _parse_parameter_file(self):
         """
@@ -601,6 +607,7 @@ class Orion2Dataset(ChomboDataset):
         self.domain_dimensions = self._calc_domain_dimensions()
         self.refine_by = self._handle['/level_0'].attrs['ref_ratio']
         self._determine_periodic()
+        self._determine_current_time()
 
     def _parse_inputs_file(self, ini_filename):
         self.fullplotdir = os.path.abspath(self.parameter_filename)
@@ -612,12 +619,30 @@ class Orion2Dataset(ChomboDataset):
         # read the file line by line, storing important parameters
         for lineI, line in enumerate(lines):
             try:
-                param, sep, vals = [v.rstrip() for v in line.partition(' ')]
-                #param, sep, vals = map(rstrip,line.partition(' '))
+                param, sep, vals = line.partition('=')
+                if not sep:
+                    # No = sign present, so split by space instead
+                    param, sep, vals = line.partition(' ')
+                param = param.strip()
+                vals = vals.strip()
+                if not param:  # skip blank lines
+                    continue
+                if param[0] == '#':  # skip comment lines
+                    continue
+                if param[0] == '[':  # skip stanza headers
+                    continue
+                vals = vals.partition("#")[0] # strip trailing comments
+                try:
+                    self.parameters[param] = np.int64(vals)
+                except ValueError:
+                    try:
+                        self.parameters[param] = np.float64(vals)
+                    except ValueError:
+                        self.parameters[param] = vals
             except ValueError:
                 mylog.error("ValueError: '%s'", line)
             if param == "GAMMA":
-                self.gamma = vals
+                self.gamma = np.float64(vals)
 
     @classmethod
     def _is_valid(self, *args, **kwargs):
@@ -660,10 +685,12 @@ class ChomboPICDataset(ChomboDataset):
     _field_info_class = ChomboPICFieldInfo3D
 
     def __init__(self, filename, dataset_type='chombo_hdf5',
-                 storage_filename=None, ini_filename=None):
+                 storage_filename=None, ini_filename=None,
+                 units_override=None):
 
         ChomboDataset.__init__(self, filename, dataset_type,
-                               storage_filename, ini_filename)
+                               storage_filename, ini_filename,
+                               units_override=units_override)
 
         if self.dimensionality == 1:
             self._field_info_class = ChomboPICFieldInfo1D
