@@ -20,9 +20,6 @@ from yt.fields.field_info_container import \
     FieldInfoContainer
 from yt.units.yt_array import \
     YTArray
-from yt.fields.species_fields import \
-    add_nuclei_density_fields, \
-    add_species_field_by_density
 from yt.utilities.physical_constants import \
     mh, me, mp, \
     mass_sun_cgs
@@ -91,16 +88,16 @@ class EnzoFieldInfo(FieldInfoContainer):
         ("particle_position_x", ("code_length", [], None)),
         ("particle_position_y", ("code_length", [], None)),
         ("particle_position_z", ("code_length", [], None)),
-        ("particle_velocity_x", (vel_units, ["particle_velocity_x"], None)),
-        ("particle_velocity_y", (vel_units, ["particle_velocity_y"], None)),
-        ("particle_velocity_z", (vel_units, ["particle_velocity_z"], None)),
+        ("particle_velocity_x", (vel_units, [], None)),
+        ("particle_velocity_y", (vel_units, [], None)),
+        ("particle_velocity_z", (vel_units, [], None)),
         ("creation_time", ("code_time", [], None)),
         ("dynamical_time", ("code_time", [], None)),
         ("metallicity_fraction", ("code_metallicity", [], None)),
         ("metallicity", ("", [], None)),
         ("particle_type", ("", [], None)),
         ("particle_index", ("", [], None)),
-        ("particle_mass", ("code_mass", ["particle_mass"], None)),
+        ("particle_mass", ("code_mass", [], None)),
         ("GridID", ("", [], None)),
         ("identifier", ("", ["particle_index"], None)),
         ("level", ("", [], None)),
@@ -140,7 +137,7 @@ class EnzoFieldInfo(FieldInfoContainer):
                        ("enzo", "%s_Density" % species))
 
     def setup_species_fields(self):
-        species_names = [fn.rsplit("_Density")[0] for ft, fn in 
+        species_names = [fn.rsplit("_Density")[0] for ft, fn in
                          self.field_list if fn.endswith("_Density")]
         species_names = [sp for sp in species_names
                          if sp in known_species_names]
@@ -152,7 +149,7 @@ class EnzoFieldInfo(FieldInfoContainer):
         for sp in species_names:
             self.add_species_field(sp)
             self.species_names.append(known_species_names[sp])
-        add_nuclei_density_fields(self, "gas")
+        self.species_names.sort()  # bb #1059
 
     def setup_fluid_fields(self):
         # Now we conditionally load a few other things.
@@ -207,11 +204,13 @@ class EnzoFieldInfo(FieldInfoContainer):
                 units="code_velocity**2")
             # Subtract off B-field energy
             def _sub_b(field, data):
-                return data[te_name] - 0.5*(
-                    data["x-velocity"]**2.0
-                    + data["y-velocity"]**2.0
-                    + data["z-velocity"]**2.0 ) \
-                    - data["MagneticEnergy"]/data["Density"]
+                ret = data[te_name] - 0.5*data["velocity_x"]**2.0
+                if data.ds.dimensionality > 1:
+                    ret -= 0.5*data["velocity_y"]**2.0
+                if data.ds.dimensionality > 2:
+                    ret -= 0.5*data["velocity_z"]**2.0
+                ret -= data["magnetic_energy"]/data["density"]
+                return ret
             self.add_field(
                 ("gas", "thermal_energy"),
                 function=_sub_b, units = "erg/g")
@@ -221,14 +220,23 @@ class EnzoFieldInfo(FieldInfoContainer):
                 units = "code_velocity**2")
             self.alias(("gas", "total_energy"), ("enzo", te_name))
             def _tot_minus_kin(field, data):
-                return data[te_name] - 0.5*(
-                    data["x-velocity"]**2.0
-                    + data["y-velocity"]**2.0
-                    + data["z-velocity"]**2.0 )
+                ret = data[te_name] - 0.5*data["velocity_x"]**2.0
+                if data.ds.dimensionality > 1:
+                    ret -= 0.5*data["velocity_y"]**2.0
+                if data.ds.dimensionality > 2:
+                    ret -= 0.5*data["velocity_z"]**2.0
+                return ret
             self.add_field(
                 ("gas", "thermal_energy"),
                 function = _tot_minus_kin,
                 units = "erg/g")
+        if multi_species == 0 and 'Mu' in params:
+            def _number_density(field, data):
+                return data['gas', 'density']/(mp*params['Mu'])
+            self.add_field(
+                ("gas", "number_density"),
+                function = _number_density,
+                units="1/cm**3")
 
     def setup_particle_fields(self, ptype):
 
