@@ -90,3 +90,22 @@ def test_ecp():
     ds = data_dir_load(ecp)
     # Now we test our species fields
     yield check_color_conservation(ds)
+
+@requires_file(enzotiny)
+def test_units_override():
+    for test in units_override_check(enzotiny):
+        yield test
+
+@requires_ds(ecp, big_data=True)
+def test_nuclei_density_fields():
+    ds = data_dir_load(ecp)
+    ad = ds.all_data()
+    yield assert_array_equal, ad["H_nuclei_density"], \
+      (ad["H_number_density"] + ad["H_p1_number_density"])
+    yield assert_array_equal, ad["He_nuclei_density"], \
+      (ad["He_number_density"] + ad["He_p1_number_density"] +
+       ad["He_p2_number_density"])
+
+@requires_file(enzotiny)
+def test_EnzoDataset():
+    assert isinstance(data_dir_load(enzotiny), EnzoDataset)
