@@ -67,8 +67,8 @@ class FLASHFieldInfo(FieldInfoContainer):
         ("ye", ("", [], "Y_e")),
         ("magp", (pres_units, [], None)),
         ("divb", ("code_magnetic*code_length", [], None)),
-        ("game", ("", [], r"\gamma_e\/\rm{(ratio\/of\/specific\/heats)}")),
-        ("gamc", ("", [], r"\gamma_c\/\rm{(ratio\/of\/specific\/heats)}")),
+        ("game", ("", [], r"\gamma_e\ \rm{(ratio\ of\ specific\ heats)}")),
+        ("gamc", ("", [], r"\gamma_c\ \rm{(ratio\ of\ specific\ heats)}")),
         ("flam", ("", [], None)),
         ("absr", ("", [], "Absorption Coefficient")),
         ("emis", ("", [], "Emissivity")),
