@@ -36,6 +36,7 @@ class IOHandlerGDFHDF5(BaseIOHandler):
     _data_string = 'data:datatype=0'
 
     def _read_fluid_selection(self, chunks, selector, fields, size):
+        from sys import version
         rv = {}
         chunks = list(chunks)
 
@@ -74,7 +75,10 @@ class IOHandlerGDFHDF5(BaseIOHandler):
                 if grid.filename is None:
                     continue
                 if fid is None:
-                    fid = h5py.h5f.open(grid.filename, h5py.h5f.ACC_RDONLY)
+                    if version < '3':
+                        fid = h5py.h5f.open(grid.filename,h5py.h5f.ACC_RDONLY)
+                    else:
+                        fid = h5py.h5f.open(bytes(grid.filename,'utf-8'),h5py.h5f.ACC_RDONLY)
                 if self.ds.field_ordering == 1:
                     # check the dtype instead
                     data = np.empty(grid.ActiveDimensions[::-1],
@@ -86,7 +90,10 @@ class IOHandlerGDFHDF5(BaseIOHandler):
                                                 dtype="float64")
                 for field in fields:
                     ftype, fname = field
-                    dg = h5py.h5d.open(fid, _field_dname(grid.id, fname))
+                    if version < '3':
+                        dg = h5py.h5d.open(fid, _field_dname(grid.id, fname))
+                    else:
+                        dg = h5py.h5d.open(fid, bytes(_field_dname(grid.id, fname),'utf-8'))
                     dg.read(h5py.h5s.ALL, h5py.h5s.ALL, data)
                     # caches
                     nd = grid.select(selector, data_view, rv[field], ind)
