@@ -58,7 +58,7 @@ from yt.fields.field_info_container import \
     FieldInfoContainer, NullFunc
 from yt.utilities.lib.misc_utilities import \
     get_box_grids_level
-from yt.utilities.lib.GridTree import \
+from yt.geometry.grid_container import \
     GridTree, \
     MatchPointsToGrids
 from yt.utilities.decompose import \
@@ -69,8 +69,9 @@ from yt.utilities.definitions import \
 from yt.utilities.flagging_methods import \
     FlaggingGrid
 from yt.data_objects.unstructured_mesh import \
-           SemiStructuredMesh
-
+           SemiStructuredMesh, \
+           UnstructuredMesh
+from yt.extern.six import string_types, iteritems
 from .fields import \
     StreamFieldInfo
 
@@ -150,7 +151,7 @@ class StreamHandler(object):
 
     def get_particle_type(self, field) :
 
-        if self.particle_types.has_key(field) :
+        if field in self.particle_types :
             return self.particle_types[field]
         else :
             return False
@@ -181,7 +182,7 @@ class StreamHierarchy(GridIndex):
         mylog.debug("Copying reverse tree")
         self.grids = []
         # We enumerate, so it's 0-indexed id and 1-indexed pid
-        for id in xrange(self.num_grids):
+        for id in range(self.num_grids):
             self.grids.append(self.grid(id, self))
             self.grids[id].Level = self.grid_levels[id, 0]
         parent_ids = self.stream_handler.parent_ids
@@ -269,7 +270,7 @@ class StreamHierarchy(GridIndex):
             self.stream_handler.particle_types[key] = particle_types[key]
 
         for i, grid in enumerate(self.grids) :
-            if data[i].has_key("number_of_particles") :
+            if "number_of_particles" in data[i] :
                 grid.NumberOfParticles = data[i].pop("number_of_particles")
             for fname in data[i]:
                 if fname in grid.field_data:
@@ -292,8 +293,8 @@ class StreamDataset(Dataset):
     _field_info_class = StreamFieldInfo
     _dataset_type = 'stream'
 
-    def __init__(self, stream_handler, storage_filename = None,
-                 geometry = "cartesian"):
+    def __init__(self, stream_handler, storage_filename=None,
+                 geometry="cartesian"):
         #if parameter_override is None: parameter_override = {}
         #self._parameter_override = parameter_override
         #if conversion_override is None: conversion_override = {}
@@ -340,7 +341,7 @@ class StreamDataset(Dataset):
         attrs = ('length_unit', 'mass_unit', 'time_unit', 'velocity_unit', 'magnetic_unit')
         cgs_units = ('cm', 'g', 's', 'cm/s', 'gauss')
         for unit, attr, cgs_unit in zip(base_units, attrs, cgs_units):
-            if isinstance(unit, basestring):
+            if isinstance(unit, string_types):
                 uq = self.quan(1.0, unit)
             elif isinstance(unit, numeric_type):
                 uq = self.quan(unit, cgs_unit)
@@ -364,8 +365,10 @@ class StreamDictFieldHandler(dict):
     _additional_fields = ()
 
     @property
-    def all_fields(self): 
-        fields = list(self._additional_fields) + self[0].keys()
+    def all_fields(self):
+        self_fields = chain.from_iterable(s.keys() for s in self.values())
+        self_fields = list(set(self_fields))
+        fields = list(self._additional_fields) + self_fields
         fields = list(set(fields))
         return fields
 
@@ -414,22 +417,26 @@ def assign_particle_data(ds, pdata) :
     
     if len(ds.stream_handler.fields) > 1:
 
-        try:
-            x, y, z = (pdata["io","particle_position_%s" % ax] for ax in 'xyz')
-        except KeyError:
-            raise KeyError("Cannot decompose particle data without position fields!")
+        if ("io", "particle_position_x") in pdata:
+            x, y, z = (pdata["io", "particle_position_%s" % ax] for ax in 'xyz')
+        elif ("io", "particle_position") in pdata:
+            x, y, z = pdata["io", "particle_position"].T
+        else:
+            raise KeyError(
+                "Cannot decompose particle data without position fields!")
         num_grids = len(ds.stream_handler.fields)
         parent_ids = ds.stream_handler.parent_ids
         num_children = np.zeros(num_grids, dtype='int64')
         # We're going to do this the slow way
         mask = np.empty(num_grids, dtype="bool")
-        for i in xrange(num_grids):
+        for i in range(num_grids):
             np.equal(parent_ids, i, mask)
             num_children[i] = mask.sum()
         levels = ds.stream_handler.levels.astype("int64").ravel()
         grid_tree = GridTree(num_grids, 
                              ds.stream_handler.left_edges,
                              ds.stream_handler.right_edges,
+                             ds.stream_handler.dimensions,
                              ds.stream_handler.parent_ids,
                              levels, num_children)
 
@@ -447,7 +454,7 @@ def assign_particle_data(ds, pdata) :
     
         pdata.pop("number_of_particles", None) 
         grid_pdata = []
-        for i, pcount in enumerate(particle_grid_count) :
+        for i, pcount in enumerate(particle_grid_count):
             grid = {}
             grid["number_of_particles"] = pcount
             start = particle_indices[i]
@@ -471,24 +478,26 @@ def unitify_data(data):
             field_units[k] = v.units
             new_data[k] = v.copy().d
         data = new_data
-    elif all([isinstance(val, np.ndarray) for val in data.values()]):
-        field_units = {field:'' for field in data.keys()}
-    elif all([(len(val) == 2) for val in data.values()]):
+    elif all([((not isinstance(val, np.ndarray)) and (len(val) == 2))
+             for val in data.values()]):
         new_data, field_units = {}, {}
         for field in data:
             try:
-                assert isinstance(field, (basestring, tuple)), \
+                assert isinstance(field, (string_types, tuple)), \
                   "Field name is not a string!"
                 assert isinstance(data[field][0], np.ndarray), \
                   "Field data is not an ndarray!"
-                assert isinstance(data[field][1], basestring), \
+                assert isinstance(data[field][1], string_types), \
                   "Unit specification is not a string!"
                 field_units[field] = data[field][1]
                 new_data[field] = data[field][0]
-            except AssertionError, e:
+            except AssertionError as e:
                 raise RuntimeError("The data dict appears to be invalid.\n" +
                                    str(e))
         data = new_data
+    elif all([iterable(val) for val in data.values()]):
+        field_units = {field:'' for field in data.keys()}
+        data = dict((field, np.asarray(val)) for field, val in iteritems(data))
     else:
         raise RuntimeError("The data dict appears to be invalid. "
                            "The data dictionary must map from field "
@@ -563,8 +572,13 @@ def load_uniform_grid(data, domain_dimensions, length_unit=None, bbox=None,
     periodicity : tuple of booleans
         Determines whether the data will be treated as periodic along
         each axis
-    geometry : string
-        "cartesian", "cylindrical" or "polar"
+    geometry : string or tuple
+        "cartesian", "cylindrical", "polar", "spherical", "geographic" or
+        "spectral_cube".  Optionally, a tuple can be provided to specify the
+        axis ordering -- for instance, to specify that the axis ordering should
+        be z, x, y, this would be: ("cartesian", ("z", "x", "y")).  The same
+        can be done for other coordinates, for instance: 
+        ("spherical", ("theta", "phi", "r")).
 
     Examples
     --------
@@ -602,12 +616,12 @@ def load_uniform_grid(data, domain_dimensions, length_unit=None, bbox=None,
     field_units, data = unitify_data(data)
     sfh = StreamDictFieldHandler()
 
-    if number_of_particles > 0 :
+    if number_of_particles > 0:
         particle_types = set_particle_types(data)
         pdata = {} # Used much further below.
         pdata["number_of_particles"] = number_of_particles
-        for key in data.keys() :
-            if len(data[key].shape) == 1 :
+        for key in list(data.keys()):
+            if len(data[key].shape) == 1 or key[0] == 'io':
                 if not isinstance(key, tuple):
                     field = ("io", key)
                     mylog.debug("Reassigning '%s' to '%s'", key, field)
@@ -615,10 +629,10 @@ def load_uniform_grid(data, domain_dimensions, length_unit=None, bbox=None,
                     field = key
                 sfh._additional_fields += (field,)
                 pdata[field] = data.pop(key)
-    else :
+    else:
         particle_types = {}
     update_field_names(data)
-    
+
     if nprocs > 1:
         temp = {}
         new_data = {}
@@ -678,26 +692,31 @@ def load_uniform_grid(data, domain_dimensions, length_unit=None, bbox=None,
 
     sds = StreamDataset(handler, geometry = geometry)
 
+    check_fields = [("io", "particle_position_x"), ("io", "particle_position")]
+
     # Now figure out where the particles go
-    if number_of_particles > 0 :
-        if ("io", "particle_position_x") not in pdata:
+    if number_of_particles > 0:
+        if all(f not in pdata for f in check_fields):
             pdata_ftype = {}
             for f in [k for k in sorted(pdata)]:
-                if not hasattr(pdata[f], "shape"): continue
+                if not hasattr(pdata[f], "shape"):
+                    continue
+                if f == 'number_of_particles':
+                    continue
                 mylog.debug("Reassigning '%s' to ('io','%s')", f, f)
                 pdata_ftype["io",f] = pdata.pop(f)
             pdata_ftype.update(pdata)
             pdata = pdata_ftype
         # This will update the stream handler too
         assign_particle_data(sds, pdata)
-    
+
     return sds
 
 def load_amr_grids(grid_data, domain_dimensions,
                    field_units=None, bbox=None, sim_time=0.0, length_unit=None,
                    mass_unit=None, time_unit=None, velocity_unit=None,
                    magnetic_unit=None, periodicity=(True, True, True),
-                   geometry = "cartesian"):
+                   geometry = "cartesian", refine_by=2):
     r"""Load a set of grids of data into yt as a
     :class:`~yt.frontends.stream.data_structures.StreamHandler`.
     This should allow a sequence of grids of varying resolution of data to be
@@ -745,8 +764,15 @@ def load_amr_grids(grid_data, domain_dimensions,
     periodicity : tuple of booleans
         Determines whether the data will be treated as periodic along
         each axis
-    geometry : string
-        "cartesian", "cylindrical" or "polar"
+    geometry : string or tuple
+        "cartesian", "cylindrical", "polar", "spherical", "geographic" or
+        "spectral_cube".  Optionally, a tuple can be provided to specify the
+        axis ordering -- for instance, to specify that the axis ordering should
+        be z, x, y, this would be: ("cartesian", ("z", "x", "y")).  The same
+        can be done for other coordinates, for instance: 
+        ("spherical", ("theta", "phi", "r")).
+    refine_by : integer
+        Specifies the refinement ratio between levels.  Defaults to 2.
 
     Examples
     --------
@@ -763,12 +789,13 @@ def load_amr_grids(grid_data, domain_dimensions,
     ...          dimensions = [32, 32, 32],
     ...          number_of_particles = 0)
     ... ]
-    ... 
-    >>> for g in grid_data:
-    ...     g["Density"] = np.random.random(g["dimensions"]) * 2**g["level"]
     ...
-    >>> units = dict(Density='g/cm**3')
-    >>> ds = load_amr_grids(grid_data, [32, 32, 32], 1.0)
+    >>> for g in grid_data:
+    ...     g["density"] = np.random.random(g["dimensions"]) * 2**g["level"]
+    ...
+    >>> units = dict(density='g/cm**3')
+    >>> ds = load_amr_grids(grid_data, [32, 32, 32], field_units=units,
+    ...                     length_unit=1.0)
     """
 
     domain_dimensions = np.array(domain_dimensions)
@@ -789,7 +816,7 @@ def load_amr_grids(grid_data, domain_dimensions,
         grid_right_edges[i,:] = g.pop("right_edge")
         grid_dimensions[i,:] = g.pop("dimensions")
         grid_levels[i,:] = g.pop("level")
-        if g.has_key("number_of_particles") :
+        if "number_of_particles" in g :
             number_of_particles[i,:] = g.pop("number_of_particles")  
         update_field_names(g)
         sfh[i] = g
@@ -822,6 +849,11 @@ def load_amr_grids(grid_data, domain_dimensions,
     if magnetic_unit is None:
         magnetic_unit = 'code_magnetic'
 
+    particle_types = {}
+
+    for grid in grid_data:
+        particle_types.update(set_particle_types(grid))
+
     handler = StreamHandler(
         grid_left_edges,
         grid_right_edges,
@@ -833,13 +865,13 @@ def load_amr_grids(grid_data, domain_dimensions,
         sfh,
         field_units,
         (length_unit, mass_unit, time_unit, velocity_unit, magnetic_unit),
-        particle_types=set_particle_types(grid_data[0])
+        particle_types=particle_types
     )
 
     handler.name = "AMRGridData"
     handler.domain_left_edge = domain_left_edge
     handler.domain_right_edge = domain_right_edge
-    handler.refine_by = 2
+    handler.refine_by = refine_by
     handler.dimensionality = 3
     handler.domain_dimensions = domain_dimensions
     handler.simulation_time = sim_time
@@ -1018,7 +1050,7 @@ def load_particles(data, length_unit = None, bbox=None,
     magnetic_unit : float
         Conversion factor from simulation magnetic units to gauss
     bbox : array_like (xdim:zdim, LE:RE), optional
-        Size of computational domain in units sim_unit_to_cm
+        Size of computational domain in units of the length_unit
     sim_time : float, optional
         The simulation time in seconds
     periodicity : tuple of booleans
@@ -1116,6 +1148,55 @@ _cis = np.fromiter(chain.from_iterable(product([0,1], [0,1], [0,1])),
 _cis.shape = (8, 3)
 
 def hexahedral_connectivity(xgrid, ygrid, zgrid):
+    r"""Define the cell coordinates and cell neighbors of a hexahedral mesh
+    for a semistructured grid. Used to specify the connectivity and
+    coordinates parameters used in
+    :function:`~yt.frontends.stream.data_structures.load_hexahedral_mesh`.
+
+    Parameters
+    ----------
+    xgrid : array_like
+       x-coordinates of boundaries of the hexahedral cells. Should be a
+       one-dimensional array.
+    ygrid : array_like
+       y-coordinates of boundaries of the hexahedral cells. Should be a
+       one-dimensional array.
+    zgrid : array_like
+       z-coordinates of boundaries of the hexahedral cells. Should be a
+       one-dimensional array.
+
+    Returns
+    -------
+    coords : array_like
+        The list of (x,y,z) coordinates of the vertices of the mesh.
+        Is of size (M,3) where M is the number of vertices.
+    connectivity : array_like
+        For each hexahedron h in the mesh, gives the index of each of h's
+        neighbors. Is of size (N,8), where N is the number of hexahedra.
+
+    Examples
+    --------
+
+    >>> xgrid = np.array([-1,-0.25,0,0.25,1])
+    >>> coords, conn = hexahedral_connectivity(xgrid,xgrid,xgrid)
+    >>> coords
+    array([[-1.  , -1.  , -1.  ],
+           [-1.  , -1.  , -0.25],
+           [-1.  , -1.  ,  0.  ],
+           ..., 
+           [ 1.  ,  1.  ,  0.  ],
+           [ 1.  ,  1.  ,  0.25],
+           [ 1.  ,  1.  ,  1.  ]])
+
+    >>> conn
+    array([[  0,   1,   5,   6,  25,  26,  30,  31],
+           [  1,   2,   6,   7,  26,  27,  31,  32],
+           [  2,   3,   7,   8,  27,  28,  32,  33],
+           ...,
+           [ 91,  92,  96,  97, 116, 117, 121, 122],
+           [ 92,  93,  97,  98, 117, 118, 122, 123],
+           [ 93,  94,  98,  99, 118, 119, 123, 124]])
+    """
     nx = len(xgrid)
     ny = len(ygrid)
     nz = len(zgrid)
@@ -1185,16 +1266,16 @@ def load_hexahedral_mesh(data, connectivity, coordinates,
     ----------
     data : dict
         This is a dict of numpy arrays, where the keys are the field names.
-        There must only be one.
+        There must only be one. Note that the data in the numpy arrays should
+        define the cell-averaged value for of the quantity in in the hexahedral
+        cell.
     connectivity : array_like
         This should be of size (N,8) where N is the number of zones.
     coordinates : array_like
         This should be of size (M,3) where M is the number of vertices
         indicated in the connectivity matrix.
-    sim_unit_to_cm : float
-        Conversion factor from simulation units to centimeters
     bbox : array_like (xdim:zdim, LE:RE), optional
-        Size of computational domain in units sim_unit_to_cm
+        Size of computational domain in units of the length unit.
     sim_time : float, optional
         The simulation time in seconds
     mass_unit : string
@@ -1208,8 +1289,13 @@ def load_hexahedral_mesh(data, connectivity, coordinates,
     periodicity : tuple of booleans
         Determines whether the data will be treated as periodic along
         each axis
-    geometry : string
-        "cartesian", "cylindrical" or "polar"
+    geometry : string or tuple
+        "cartesian", "cylindrical", "polar", "spherical", "geographic" or
+        "spectral_cube".  Optionally, a tuple can be provided to specify the
+        axis ordering -- for instance, to specify that the axis ordering should
+        be z, x, y, this would be: ("cartesian", ("z", "x", "y")).  The same
+        can be done for other coordinates, for instance: 
+        ("spherical", ("theta", "phi", "r")).
 
     """
 
@@ -1229,6 +1315,14 @@ def load_hexahedral_mesh(data, connectivity, coordinates,
     sfh.update({'connectivity': connectivity,
                 'coordinates': coordinates,
                 0: data})
+    # Simple check for axis length correctness
+    if len(data) > 0:
+        fn = list(sorted(data))[0]
+        array_values = data[fn]
+        if array_values.size != connectivity.shape[0]:
+            mylog.error("Dimensions of array must be one fewer than the" +
+                        " coordinate set.")
+            raise RuntimeError
     grid_left_edges = domain_left_edge
     grid_right_edges = domain_right_edge
     grid_dimensions = domain_dimensions.reshape(nprocs,3).astype("int32")
@@ -1488,3 +1582,39 @@ def load_octree(octree_mask, data,
     sds.over_refine_factor = over_refine_factor
 
     return sds
+
+class StreamUnstructuredMesh(UnstructuredMesh):
+    _index_offset = 0
+
+    def __init__(self, *args, **kwargs):
+        super(StreamUnstructuredMesh, self).__init__(*args, **kwargs)
+        self._connectivity_length = self.connectivity_indices.shape[1]
+
+
+class StreamUnstructuredIndex(UnstructuredIndex):
+
+    def __init__(self, ds, dataset_type = None):
+        self.stream_handler = ds.stream_handler
+        super(StreamUnstructuredIndex, self).__init__(ds, dataset_type)
+
+    def _initialize_mesh(self):
+        coords = ensure_list(self.stream_handler.fields.pop("coordinates"))
+        connec = ensure_list(self.stream_handler.fields.pop("connectivity"))
+        self.meshes = [StreamUnstructuredMesh(
+          i, self.index_filename, c1, c2, self)
+          for i, (c1, c2) in enumerate(zip(coords, connec))]
+
+    def _setup_data_io(self):
+        if self.stream_handler.io is not None:
+            self.io = self.stream_handler.io
+        else:
+            self.io = io_registry[self.dataset_type](self.ds)
+
+    def _detect_output_fields(self):
+        self.field_list = list(set(self.stream_handler.get_fields()))
+
+class StreamUnstructuredMeshDataset(StreamDataset):
+    _index_class = StreamUnstructuredMesh
+    _field_info_class = StreamFieldInfo
+    _dataset_type = "stream_unstructured"
+
