@@ -4,6 +4,7 @@ Useful functions.  If non-original, see function for citation.
 
 
 """
+from __future__ import print_function
 
 #-----------------------------------------------------------------------------
 # Copyright (c) 2013, yt Development Team.
@@ -13,7 +14,7 @@ Useful functions.  If non-original, see function for citation.
 # The full license is in the file COPYING.txt, distributed with this software.
 #-----------------------------------------------------------------------------
 
-import time, types, signal, inspect, traceback, sys, pdb, os, re
+from yt.extern.six import string_types
 import time, types, signal, inspect, traceback, sys, pdb, os, re
 import contextlib
 import warnings, struct, subprocess
@@ -22,7 +23,7 @@ from distutils.version import LooseVersion
 from math import floor, ceil
 from numbers import Number as numeric_type
 
-from yt.extern.six.moves import builtins
+from yt.extern.six.moves import builtins, urllib
 from yt.utilities.exceptions import *
 from yt.utilities.logger import ytLogger as mylog
 import yt.extern.progressbar as pb
@@ -50,7 +51,7 @@ def ensure_list(obj):
     """
     if obj is None:
         return [obj]
-    if not isinstance(obj, types.ListType):
+    if not isinstance(obj, list):
         return [obj]
     return obj
 
@@ -64,7 +65,7 @@ def ensure_numpy_array(obj):
             return np.array([obj])
         # We cast to ndarray to catch ndarray subclasses
         return np.array(obj)
-    elif isinstance(obj, (types.ListType, types.TupleType)):
+    elif isinstance(obj, (list, tuple)):
         return np.asarray(obj)
     else:
         return np.asarray([obj])
@@ -75,9 +76,9 @@ def ensure_tuple(obj):
     scalar, list, or array arguments specified by a user in a context where
     we assume a tuple internally
     """
-    if isinstance(obj, types.TupleType):
+    if isinstance(obj, tuple):
         return obj
-    elif isinstance(obj, (types.ListType, np.ndarray)):
+    elif isinstance(obj, (list, np.ndarray)):
         return tuple(obj)
     else:
         return (obj,)
@@ -99,6 +100,25 @@ def just_one(obj):
         return obj[0]
     return obj
 
+
+def compare_dicts(dict1, dict2):
+    if not set(dict1) <= set(dict2):
+        return False
+    for key in dict1.keys():
+        if dict1[key] is not None and dict2[key] is not None:
+            if isinstance(dict1[key], dict):
+                if compare_dicts(dict1[key], dict2[key]):
+                    continue
+                else:
+                    return False
+            try:
+                comparison = (dict1[key] == dict2[key]).all()
+            except AttributeError:
+                comparison = (dict1[key] == dict2[key])
+            if not comparison:
+                return False
+    return True
+
 # Taken from
 # http://www.goldb.org/goldblog/2008/02/06/PythonConvertSecsIntoHumanReadableTimeStringHHMMSS.aspx
 def humanize_time(secs):
@@ -147,7 +167,7 @@ def time_execution(func):
         t1 = time.time()
         res = func(*arg, **kw)
         t2 = time.time()
-        mylog.debug('%s took %0.3f s', func.func_name, (t2-t1))
+        mylog.debug('%s took %0.3f s', func.__name__, (t2-t1))
         return res
     from yt.config import ytcfg
     if ytcfg.getboolean("yt","timefunctions") == True:
@@ -216,7 +236,7 @@ def deprecate(func):
     @wraps(func)
     def run_func(*args, **kwargs):
         warnings.warn("%s has been deprecated and may be removed without notice!" \
-                % func.func_name, DeprecationWarning, stacklevel=2)
+                % func.__name__, DeprecationWarning, stacklevel=2)
         func(*args, **kwargs)
     return run_func
 
@@ -385,7 +405,7 @@ def is_root():
     cfg_option = "__topcomm_parallel_rank"
     if not ytcfg.getboolean("yt","__parallel"):
         return True
-    if ytcfg.getint("yt", cfg_option) > 0: 
+    if ytcfg.getint("yt", cfg_option) > 0:
         return False
     return True
 
@@ -395,7 +415,7 @@ def is_root():
 #
 
 def signal_print_traceback(signo, frame):
-    print traceback.print_stack(frame)
+    print(traceback.print_stack(frame))
 
 def signal_problem(signo, frame):
     raise RuntimeError()
@@ -409,54 +429,44 @@ def paste_traceback(exc_type, exc, tb):
     Should only be used in sys.excepthook.
     """
     sys.__excepthook__(exc_type, exc, tb)
-    from yt.extern.six.moves import StringIO
-    import xmlrpclib
-    p = xmlrpclib.ServerProxy(
+    from yt.extern.six.moves import StringIO, xmlrpc_client
+    p = xmlrpc_client.ServerProxy(
             "http://paste.yt-project.org/xmlrpc/",
             allow_none=True)
     s = StringIO()
     traceback.print_exception(exc_type, exc, tb, file=s)
     s = s.getvalue()
     ret = p.pastes.newPaste('pytb', s, None, '', '', True)
-    print
-    print "Traceback pasted to http://paste.yt-project.org/show/%s" % (ret)
-    print
+    print()
+    print("Traceback pasted to http://paste.yt-project.org/show/%s" % (ret))
+    print()
 
 def paste_traceback_detailed(exc_type, exc, tb):
     """
     This is a traceback handler that knows how to paste to the pastebin.
     Should only be used in sys.excepthook.
     """
-    import xmlrpclib, cgitb
-    from yt.extern.six.moves import StringIO
+    import cgitb
+    from yt.extern.six.moves import StringIO, xmlrpc_client
     s = StringIO()
     handler = cgitb.Hook(format="text", file = s)
     handler(exc_type, exc, tb)
     s = s.getvalue()
-    print s
-    p = xmlrpclib.ServerProxy(
+    print(s)
+    p = xmlrpc_client.ServerProxy(
             "http://paste.yt-project.org/xmlrpc/",
             allow_none=True)
     ret = p.pastes.newPaste('text', s, None, '', '', True)
-    print
-    print "Traceback pasted to http://paste.yt-project.org/show/%s" % (ret)
-    print
-
-def traceback_writer_hook(file_suffix = ""):
-    def write_to_file(exc_type, exc, tb):
-        sys.__excepthook__(exc_type, exc, tb)
-        fn = "yt_traceback%s" % file_suffix
-        f = open(fn, "w")
-        traceback.print_exception(exc_type, exc, tb, file=f)
-        print "Wrote traceback to %s" % fn
-    return write_to_file
+    print()
+    print("Traceback pasted to http://paste.yt-project.org/show/%s" % (ret))
+    print()
 
 _ss = "fURbBUUBE0cLXgETJnZgJRMXVhVGUQpQAUBuehQMUhJWRFFRAV1ERAtBXw1dAxMLXT4zXBFfABNN\nC0ZEXw1YUURHCxMXVlFERwxWCQw=\n"
 def _rdbeta(key):
     import itertools, base64
     enc_s = base64.decodestring(_ss)
     dec_s = ''.join([ chr(ord(a) ^ ord(b)) for a, b in zip(enc_s, itertools.cycle(key)) ])
-    print dec_s
+    print(dec_s)
 
 #
 # Some exceptions
@@ -469,50 +479,51 @@ class YTEmptyClass(object):
     pass
 
 def update_hg(path, skip_rebuild = False):
-    from mercurial import hg, ui, commands
+    try:
+        import hglib
+    except ImportError:
+        print("Updating requires python-hglib to be installed.")
+        print("Try: pip install python-hglib")
+        return -1
     f = open(os.path.join(path, "yt_updater.log"), "a")
-    u = ui.ui()
-    u.pushbuffer()
-    config_fn = os.path.join(path, ".hg", "hgrc")
-    print "Reading configuration from ", config_fn
-    u.readconfig(config_fn)
-    repo = hg.repository(u, path)
-    commands.pull(u, repo)
-    f.write(u.popbuffer())
-    f.write("\n\n")
-    u.pushbuffer()
-    commands.identify(u, repo)
-    if "+" in u.popbuffer():
-        print "Can't rebuild modules by myself."
-        print "You will have to do this yourself.  Here's a sample commands:"
-        print
-        print "    $ cd %s" % (path)
-        print "    $ hg up"
-        print "    $ %s setup.py develop" % (sys.executable)
+    repo = hglib.open(path)
+    repo.pull()
+    ident = repo.identify().decode("utf-8")
+    if "+" in ident:
+        print("Can't rebuild modules by myself.")
+        print("You will have to do this yourself.  Here's a sample commands:")
+        print("")
+        print("    $ cd %s" % (path))
+        print("    $ hg up")
+        print("    $ %s setup.py develop" % (sys.executable))
         return 1
-    print "Updating the repository"
+    print("Updating the repository")
     f.write("Updating the repository\n\n")
-    commands.update(u, repo, check=True)
+    repo.update(check=True)
+    f.write("Updated from %s to %s\n\n" % (ident, repo.identify()))
     if skip_rebuild: return
     f.write("Rebuilding modules\n\n")
     p = subprocess.Popen([sys.executable, "setup.py", "build_ext", "-i"], cwd=path,
                         stdout = subprocess.PIPE, stderr = subprocess.STDOUT)
     stdout, stderr = p.communicate()
-    f.write(stdout)
+    f.write(stdout.decode('utf-8'))
     f.write("\n\n")
     if p.returncode:
-        print "BROKEN: See %s" % (os.path.join(path, "yt_updater.log"))
+        print("BROKEN: See %s" % (os.path.join(path, "yt_updater.log")))
         sys.exit(1)
     f.write("Successful!\n")
-    print "Updated successfully."
+    print("Updated successfully.")
 
 def get_hg_version(path):
-    from mercurial import hg, ui, commands
-    u = ui.ui()
-    u.pushbuffer()
-    repo = hg.repository(u, path)
-    commands.identify(u, repo)
-    return u.popbuffer()
+    try:
+        import hglib
+    except ImportError:
+        print("Updating and precise version information requires ")
+        print("python-hglib to be installed.")
+        print("Try: pip install python-hglib")
+        return -1
+    repo = hglib.open(path)
+    return repo.identify()
 
 def get_yt_version():
     try:
@@ -547,57 +558,54 @@ def get_script_contents():
     return contents
 
 def download_file(url, filename):
-    import urllib
-    class MyURLopener(urllib.FancyURLopener):
+    class MyURLopener(urllib.request.FancyURLopener):
         def http_error_default(self, url, fp, errcode, errmsg, headers):
-            raise RuntimeError, \
-              "Attempt to download file from %s failed with error %s: %s." % \
-              (url, errcode, errmsg)
+            raise RuntimeError("Attempt to download file from %s failed with error %s: %s." % \
+              (url, errcode, errmsg))
     fn, h = MyURLopener().retrieve(url, filename)
     return fn
 
 # This code snippet is modified from Georg Brandl
 def bb_apicall(endpoint, data, use_pass = True):
-    import urllib, urllib2
     uri = 'https://api.bitbucket.org/1.0/%s/' % endpoint
     # since bitbucket doesn't return the required WWW-Authenticate header when
     # making a request without Authorization, we cannot use the standard urllib2
     # auth handlers; we have to add the requisite header from the start
     if data is not None:
-        data = urllib.urlencode(data)
-    req = urllib2.Request(uri, data)
+        data = urllib.parse.urlencode(data)
+    req = urllib.request.Request(uri, data)
     if use_pass:
         username = raw_input("Bitbucket Username? ")
         password = getpass.getpass()
         upw = '%s:%s' % (username, password)
         req.add_header('Authorization', 'Basic %s' % base64.b64encode(upw).strip())
-    return urllib2.urlopen(req).read()
+    return urllib.request.urlopen(req).read()
 
 def get_yt_supp():
     supp_path = os.path.join(os.environ["YT_DEST"], "src",
                              "yt-supplemental")
     # Now we check that the supplemental repository is checked out.
     if not os.path.isdir(supp_path):
-        print
-        print "*** The yt-supplemental repository is not checked ***"
-        print "*** out.  I can do this for you, but because this ***"
-        print "*** is a delicate act, I require you to respond   ***"
-        print "*** to the prompt with the word 'yes'.            ***"
-        print
+        print()
+        print("*** The yt-supplemental repository is not checked ***")
+        print("*** out.  I can do this for you, but because this ***")
+        print("*** is a delicate act, I require you to respond   ***")
+        print("*** to the prompt with the word 'yes'.            ***")
+        print()
         response = raw_input("Do you want me to try to check it out? ")
         if response != "yes":
-            print
-            print "Okay, I understand.  You can check it out yourself."
-            print "This command will do it:"
-            print
-            print "$ hg clone http://hg.yt-project.org/yt-supplemental/ ",
-            print "%s" % (supp_path)
-            print
+            print()
+            print("Okay, I understand.  You can check it out yourself.")
+            print("This command will do it:")
+            print()
+            print("$ hg clone http://bitbucket.org/yt_analysis/yt-supplemental/ ", end=' ')
+            print("%s" % (supp_path))
+            print()
             sys.exit(1)
         rv = commands.clone(uu,
-                "http://hg.yt-project.org/yt-supplemental/", supp_path)
+                "http://bitbucket.org/yt_analysis/yt-supplemental/", supp_path)
         if rv:
-            print "Something has gone wrong.  Quitting."
+            print("Something has gone wrong.  Quitting.")
             sys.exit(1)
     # Now we think we have our supplemental repository.
     return supp_path
@@ -615,7 +623,7 @@ def fix_length(length, ds=None):
     if isinstance(length, numeric_type):
         return YTArray(length, 'code_length', registry=registry)
     length_valid_tuple = isinstance(length, (list, tuple)) and len(length) == 2
-    unit_is_string = isinstance(length[1], types.StringTypes)
+    unit_is_string = isinstance(length[1], str)
     if length_valid_tuple and unit_is_string:
         return YTArray(*length, registry=registry)
     else:
@@ -681,7 +689,7 @@ def ensure_dir(path):
 def validate_width_tuple(width):
     if not iterable(width) or len(width) != 2:
         raise YTInvalidWidthError("width (%s) is not a two element tuple" % width)
-    if not isinstance(width[0], numeric_type) and isinstance(width[1], basestring):
+    if not isinstance(width[0], numeric_type) and isinstance(width[1], string_types):
         msg = "width (%s) is invalid. " % str(width)
         msg += "Valid widths look like this: (12, 'au')"
         raise YTInvalidWidthError(msg)
@@ -727,7 +735,7 @@ def memory_checker(interval = 15, dest = None):
 
         def run(self):
             while not self.event.wait(self.interval):
-                print >> dest, "MEMORY: %0.3e gb" % (get_memory_usage()/1024.)
+                print("MEMORY: %0.3e gb" % (get_memory_usage()/1024.), file=dest)
 
     e = threading.Event()
     mem_check = MemoryChecker(e, interval)
@@ -748,9 +756,10 @@ def deprecated_class(cls):
             SyntaxWarning, stacklevel=2)
         return cls(*args, **kwargs)
     return _func
-    
+
 def enable_plugins():
     import yt
+    from yt.fields.my_plugin_fields import my_plugins_fields
     from yt.config import ytcfg
     my_plugin_name = ytcfg.get("yt","pluginfilename")
     # We assume that it is with respect to the $HOME/.yt directory
@@ -760,7 +769,11 @@ def enable_plugins():
         _fn = os.path.expanduser("~/.yt/%s" % my_plugin_name)
     if os.path.isfile(_fn):
         mylog.info("Loading plugins from %s", _fn)
-        execfile(_fn, yt.__dict__)
+        execdict = yt.__dict__.copy()
+        execdict['add_field'] = my_plugins_fields.add_field
+        with open(_fn) as f:
+            code = compile(f.read(), _fn, 'exec')
+            exec(code, execdict)
 
 def fix_unitary(u):
     if u == '1':
