@@ -22,7 +22,7 @@ from .coordinate_handler import \
     cylindrical_to_cartesian, \
     cartesian_to_cylindrical
 import yt.visualization._MPL as _MPL
-from yt.utilities.lib.misc_utilities import \
+from yt.utilities.lib.pixelization_routines import \
     pixelize_cylinder
 #
 # Cylindrical fields
@@ -30,9 +30,12 @@ from yt.utilities.lib.misc_utilities import \
 
 class CylindricalCoordinateHandler(CoordinateHandler):
 
-    def __init__(self, ds, ordering = 'rzt'):
-        if ordering != 'rzt': raise NotImplementedError
-        super(CylindricalCoordinateHandler, self).__init__(ds)
+    def __init__(self, ds, ordering = ('r', 'z', 'theta')):
+        super(CylindricalCoordinateHandler, self).__init__(ds, ordering)
+        self.image_units = {}
+        self.image_units[self.axis_id['r']] = ("rad", None)
+        self.image_units[self.axis_id['theta']] = (None, None)
+        self.image_units[self.axis_id['z']] = (None, None)
 
     def setup_fields(self, registry):
         # return the fields for r, z, theta
@@ -40,7 +43,7 @@ class CylindricalCoordinateHandler(CoordinateHandler):
         registry.add_field(("index", "dy"), function=_unknown_coord)
         registry.add_field(("index", "x"), function=_unknown_coord)
         registry.add_field(("index", "y"), function=_unknown_coord)
-        f1, f2 = _get_coord_fields(0)
+        f1, f2 = _get_coord_fields(self.axis_id['r'])
         registry.add_field(("index", "dr"), function = f1,
                            display_field = False,
                            units = "code_length")
@@ -125,18 +128,8 @@ class CylindricalCoordinateHandler(CoordinateHandler):
                                  size, data_source[field], bounds)
         return buff
 
-    axis_name = { 0  : 'r',  1  : 'z',  2  : 'theta',
-                 'r' : 'r', 'z' : 'z', 'theta' : 'theta',
-                 'R' : 'r', 'Z' : 'z', 'Theta' : 'theta'}
-
-    axis_id = { 'r' : 0, 'z' : 1, 'theta' : 2,
-                 0  : 0,  1  : 1,  2  : 2}
-
-    x_axis = { 'r' : 2, 'z' : 0, 'theta' : 0,
-                0  : 2,  1  : 0,  2  : 0}
-
-    y_axis = { 'r' : 1, 'z' : 2, 'theta' : 1,
-                0  : 1,  1  : 2,  2  : 1}
+    _x_pairs = (('r', 'theta'), ('z', 'r'), ('theta', 'r'))
+    _y_pairs = (('r', 'z'), ('z', 'theta'), ('theta', 'z'))
 
     _image_axis_name = None
 
@@ -150,7 +143,7 @@ class CylindricalCoordinateHandler(CoordinateHandler):
         rv = {self.axis_id['r']: ('theta', 'z'),
               self.axis_id['z']: ('x', 'y'),
               self.axis_id['theta']: ('r', 'z')}
-        for i in rv.keys():
+        for i in list(rv.keys()):
             rv[self.axis_name[i]] = rv[i]
             rv[self.axis_name[i].upper()] = rv[i]
         self._image_axis_name = rv
