@@ -15,7 +15,7 @@ Geometry container base class.
 #-----------------------------------------------------------------------------
 
 import os
-import cPickle
+from yt.extern.six.moves import cPickle
 import weakref
 import h5py
 import numpy as np
@@ -252,7 +252,6 @@ class Index(ParallelAnalysisInterface):
             chunk_size)
         return fields_to_return, fields_to_generate
 
-
     def _chunk(self, dobj, chunking_style, ngz = 0, **kwargs):
         # A chunk is either None or (grids, size)
         if dobj._current_chunk is None:
@@ -269,7 +268,7 @@ class Index(ParallelAnalysisInterface):
             raise NotImplementedError
 
 def cached_property(func):
-    n = '_%s' % func.func_name
+    n = '_%s' % func.__name__
     def cached_func(self):
         if self._cache and getattr(self, n, None) is not None:
             return getattr(self, n)
@@ -286,20 +285,21 @@ def cached_property(func):
 class YTDataChunk(object):
 
     def __init__(self, dobj, chunk_type, objs, data_size = None,
-                 field_type = None, cache = False):
+                 field_type = None, cache = False, fast_index = None):
         self.dobj = dobj
         self.chunk_type = chunk_type
         self.objs = objs
         self.data_size = data_size
         self._field_type = field_type
         self._cache = cache
+        self._fast_index = fast_index
 
     def _accumulate_values(self, method):
         # We call this generically.  It's somewhat slower, since we're doing
         # costly getattr functions, but this allows us to generalize.
         mname = "select_%s" % method
         arrs = []
-        for obj in self.objs:
+        for obj in self._fast_index or self.objs:
             f = getattr(obj, mname)
             arrs.append(f(self.dobj))
         if method == "dtcoords":
@@ -312,12 +312,18 @@ class YTDataChunk(object):
 
     @cached_property
     def fcoords(self):
+        if self._fast_index is not None:
+            ci = self._fast_index.select_fcoords(
+                self.dobj.selector, self.data_size)
+            ci = YTArray(ci, input_units = "code_length",
+                         registry = self.dobj.ds.unit_registry)
+            return ci
         ci = np.empty((self.data_size, 3), dtype='float64')
         ci = YTArray(ci, input_units = "code_length",
                      registry = self.dobj.ds.unit_registry)
         if self.data_size == 0: return ci
         ind = 0
-        for obj in self.objs:
+        for obj in self._fast_index or self.objs:
             c = obj.select_fcoords(self.dobj)
             if c.shape[0] == 0: continue
             ci[ind:ind+c.shape[0], :] = c
@@ -326,10 +332,14 @@ class YTDataChunk(object):
 
     @cached_property
     def icoords(self):
+        if self._fast_index is not None:
+            ci = self._fast_index.select_icoords(
+                self.dobj.selector, self.data_size)
+            return ci
         ci = np.empty((self.data_size, 3), dtype='int64')
         if self.data_size == 0: return ci
         ind = 0
-        for obj in self.objs:
+        for obj in self._fast_index or self.objs:
             c = obj.select_icoords(self.dobj)
             if c.shape[0] == 0: continue
             ci[ind:ind+c.shape[0], :] = c
@@ -338,12 +348,18 @@ class YTDataChunk(object):
 
     @cached_property
     def fwidth(self):
+        if self._fast_index is not None:
+            ci = self._fast_index.select_fwidth(
+                self.dobj.selector, self.data_size)
+            ci = YTArray(ci, input_units = "code_length",
+                         registry = self.dobj.ds.unit_registry)
+            return ci
         ci = np.empty((self.data_size, 3), dtype='float64')
         ci = YTArray(ci, input_units = "code_length",
                      registry = self.dobj.ds.unit_registry)
         if self.data_size == 0: return ci
         ind = 0
-        for obj in self.objs:
+        for obj in self._fast_index or self.objs:
             c = obj.select_fwidth(self.dobj)
             if c.shape[0] == 0: continue
             ci[ind:ind+c.shape[0], :] = c
@@ -352,10 +368,14 @@ class YTDataChunk(object):
 
     @cached_property
     def ires(self):
+        if self._fast_index is not None:
+            ci = self._fast_index.select_ires(
+                self.dobj.selector, self.data_size)
+            return ci
         ci = np.empty(self.data_size, dtype='int64')
         if self.data_size == 0: return ci
         ind = 0
-        for obj in self.objs:
+        for obj in self._fast_index or self.objs:
             c = obj.select_ires(self.dobj)
             if c.shape == 0: continue
             ci[ind:ind+c.size] = c
@@ -374,7 +394,7 @@ class YTDataChunk(object):
         self._tcoords = ct # Se this for tcoords
         if self.data_size == 0: return cdt
         ind = 0
-        for obj in self.objs:
+        for obj in self._fast_index or self.objs:
             gdt, gt = obj.select_tcoords(self.dobj)
             if gt.shape == 0: continue
             ct[ind:ind+gt.size] = gt
@@ -399,11 +419,14 @@ class ChunkDataCache(object):
     def __iter__(self):
         return self
     
+    def __next__(self):
+        return self.next()
+        
     def next(self):
         if len(self.queue) == 0:
             for i in range(self.max_length):
                 try:
-                    self.queue.append(self.base_iter.next())
+                    self.queue.append(next(self.base_iter))
                 except StopIteration:
                     break
             # If it's still zero ...
