@@ -8,6 +8,19 @@ def configuration(parent_package='',top_path=None):
     from numpy.distutils.misc_util import Configuration
     config = Configuration('geometry',parent_package,top_path)
     config.add_subpackage('coordinates')
+    config.add_extension("grid_visitors", 
+                ["yt/geometry/grid_visitors.pyx"],
+                include_dirs=["yt/utilities/lib/"],
+                libraries=["m"],
+                depends=["yt/utilities/lib/fp_utils.pxd",
+                         "yt/geometry/grid_visitors.pxd"])
+    config.add_extension("grid_container", 
+                ["yt/geometry/grid_container.pyx"],
+                include_dirs=["yt/utilities/lib/"],
+                libraries=["m"],
+                depends=["yt/utilities/lib/fp_utils.pxd",
+                         "yt/geometry/grid_container.pxd",
+                         "yt/geometry/grid_visitors.pxd"])
     config.add_extension("oct_container", 
                 ["yt/geometry/oct_container.pyx"],
                 include_dirs=["yt/utilities/lib/"],
@@ -35,6 +48,9 @@ def configuration(parent_package='',top_path=None):
                 libraries=["m"],
                 depends=["yt/utilities/lib/fp_utils.pxd",
                          "yt/geometry/oct_container.pxd",
+                         "yt/geometry/oct_visitors.pxd",
+                         "yt/geometry/grid_container.pxd",
+                         "yt/geometry/grid_visitors.pxd",
                          "yt/geometry/selection_routines.pxd"])
     config.add_extension("particle_deposit", 
                 ["yt/geometry/particle_deposit.pyx"],
