@@ -9,9 +9,12 @@ def configuration(parent_package='', top_path=None):
     config = Configuration('yt', parent_package, top_path)
     config.add_subpackage('analysis_modules')
     config.add_subpackage('data_objects')
+    config.add_subpackage('fields')
     config.add_subpackage('extern')
     config.add_subpackage('frontends')
+    config.add_subpackage('geometry')
     config.add_subpackage('gui')
+    config.add_subpackage('units')
     config.add_subpackage('utilities')
     config.add_subpackage('visualization')
     config.make_config_py()
