@@ -15,7 +15,7 @@ Test ndarray subclass that handles symbolic units.
 # ----------------------------------------------------------------------------
 
 import copy
-import cPickle as pickle
+from yt.extern.six.moves import cPickle as pickle
 import itertools
 import numpy as np
 import operator
@@ -32,13 +32,19 @@ from numpy.testing import \
 from numpy import array
 from yt.units.yt_array import \
     YTArray, YTQuantity, \
-    unary_operators, binary_operators
+    unary_operators, binary_operators, \
+    uconcatenate, uintersect1d, \
+    uunion1d
 from yt.utilities.exceptions import \
     YTUnitOperationError, YTUfuncUnitError
-from yt.testing import fake_random_ds, requires_module
+from yt.testing import \
+    fake_random_ds, requires_module, \
+    assert_allclose_units
 from yt.funcs import fix_length
 from yt.units.unit_symbols import \
     cm, m, g
+from yt.utilities.physical_ratios import \
+    metallicity_sun
 
 def operate_and_compare(a, b, op, answer):
     # Test generator for YTArrays tests
@@ -264,11 +270,15 @@ def test_division():
     a3 = [4*cm, 5*cm, 6*cm]
     answer1 = YTArray([0.25, 0.4, 0.5])
     answer2 = YTArray([4, 2.5, 2])
+    if "div" in dir(operator):
+        op = operator.div
+    else:
+        op = operator.truediv
 
-    yield operate_and_compare, a1, a2, operator.div, answer1
-    yield operate_and_compare, a2, a1, operator.div, answer2
-    yield operate_and_compare, a1, a3, operator.div, answer1
-    yield operate_and_compare, a3, a1, operator.div, answer2
+    yield operate_and_compare, a1, a2, op, answer1
+    yield operate_and_compare, a2, a1, op, answer2
+    yield operate_and_compare, a1, a3, op, answer1
+    yield operate_and_compare, a3, a1, op, answer2
     yield operate_and_compare, a1, a2, np.divide, answer1
     yield operate_and_compare, a2, a1, np.divide, answer2
     yield operate_and_compare, a1, a3, np.divide, answer1
@@ -283,10 +293,10 @@ def test_division():
     answer3 = YTArray([0.25, 0.4, 0.5], 'cm/m')
     answer4 = YTArray([4.0, 2.5, 2.0], 'm/cm')
 
-    yield operate_and_compare, a1, a2, operator.div, answer1
-    yield operate_and_compare, a2, a1, operator.div, answer2
-    yield operate_and_compare, a1, a3, operator.div, answer1
-    yield operate_and_compare, a3, a1, operator.div, answer2
+    yield operate_and_compare, a1, a2, op, answer1
+    yield operate_and_compare, a2, a1, op, answer2
+    yield operate_and_compare, a1, a3, op, answer1
+    yield operate_and_compare, a3, a1, op, answer2
     yield operate_and_compare, a1, a2, np.divide, answer3
     yield operate_and_compare, a2, a1, np.divide, answer4
     yield operate_and_compare, a1, a3, np.divide, answer3
@@ -299,10 +309,10 @@ def test_division():
     answer1 = YTArray([0.25, 0.4, 0.5], 'cm/g')
     answer2 = YTArray([4, 2.5, 2], 'g/cm')
 
-    yield operate_and_compare, a1, a2, operator.div, answer1
-    yield operate_and_compare, a2, a1, operator.div, answer2
-    yield operate_and_compare, a1, a3, operator.div, answer1
-    yield operate_and_compare, a3, a1, operator.div, answer2
+    yield operate_and_compare, a1, a2, op, answer1
+    yield operate_and_compare, a2, a1, op, answer2
+    yield operate_and_compare, a1, a3, op, answer1
+    yield operate_and_compare, a3, a1, op, answer2
     yield operate_and_compare, a1, a2, np.divide, answer1
     yield operate_and_compare, a2, a1, np.divide, answer2
     yield operate_and_compare, a1, a3, np.divide, answer1
@@ -315,10 +325,10 @@ def test_division():
     answer1 = YTArray([0.25, 0.4, 0.5], 'cm')
     answer2 = YTArray([4, 2.5, 2], '1/cm')
 
-    yield operate_and_compare, a1, a2, operator.div, answer1
-    yield operate_and_compare, a2, a1, operator.div, answer2
-    yield operate_and_compare, a1, a3, operator.div, answer1
-    yield operate_and_compare, a3, a1, operator.div, answer2
+    yield operate_and_compare, a1, a2, op, answer1
+    yield operate_and_compare, a2, a1, op, answer2
+    yield operate_and_compare, a1, a3, op, answer1
+    yield operate_and_compare, a3, a1, op, answer2
     yield operate_and_compare, a1, a2, np.divide, answer1
     yield operate_and_compare, a2, a1, np.divide, answer2
     yield operate_and_compare, a1, a3, np.divide, answer1
@@ -331,10 +341,10 @@ def test_division():
     answer1 = YTArray([0.25, 0.4, 0.5])
     answer2 = YTArray([4, 2.5, 2])
 
-    yield operate_and_compare, a1, a2, operator.div, answer1
-    yield operate_and_compare, a2, a1, operator.div, answer2
-    yield operate_and_compare, a1, a3, operator.div, answer1
-    yield operate_and_compare, a3, a1, operator.div, answer2
+    yield operate_and_compare, a1, a2, op, answer1
+    yield operate_and_compare, a2, a1, op, answer2
+    yield operate_and_compare, a1, a3, op, answer1
+    yield operate_and_compare, a3, a1, op, answer2
     yield operate_and_compare, a1, a3, np.divide, answer1
     yield operate_and_compare, a3, a1, np.divide, answer2
     yield operate_and_compare, a1, a3, np.divide, answer1
@@ -469,21 +479,21 @@ def test_temperature_conversions():
 
     km = YTQuantity(1, 'km')
     balmy = YTQuantity(300, 'K')
-    balmy_F = YTQuantity(80.33, 'F')
-    balmy_C = YTQuantity(26.85, 'C')
+    balmy_F = YTQuantity(80.33, 'degF')
+    balmy_C = YTQuantity(26.85, 'degC')
     balmy_R = YTQuantity(540, 'R')
 
-    assert_array_almost_equal(balmy.in_units('F'), balmy_F)
-    assert_array_almost_equal(balmy.in_units('C'), balmy_C)
+    assert_array_almost_equal(balmy.in_units('degF'), balmy_F)
+    assert_array_almost_equal(balmy.in_units('degC'), balmy_C)
     assert_array_almost_equal(balmy.in_units('R'), balmy_R)
 
     balmy_view = balmy.ndarray_view()
 
-    balmy.convert_to_units('F')
+    balmy.convert_to_units('degF')
     yield assert_true, balmy_view.base is balmy.base
     yield assert_array_almost_equal, np.array(balmy), np.array(balmy_F)
 
-    balmy.convert_to_units('C')
+    balmy.convert_to_units('degC')
     yield assert_true, balmy_view.base is balmy.base
     yield assert_array_almost_equal, np.array(balmy), np.array(balmy_C)
 
@@ -491,13 +501,13 @@ def test_temperature_conversions():
     yield assert_true, balmy_view.base is balmy.base
     yield assert_array_almost_equal, np.array(balmy), np.array(balmy_R)
 
-    balmy.convert_to_units('F')
+    balmy.convert_to_units('degF')
     yield assert_true, balmy_view.base is balmy.base
     yield assert_array_almost_equal, np.array(balmy), np.array(balmy_F)
 
     yield assert_raises, InvalidUnitOperation, np.multiply, balmy, km
 
-    # Does CGS convergion from F to K work?
+    # Does CGS conversion from F to K work?
     yield assert_array_almost_equal, balmy.in_cgs(), YTQuantity(300, 'K')
 
 
@@ -543,9 +553,6 @@ def test_selecting():
     # .base points to the original array for a numpy view.  If it is not a
     # view, .base is None.
     yield assert_true, a_slice.base is a
-    yield assert_true, a_fancy_index.base is None
-    yield assert_true, a_array_fancy_index.base is None
-    yield assert_true, a_boolean_index.base is None
 
 
 def test_fix_length():
@@ -576,8 +583,8 @@ def test_ytarray_pickle():
         yield assert_equal, data.units, loaded_data.units
         yield assert_array_equal, array(data.in_cgs()), \
             array(loaded_data.in_cgs())
-        yield assert_equal, float(data.units.cgs_value), \
-            float(loaded_data.units.cgs_value)
+        yield assert_equal, float(data.units.base_value), \
+            float(loaded_data.units.base_value)
 
 
 def test_copy():
@@ -625,7 +632,7 @@ def unary_ufunc_comparison(ufunc, a):
         # In-place copies do not drop units.
         assert_true(hasattr(out, 'units'))
         assert_true(not hasattr(ret, 'units'))
-    elif ufunc in (np.absolute, np.conjugate, np.floor, np.ceil,
+    elif ufunc in (np.absolute, np.fabs, np.conjugate, np.floor, np.ceil,
                    np.trunc, np.negative):
         ret = ufunc(a, out=out)
 
@@ -692,7 +699,12 @@ def binary_ufunc_comparison(ufunc, a, b):
         assert_true(not isinstance(ret, YTArray) and
                     isinstance(ret, np.ndarray))
     assert_array_equal(ret, out)
-    assert_array_equal(ret, ufunc(np.array(a), np.array(b)))
+    if (ufunc in (np.divide, np.true_divide, np.arctan2) and
+        (a.units.dimensions == b.units.dimensions)):
+        assert_array_almost_equal(
+            np.array(ret), ufunc(np.array(a.in_cgs()), np.array(b.in_cgs())))
+    else:
+        assert_array_almost_equal(np.array(ret), ufunc(np.array(a), np.array(b)))
 
 
 def test_ufuncs():
@@ -773,8 +785,11 @@ def test_registry_association():
         assert_equal(id(c.units.registry), id(ds.unit_registry))
         assert_equal(id(d.units.registry), id(b.units.registry))
 
-    for op in [operator.add, operator.sub, operator.mul, operator.div,
-               operator.truediv]:
+    binary_ops = [operator.add, operator.sub, operator.mul, 
+                  operator.truediv]
+    if hasattr(operator, "div"):
+        binary_ops.append(operator.div)
+    for op in binary_ops:
         yield binary_op_registry_comparison, op
 
     for op in [operator.abs, operator.neg, operator.pos]:
@@ -821,7 +836,10 @@ def test_subclass():
         assert_isinstance(op(inst1, inst2), compare_class)
         assert_isinstance(op(inst2, inst1), compare_class)
 
-    for op in (operator.mul, operator.div, operator.truediv):
+    ops = [operator.mul, operator.truediv]
+    if hasattr(operator, "div"):
+        ops.append(operator.div)
+    for op in ops:
         for inst in (b, ytq, ndf, yta, nda, loq):
             yield op_comparison, op, a, inst, YTASubclass
 
@@ -856,3 +874,191 @@ def test_h5_io():
 
     os.chdir(curdir)
     shutil.rmtree(tmpdir)
+
+def test_equivalencies():
+    from yt.utilities.physical_constants import clight, mp, kboltz, hcgs, mh, me, \
+        mass_sun_cgs, G, stefan_boltzmann_constant_cgs
+    import yt.units as u
+
+    # Mass-energy
+
+    E = mp.to_equivalent("keV","mass_energy")
+    yield assert_equal, E, mp*clight*clight
+    yield assert_allclose_units, mp, E.to_equivalent("g", "mass_energy")
+
+    # Thermal
+
+    T = YTQuantity(1.0e8,"K")
+    E = T.to_equivalent("W*hr","thermal")
+    yield assert_equal, E, (kboltz*T).in_units("W*hr")
+    yield assert_allclose_units, T, E.to_equivalent("K", "thermal")
+
+    # Spectral
+
+    l = YTQuantity(4000.,"angstrom")
+    nu = l.to_equivalent("Hz","spectral")
+    yield assert_equal, nu, clight/l
+    E = hcgs*nu
+    l2 = E.to_equivalent("angstrom", "spectral")
+    yield assert_allclose_units, l, l2
+    nu2 = clight/l2.in_units("cm")
+    yield assert_allclose_units, nu, nu2
+    E2 = nu2.to_equivalent("keV", "spectral")
+    yield assert_allclose_units, E2, E.in_units("keV")
+
+    # Sound-speed
+
+    mu = 0.6
+    gg = 5./3.
+    c_s = T.to_equivalent("km/s","sound_speed")
+    yield assert_equal, c_s, np.sqrt(gg*kboltz*T/(mu*mh))
+    yield assert_allclose_units, T, c_s.to_equivalent("K","sound_speed")
+
+    mu = 0.5
+    gg = 4./3.
+    c_s = T.to_equivalent("km/s","sound_speed", mu=mu, gamma=gg)
+    yield assert_equal, c_s, np.sqrt(gg*kboltz*T/(mu*mh))
+    yield assert_allclose_units, T, c_s.to_equivalent("K","sound_speed",
+                                                    mu=mu, gamma=gg)
+
+    # Lorentz
+
+    v = 0.8*clight
+    g = v.to_equivalent("dimensionless","lorentz")
+    g2 = YTQuantity(1./np.sqrt(1.-0.8*0.8), "dimensionless")
+    yield assert_allclose_units, g, g2
+    v2 = g2.to_equivalent("mile/hr", "lorentz")
+    yield assert_allclose_units, v2, v.in_units("mile/hr")
+
+    # Schwarzschild
+
+    R = mass_sun_cgs.to_equivalent("kpc","schwarzschild")
+    yield assert_equal, R.in_cgs(), 2*G*mass_sun_cgs/(clight*clight)
+    yield assert_allclose_units, mass_sun_cgs, R.to_equivalent("g", "schwarzschild")
+
+    # Compton
+
+    l = me.to_equivalent("angstrom","compton")
+    yield assert_equal, l, hcgs/(me*clight)
+    yield assert_allclose_units, me, l.to_equivalent("g", "compton")
+
+    # Number density
+
+    rho = mp/u.cm**3
+
+    n = rho.to_equivalent("cm**-3","number_density")
+    yield assert_equal, n, rho/(mh*0.6)
+    yield assert_allclose_units, rho, n.to_equivalent("g/cm**3","number_density")
+
+    n = rho.to_equivalent("cm**-3","number_density", mu=0.75)
+    yield assert_equal, n, rho/(mh*0.75)
+    yield assert_allclose_units, rho, n.to_equivalent("g/cm**3","number_density", mu=0.75)
+
+    # Effective temperature
+
+    T = YTQuantity(1.0e4, "K")
+    F = T.to_equivalent("erg/s/cm**2","effective_temperature")
+    yield assert_equal, F, stefan_boltzmann_constant_cgs*T**4
+    yield assert_allclose_units, T, F.to_equivalent("K", "effective_temperature")
+
+def test_electromagnetic():
+    from yt.units.dimensions import charge_mks, pressure, current_cgs, \
+        magnetic_field_mks, magnetic_field_cgs, power
+    from yt.utilities.physical_constants import mu_0, qp
+    from yt.utilities.physical_ratios import speed_of_light_cm_per_s
+
+    # Various tests of SI and CGS electromagnetic units
+
+    qp_mks = qp.to_equivalent("C", "SI")
+    yield assert_equal, qp_mks.units.dimensions, charge_mks
+    yield assert_array_almost_equal, qp_mks.v, 10.0*qp.v/speed_of_light_cm_per_s
+
+    qp_cgs = qp_mks.to_equivalent("esu", "CGS")
+    yield assert_array_almost_equal, qp_cgs, qp
+    yield assert_equal, qp_cgs.units.dimensions, qp.units.dimensions
+    
+    qp_mks_k = qp.to_equivalent("kC", "SI")
+    yield assert_array_almost_equal, qp_mks_k.v, 1.0e-2*qp.v/speed_of_light_cm_per_s
+
+    B = YTQuantity(1.0, "T")
+    B_cgs = B.to_equivalent("gauss", "CGS")
+    yield assert_equal, B.units.dimensions, magnetic_field_mks
+    yield assert_equal, B_cgs.units.dimensions, magnetic_field_cgs
+    yield assert_array_almost_equal, B_cgs, YTQuantity(1.0e4, "gauss")
+
+    u_mks = B*B/(2*mu_0)
+    yield assert_equal, u_mks.units.dimensions, pressure
+    u_cgs = B_cgs*B_cgs/(8*np.pi)
+    yield assert_equal, u_cgs.units.dimensions, pressure
+    yield assert_array_almost_equal, u_mks.in_cgs(), u_cgs
+    
+    I = YTQuantity(1.0, "A")
+    I_cgs = I.to_equivalent("statA", "CGS")
+    yield assert_array_almost_equal, I_cgs, YTQuantity(0.1*speed_of_light_cm_per_s, "statA")
+    yield assert_array_almost_equal, I_cgs.to_equivalent("mA", "SI"), I.in_units("mA")
+    yield assert_equal, I_cgs.units.dimensions, current_cgs
+    
+    R = YTQuantity(1.0, "ohm")
+    R_cgs = R.to_equivalent("statohm", "CGS")
+    P_mks = I*I*R
+    P_cgs = I_cgs*I_cgs*R_cgs
+    yield assert_equal, P_mks.units.dimensions, power
+    yield assert_equal, P_cgs.units.dimensions, power
+    yield assert_array_almost_equal, P_cgs.in_cgs(), P_mks.in_cgs()
+    yield assert_array_almost_equal, P_cgs.in_mks(), YTQuantity(1.0, "W")
+    
+    V = YTQuantity(1.0, "statV")
+    V_mks = V.to_equivalent("V", "SI")
+    yield assert_array_almost_equal, V_mks.v, 1.0e8*V.v/speed_of_light_cm_per_s
+
+def test_ytarray_coercion():
+    a = YTArray([1, 2, 3], 'cm')
+    q = YTQuantity(3, 'cm')
+    na = np.array([1, 2, 3])
+
+    assert_isinstance(a*q, YTArray)
+    assert_isinstance(q*na, YTArray)
+    assert_isinstance(q*3, YTQuantity)
+    assert_isinstance(q*np.float64(3), YTQuantity)
+    assert_isinstance(q*np.array(3), YTQuantity)
+
+def test_numpy_wrappers():
+    a1 = YTArray([1, 2, 3], 'cm')
+    a2 = YTArray([2, 3, 4, 5, 6], 'cm')
+    catenate_answer = [1, 2, 3, 2, 3, 4, 5, 6]
+    intersect_answer = [2, 3]
+    union_answer = [1, 2, 3, 4, 5, 6]
+
+    yield (assert_array_equal, YTArray(catenate_answer, 'cm'),
+           uconcatenate((a1, a2)))
+    yield assert_array_equal, catenate_answer, np.concatenate((a1, a2))
+
+    yield (assert_array_equal, YTArray(intersect_answer, 'cm'),
+           uintersect1d(a1, a2))
+    yield assert_array_equal, intersect_answer, np.intersect1d(a1, a2)
+
+    yield assert_array_equal, YTArray(union_answer, 'cm'), uunion1d(a1, a2)
+    yield assert_array_equal, union_answer, np.union1d(a1, a2)
+
+def test_dimensionless_conversion():
+    a = YTQuantity(1, 'Zsun')
+    b = a.in_units('Zsun')
+    a.convert_to_units('Zsun')
+    yield assert_true, a.units.base_value == metallicity_sun
+    yield assert_true, b.units.base_value == metallicity_sun
+
+def test_modified_unit_division():
+    ds1 = fake_random_ds(64)
+    ds2 = fake_random_ds(64)
+
+    # this mocks comoving coordinates without going through the trouble
+    # of setting up a fake cosmological dataset
+    ds1.unit_registry.modify('m', 50)
+
+    a = ds1.quan(3, 'm')
+    b = ds2.quan(3, 'm')
+
+    ret = a/b
+    yield assert_true, ret == 0.5
+    yield assert_true, ret.units.is_dimensionless
+    yield assert_true, ret.units.base_value == 1.0
