@@ -3,6 +3,8 @@ AMR kD-Tree Framework
 
 
 """
+from __future__ import print_function
+from __future__ import absolute_import
 
 #-----------------------------------------------------------------------------
 # Copyright (c) 2013, yt Development Team.
@@ -15,7 +17,7 @@ AMR kD-Tree Framework
 from yt.funcs import *
 import numpy as np
 import h5py
-from amr_kdtools import \
+from .amr_kdtools import \
         receive_and_reduce, send_to_parent, scatter_image
 
 from yt.utilities.lib.amr_kdtools import Node, add_pygrids, find_node, \
@@ -81,8 +83,6 @@ class Tree(object):
         for lvl in lvl_range:
             #grids = self.data_source.select_grids(lvl)
             grids = np.array([b for b, mask in self.data_source.blocks if b.Level == lvl])
-            gids = np.array([g.id for g in grids if g.Level == lvl],
-                            dtype="int64")
             if len(grids) == 0: continue
             self.add_grids(grids)
 
@@ -93,7 +93,6 @@ class Tree(object):
             grid = self.ds.index.grids[node.grid - self._id_offset]
             dds = grid.dds
             gle = grid.LeftEdge
-            gre = grid.RightEdge
             nle = self.ds.arr(get_left_edge(node), input_units="code_length")
             nre = self.ds.arr(get_right_edge(node), input_units="code_length")
             li = np.rint((nle-gle)/dds).astype('int32')
@@ -507,7 +506,7 @@ class AMRKDTree(ParallelAnalysisInterface):
                     les[0], res[0], gids[0], nids[0]) 
 
         N = nids.shape[0]
-        for i in xrange(N):
+        for i in range(N):
             n = self.get_node(nids[i])
             n.set_left_edge(les[i])
             n.set_right_edge(res[i])
@@ -546,6 +545,6 @@ if __name__ == "__main__":
     hv = AMRKDTree(ds)
     t2 = time()
 
-    print kd_sum_volume(hv.tree.trunk)
-    print kd_node_check(hv.tree.trunk)
-    print 'Time: %e seconds' % (t2-t1)
+    print(kd_sum_volume(hv.tree.trunk))
+    print(kd_node_check(hv.tree.trunk))
+    print('Time: %e seconds' % (t2-t1))
