@@ -21,7 +21,7 @@ SIEVE_PRIMES = \
 
 def decompose_to_primes(max_prime):
     """ Decompose number into the primes """
-    for prime in SIEVE_PRIMES(range(2, max_prime)):
+    for prime in SIEVE_PRIMES(list(range(2, max_prime))):
         if prime * prime > max_prime:
             break
         while max_prime % prime == 0:
@@ -34,13 +34,7 @@ def decompose_array(shape, psize, bbox):
     """ Calculate list of product(psize) subarrays of arr, along with their
         left and right edges
     """
-    grid_left_edges = np.empty([np.product(psize), 3], dtype=np.float64)
-    grid_right_edges = np.empty([np.product(psize), 3], dtype=np.float64)
-    n_d = shape
-    d_s = (bbox[:, 1] - bbox[:, 0]) / n_d
-    grid_left_edges, grid_right_edges, shapes, slices = \
-            split_array(bbox[:, 0], bbox[:, 1], shape, psize)
-    return grid_left_edges, grid_right_edges, shapes, slices
+    return split_array(bbox[:, 0], bbox[:, 1], shape, psize)
 
 
 def evaluate_domain_decomposition(n_d, pieces, ldom):
@@ -53,7 +47,7 @@ def evaluate_domain_decomposition(n_d, pieces, ldom):
     nd_arr = np.array(n_d, dtype=np.float64)[mask]
     bsize = int(np.sum(ldom[mask] / nd_arr * np.product(nd_arr)))
     load_balance = float(np.product(n_d)) / \
-        (float(pieces) * np.product((n_d - 1) / ldom + 1))
+        (float(pieces) * np.product((n_d - 1) // ldom + 1))
 
     # 0.25 is magic number
     quality = load_balance / (1 + 0.25 * (bsize / ideal_bsize - 1.0))
@@ -129,8 +123,8 @@ def split_array(gle, gre, shape, psize):
         for j in range(psize[1]):
             for k in range(psize[2]):
                 piece = np.array((i, j, k), dtype=np.int64)
-                lei = n_d * piece / psize
-                rei = n_d * (piece + np.ones(3, dtype=np.int64)) / psize
+                lei = n_d * piece // psize
+                rei = n_d * (piece + np.ones(3, dtype=np.int64)) // psize
                 lle = gle + lei*dds
                 lre = gle + rei*dds
                 left_edges.append(lle)
