@@ -1,14 +1,13 @@
+from __future__ import print_function
+from __future__ import absolute_import
 import os
 import weakref
 import numpy as np
 import h5py as h5
-from conversion_abc import *
+from .conversion_abc import *
 from glob import glob
 from collections import \
     defaultdict
-from string import \
-    strip, \
-    rstrip
 from stat import \
     ST_CTIME
 
@@ -89,7 +88,7 @@ class AthenaDistributedConverter(Converter):
         
 
 
-    def read_and_write_hierarchy(self,basename, ddn, gdf_name):
+    def read_and_write_index(self,basename, ddn, gdf_name):
         """ Read Athena legacy vtk file from multiple cpus """
         proc_names = glob(self.source_dir+'id*')
         #print 'Reading a dataset from %i Processor Files' % len(proc_names)
@@ -107,7 +106,7 @@ class AthenaDistributedConverter(Converter):
             else:
                 fn = self.source_dir+'id%i/'%i + basename + '-id%i'%i + '.%04i'%ddn + '.vtk'
 
-            print 'Reading file %s' % fn
+            print('Reading file %s' % fn)
             f = open(fn,'rb')
             grid = {}
             grid['read_field'] = None
@@ -132,8 +131,8 @@ class AthenaDistributedConverter(Converter):
                 grid['dimensions'] -= 1
                 grid['dimensions'][grid['dimensions']==0]=1
             if np.prod(grid['dimensions']) != grid['ncells']:
-                print 'product of dimensions %i not equal to number of cells %i' % \
-                      (np.prod(grid['dimensions']), grid['ncells'])
+                print('product of dimensions %i not equal to number of cells %i' % \
+                      (np.prod(grid['dimensions']), grid['ncells']))
                 raise TypeError
 
             # Append all hierachy info before reading this grid's data
@@ -234,8 +233,8 @@ class AthenaDistributedConverter(Converter):
                         grid_dims -= 1
                         grid_dims[grid_dims==0]=1
                     if np.prod(grid_dims) != grid_ncells:
-                        print 'product of dimensions %i not equal to number of cells %i' % \
-                              (np.prod(grid_dims), grid_ncells)
+                        print('product of dimensions %i not equal to number of cells %i' % \
+                              (np.prod(grid_dims), grid_ncells))
                         raise TypeError
                     break
                 else:
@@ -294,10 +293,10 @@ class AthenaDistributedConverter(Converter):
                 this_field.attrs['field_to_cgs'] = np.float64('1.0') # For Now
             
 
-    def convert(self, hierarchy=True, data=True):
+    def convert(self, index=True, data=True):
         self.handle = h5.File(self.outname, 'a')
-        if hierarchy:
-            self.read_and_write_hierarchy(self.basename, self.ddn ,self.outname)
+        if index:
+            self.read_and_write_index(self.basename, self.ddn ,self.outname)
         if data:
             self.read_and_write_data(self.basename, self.ddn ,self.outname)
         self.handle.close()
@@ -368,8 +367,8 @@ class AthenaConverter(Converter):
             if np.prod(grid['dimensions']) != grid['ncells']:
                 grid['dimensions'] -= 1
             if np.prod(grid['dimensions']) != grid['ncells']:
-                print 'product of dimensions %i not equal to number of cells %i' % \
-                      (np.prod(grid['dimensions']), grid['ncells'])
+                print('product of dimensions %i not equal to number of cells %i' % \
+                      (np.prod(grid['dimensions']), grid['ncells']))
                 raise TypeError
 
             if grid['read_type'] is 'scalar':
@@ -433,7 +432,7 @@ class AthenaConverter(Converter):
 
         ## --------- Done with top level nodes --------- ##
 
-        f.create_group('hierarchy')
+        f.create_group('index')
 
         ## --------- Store Grid Data --------- ##
 
