@@ -132,4 +132,26 @@ class scipy_imports:
             self._interpolate = interpolate
         return self._interpolate
 
-_scipy = scipy_imports()
\ No newline at end of file
+    _special = None
+    @property
+    def special(self):
+        if self._special is None:
+            try:
+                import scipy.special as special
+            except ImportError:
+                special = NotAModule(self._name)
+            self._special = special
+        return self._special
+    
+    _signal = None
+    @property
+    def signal(self):
+        if self._signal is None:
+            try:
+                import scipy.signal as signal
+            except ImportError:
+                signal = NotAModule(self._name)
+            self._signal = signal
+        return self._signal
+
+_scipy = scipy_imports()
