@@ -11,7 +11,7 @@ Writing PNGs
 #-----------------------------------------------------------------------------
 
 import matplotlib._png as _png
-import cStringIO
+from yt.extern.six.moves import cStringIO
 
 def write_png(buffer, filename, dpi=100):
     width = buffer.shape[1]
@@ -21,7 +21,7 @@ def write_png(buffer, filename, dpi=100):
 def write_png_to_string(buffer, dpi=100, gray=0):
     width = buffer.shape[1]
     height = buffer.shape[0]
-    fileobj = cStringIO.StringIO()
+    fileobj = cStringIO()
     _png.write_png(buffer, width, height, fileobj, dpi)
     png_str = fileobj.getvalue()
     fileobj.close()
