@@ -2,11 +2,11 @@ from yt.testing import *
 from yt.utilities.flagging_methods import flagging_method_registry
 
 def setup():
-    global pf
-    pf = fake_random_pf(64)
-    pf.h
+    global ds
+    ds = fake_random_ds(64)
+    ds.index
 
 def test_over_density():
     od_flag = flagging_method_registry["overdensity"](0.75) 
-    criterion = (pf.h.grids[0]["Density"] > 0.75)
-    assert( np.all( od_flag(pf.h.grids[0]) == criterion) )
+    criterion = (ds.index.grids[0]["density"] > 0.75)
+    assert( np.all( od_flag(ds.index.grids[0]) == criterion) )
