@@ -5,14 +5,13 @@ from yt.utilities.particle_generator import *
 from yt.frontends.stream.api import load_uniform_grid, refine_amr
 import yt.utilities.initial_conditions as ic
 import yt.utilities.flagging_methods as fm
-from IPython import embed
 from yt.units.yt_array import uconcatenate
 
 def setup() :
     pass
 
-def test_particle_generator() :
-    # First generate our pf
+def test_particle_generator():
+    # First generate our dataset
     domain_dims = (128, 128, 128)
     dens = np.zeros(domain_dims) + 0.1
     temp = 4.*np.ones(domain_dims)
