@@ -2,6 +2,8 @@
 
 
 """
+from __future__ import print_function
+from __future__ import absolute_import
 
 #-----------------------------------------------------------------------------
 # Copyright (c) 2013, yt Development Team.
@@ -18,11 +20,23 @@ import numpy as np
 
 from yt.funcs import *
 from yt.utilities.exceptions import YTNotInsideNotebook
-from color_maps import mcm
-import _colormap_data as cmd
+from .color_maps import mcm
+from . import _colormap_data as cmd
 import yt.utilities.lib.image_utilities as au
 import yt.utilities.png_writer as pw
 from yt.extern.six.moves import builtins
+import warnings
+try:
+    import palettable
+    has_palettable = True
+except:
+    has_palettable = False
+try:
+    import brewer2mpl
+    has_brewer = True
+except:
+    has_brewer = False
+
 
 def scale_image(image, mi=None, ma=None):
     r"""Scale an image ([NxNxM] where M = 1-4) to be uint8 and values scaled 
@@ -41,7 +55,7 @@ def scale_image(image, mi=None, ma=None):
     """
     if isinstance(image, np.ndarray) and image.dtype == np.uint8:
         return image
-    if isinstance(image, (types.TupleType, types.ListType)):
+    if isinstance(image, (tuple, list)):
         image, mi, ma = image
     if mi is None:
         mi = image.min()
@@ -156,7 +170,7 @@ def write_bitmap(bitmap_array, filename, max_val = None, transpose=False):
         bitmap_array = np.concatenate([bitmap_array.astype('uint8'),
                                        alpha_channel], axis=-1)
     if transpose:
-        bitmap_array = bitmap_array.swapaxes(0,1)
+        bitmap_array = bitmap_array.swapaxes(0,1).copy(order="C")
     if filename is not None:
         pw.write_png(bitmap_array, filename)
     else:
@@ -248,32 +262,55 @@ def map_to_colors(buff, cmap_name):
         lut = cmd.color_map_luts[cmap_name]
     except KeyError:
         try:
-            cmap = mcm.get_cmap(cmap_name)
+            # if cmap is tuple, then we're using palettable or brewer2mpl cmaps
+            if isinstance(cmap_name, tuple):
+                if has_palettable:
+                    bmap = palettable.colorbrewer.get_map(*cmap_name)
+                elif has_brewer:
+                    warnings.warn("Using brewer2mpl colormaps is deprecated. "
+                                  "Please install the successor to brewer2mpl, "
+                                  "palettable, with `pip install palettable`. "
+                                  "Colormap tuple names remain unchanged.")
+                    bmap = brewer2mpl.get_map(*cmap_name)
+                else:
+                    raise RuntimeError("Please install palettable to use colorbrewer colormaps")
+                cmap = bmap.get_mpl_colormap(N=cmap_name[2])
+            else:
+                cmap = mcm.get_cmap(cmap_name)
             dummy = cmap(0.0)
             lut = cmap._lut.T
         except ValueError:
-            print "Your color map was not found in either the extracted" +\
-                " colormap file or matplotlib colormaps"
+            print("Your color map was not found in either the extracted" +\
+                " colormap file or matplotlib colormaps")
             raise KeyError(cmap_name)
 
-    x = np.mgrid[0.0:1.0:lut[0].shape[0]*1j]
-    shape = buff.shape
-    mapped = np.dstack(
-            [(np.interp(buff, x, v)*255) for v in lut ]).astype("uint8")
+    if isinstance(cmap_name, tuple) and (has_palettable or has_brewer):
+        # If we are using the colorbrewer maps, don't interpolate
+        shape = buff.shape
+        # We add float_eps so that digitize doesn't go out of bounds
+        x = np.mgrid[0.0:1.0+np.finfo(np.float32).eps:lut[0].shape[0]*1j]
+        inds = np.digitize(buff.ravel(), x)
+        inds.shape = (shape[0], shape[1])
+        mapped = np.dstack([(v[inds]*255).astype('uint8') for v in lut])
+        del inds
+    else:
+        x = np.mgrid[0.0:1.0:lut[0].shape[0]*1j]
+        mapped = np.dstack(
+                [(np.interp(buff, x, v)*255).astype('uint8') for v in lut ])
     return mapped.copy("C")
 
 def strip_colormap_data(fn = "color_map_data.py",
             cmaps = ("jet", "algae", "hot", "gist_stern", "RdBu",
                      "kamae")):
     import pprint
-    import color_maps as rcm
+    from . import color_maps as rcm
     f = open(fn, "w")
     f.write("### Auto-generated colormap tables, taken from Matplotlib ###\n\n")
     f.write("from numpy import array\n")
     f.write("color_map_luts = {}\n\n\n")
     if cmaps is None: cmaps = rcm.ColorMaps
     for cmap_name in sorted(cmaps):
-        print "Stripping", cmap_name
+        print("Stripping", cmap_name)
         vals = rcm._extract_lookup_table(cmap_name)
         f.write("### %s ###\n\n" % (cmap_name))
         f.write("color_map_luts['%s'] = \\\n" % (cmap_name))
