@@ -4,6 +4,7 @@ This is a simple mechanism for interfacing with Profile and Phase plots
 
 
 """
+from __future__ import absolute_import
 
 #-----------------------------------------------------------------------------
 # Copyright (c) 2013, yt Development Team.
@@ -13,31 +14,34 @@ This is a simple mechanism for interfacing with Profile and Phase plots
 # The full license is in the file COPYING.txt, distributed with this software.
 #-----------------------------------------------------------------------------
 
-
-import __builtin__
+from yt.extern.six.moves import builtins
+from yt.extern.six.moves import zip as izip
+from yt.extern.six import string_types, iteritems
 import base64
 import os
-import types
 
 from functools import wraps
-from itertools import izip
 import matplotlib
 import numpy as np
-import cStringIO
+from io import BytesIO
 
 
 from .base_plot_types import ImagePlotMPL
+from yt.units.yt_array import YTArray
 from .plot_container import \
     ImagePlotContainer, \
-    log_transform, linear_transform
+    log_transform, linear_transform, get_log_minorticks, \
+    validate_plot, invalidate_plot
 from yt.data_objects.profiles import \
-    create_profile
+    create_profile, \
+    sanitize_field_tuple_keys
 from yt.utilities.exceptions import \
     YTNotInsideNotebook
 from yt.utilities.logger import ytLogger as mylog
-import _mpl_imports as mpl
+from . import _mpl_imports as mpl
 from yt.funcs import \
     ensure_list, \
+    iterable, \
     get_image_suffix, \
     get_ipython_api_version
 
@@ -57,15 +61,6 @@ def get_canvas(name):
         canvas_cls = mpl.FigureCanvasAgg
     return canvas_cls
 
-def invalidate_plot(f):
-    @wraps(f)
-    def newfunc(*args, **kwargs):
-        rv = f(*args, **kwargs)
-        args[0]._plot_valid = False
-        args[0]._setup_plots()
-        return rv
-    return newfunc
-
 class FigureContainer(dict):
     def __init__(self):
         super(FigureContainer, self).__init__()
@@ -100,7 +95,7 @@ def sanitize_label(label, nprofiles):
         raise RuntimeError("Number of labels must match number of profiles")
 
     for l in label:
-        if l is not None and not isinstance(l, basestring):
+        if l is not None and not isinstance(l, string_types):
             raise RuntimeError("All labels must be None or a string")
 
     return label
@@ -120,7 +115,7 @@ class ProfilePlot(object):
     
     Parameters
     ----------
-    data_source : AMR3DData Object
+    data_source : YTSelectionContainer Object
         The data object to be profiled, such as all_data, region, or 
         sphere.
     x_field : str
@@ -150,6 +145,15 @@ class ProfilePlot(object):
         A dictionary or list of dictionaries containing plot keyword 
         arguments.  For example, dict(color="red", linestyle=":").
         Default: None.
+    x_log : bool
+        If not None, whether the x_axis should be plotted with a logarithmic
+        scaling.
+        Default: None
+    y_log : dict
+        A dictionary containing field:boolean pairs, setting the logarithmic
+        property for that field. May be overridden after instantiation using 
+        set_log.
+        Default: None
 
     Examples
     --------
@@ -189,7 +193,6 @@ class ProfilePlot(object):
     """
     x_log = None
     y_log = None
-    z_log = None
     x_title = None
     y_title = None
     _plot_valid = False
@@ -197,56 +200,72 @@ class ProfilePlot(object):
     def __init__(self, data_source, x_field, y_fields,
                  weight_field="cell_mass", n_bins=64,
                  accumulation=False, fractional=False,
-                 label=None, plot_spec=None):
+                 label=None, plot_spec=None,
+                 x_log=None, y_log=None):
+
+        if x_log is None:
+            logs = None
+        else:
+            logs = {x_field:x_log}
 
         profiles = [create_profile(data_source, [x_field],
                                    n_bins=[n_bins],
                                    fields=ensure_list(y_fields),
                                    weight_field=weight_field,
                                    accumulation=accumulation,
-                                   fractional=fractional)]
+                                   fractional=fractional,
+                                   logs=logs)]
 
         if plot_spec is None:
             plot_spec = [dict() for p in profiles]
         if not isinstance(plot_spec, list):
             plot_spec = [plot_spec.copy() for p in profiles]
 
-        ProfilePlot._initialize_instance(self, profiles, label, plot_spec)
-
-    def save(self, name=None):
+        ProfilePlot._initialize_instance(self, profiles, label, plot_spec, y_log)
+        
+    @validate_plot
+    def save(self, name=None, suffix=None):
         r"""
-         Saves a 1d profile plot.
+        Saves a 1d profile plot.
 
-         Parameters
-         ----------
-         name : str
-             The output file keyword.
-
-         """
+        Parameters
+        ----------
+        name : str
+            The output file keyword.
+        suffix : string
+            Specify the image type by its suffix. If not specified, the output
+            type will be inferred from the filename. Defaults to PNG.
+        """
         if not self._plot_valid:
             self._setup_plots()
         unique = set(self.figures.values())
         if len(unique) < len(self.figures):
-            iters = izip(xrange(len(unique)), sorted(unique))
+            iters = izip(range(len(unique)), sorted(unique))
         else:
-            iters = self.figures.iteritems()
+            iters = iteritems(self.figures)
+        if not suffix:
+            suffix = "png"
+        suffix = ".%s" % suffix
         if name is None:
             if len(self.profiles) == 1:
                 prefix = self.profiles[0].ds
             else:
                 prefix = "Multi-data"
-            name = "%s.png" % prefix
-        suffix = get_image_suffix(name)
-        prefix = name[:name.rfind(suffix)]
+            name = "%s%s" % (prefix, suffix)
+        else:
+            sfx = get_image_suffix(name)
+            if sfx != '':
+                suffix = sfx
+                prefix = name[:name.rfind(suffix)]
+            else:
+                prefix = name
         xfn = self.profiles[0].x_field
-        if isinstance(xfn, types.TupleType):
+        if isinstance(xfn, tuple):
             xfn = xfn[1]
-        if not suffix:
-            suffix = ".png"
         canvas_cls = get_canvas(name)
         fns = []
         for uid, fig in iters:
-            if isinstance(uid, types.TupleType):
+            if isinstance(uid, tuple):
                 uid = uid[1]
             canvas = canvas_cls(fig)
             fns.append("%s_1d-Profile_%s_%s%s" % (prefix, xfn, uid, suffix))
@@ -254,6 +273,7 @@ class ProfilePlot(object):
             canvas.print_figure(fns[-1])
         return fns
 
+    @validate_plot
     def show(self):
         r"""This will send any existing plots to the IPython notebook.
 
@@ -273,7 +293,7 @@ class ProfilePlot(object):
         >>> pp.show()
 
         """
-        if "__IPYTHON__" in dir(__builtin__):
+        if "__IPYTHON__" in dir(builtins):
             api_version = get_ipython_api_version()
             if api_version in ('0.10', '0.11'):
                 self._send_zmq()
@@ -283,25 +303,29 @@ class ProfilePlot(object):
         else:
             raise YTNotInsideNotebook
 
+    @validate_plot
     def _repr_html_(self):
         """Return an html representation of the plot object. Will display as a
         png for each WindowPlotMPL instance in self.plots"""
         ret = ''
         unique = set(self.figures.values())
         if len(unique) < len(self.figures):
-            iters = izip(xrange(len(unique)), sorted(unique))
+            iters = izip(range(len(unique)), sorted(unique))
         else:
-            iters = self.figures.iteritems()
+            iters = iteritems(self.figures)
         for uid, fig in iters:
             canvas = mpl.FigureCanvasAgg(fig)
-            f = cStringIO.StringIO()
+            f = BytesIO()
             canvas.print_figure(f)
             f.seek(0)
-            img = base64.b64encode(f.read())
-            ret += '<img src="data:image/png;base64,%s"><br>' % img
+            img = base64.b64encode(f.read()).decode()
+            ret += r'<img style="max-width:100%%;max-height:100%%;" ' \
+                   r'src="data:image/png;base64,{0}"><br>'.format(img)
         return ret
 
     def _setup_plots(self):
+        if self._plot_valid is True:
+            return
         for f in self.axes:
             self.axes[f].cla()
         for i, profile in enumerate(self.profiles):
@@ -323,11 +347,15 @@ class ProfilePlot(object):
         self._plot_valid = True
 
     @classmethod
-    def _initialize_instance(cls, obj, profiles, labels, plot_specs):
-        obj.y_log = {}
-        obj.y_title = {}
-        obj.x_log = None
+    def _initialize_instance(cls, obj, profiles, labels, plot_specs, y_log):
         obj.profiles = ensure_list(profiles)
+        obj.x_log = None
+        obj.y_log = {}
+        if y_log is not None:
+            for field, log in y_log.items():
+                field, = obj.profiles[0].data_source._determine_fields([field])
+                obj.y_log[field] = log
+        obj.y_title = {}
         obj.label = sanitize_label(labels, len(obj.profiles))
         if plot_specs is None:
             plot_specs = [dict() for p in obj.profiles]
@@ -338,7 +366,7 @@ class ProfilePlot(object):
         return obj
 
     @classmethod
-    def from_profiles(cls, profiles, labels=None, plot_specs=None):
+    def from_profiles(cls, profiles, labels=None, plot_specs=None, y_log=None):
         r"""
         Instantiate a ProfilePlot object from a list of profiles
         created with :func:`~yt.data_objects.profiles.create_profile`.
@@ -384,7 +412,7 @@ class ProfilePlot(object):
         if plot_specs is not None and len(plot_specs) != len(profiles):
             raise RuntimeError("Profiles list and plot_specs list must be the same size.")
         obj = cls.__new__(cls)
-        return cls._initialize_instance(obj, profiles, labels, plot_specs)
+        return cls._initialize_instance(obj, profiles, labels, plot_specs, y_log)
 
     @invalidate_plot
     def set_line_property(self, property, value, index=None):
@@ -502,15 +530,20 @@ class ProfilePlot(object):
                 xma = xmax
             extrema = {p.x_field: ((xmi, str(p.x.units)), (xma, str(p.x.units)))}
             units = {p.x_field: str(p.x.units)}
+            if self.x_log is None:
+                logs = None
+            else:
+                logs = {p.x_field: self.x_log}
             for field in p.field_map.values():
                 units[field] = str(p.field_data[field].units)
             self.profiles[i] = \
                 create_profile(p.data_source, p.x_field,
-                               n_bins=len(p.x_bins)-2,
-                               fields=p.field_map.values(),
+                               n_bins=len(p.x_bins)-1,
+                               fields=list(p.field_map.values()),
                                weight_field=p.weight_field,
                                accumulation=p.accumulation,
                                fractional=p.fractional,
+                               logs=logs,
                                extrema=extrema, units=units)
         return self
 
@@ -577,16 +610,16 @@ class ProfilePlot(object):
         if isinstance(field, tuple): field = field[1]
         if field_name is None:
             field_name = r'$\rm{'+field+r'}$'
-            field_name = r'$\rm{'+field.replace('_','\/').title()+r'}$'
+            field_name = r'$\rm{'+field.replace('_','\ ').title()+r'}$'
         elif field_name.find('$') == -1:
-            field_name = field_name.replace(' ','\/')
+            field_name = field_name.replace(' ','\ ')
             field_name = r'$\rm{'+field_name+r'}$'
         if fractional:
-            label = field_name + r'$\rm{\/Probability\/Density}$'
+            label = field_name + r'$\rm{\ Probability\ Density}$'
         elif field_unit is None or field_unit == '':
             label = field_name
         else:
-            label = field_name+r'$\/\/('+field_unit+r')$'
+            label = field_name+r'$\ \ ('+field_unit+r')$'
         return label
 
     def _get_field_title(self, field_y, profile):
@@ -615,10 +648,10 @@ class PhasePlot(ImagePlotContainer):
     y field, and z field (or fields), this will create a two-dimensional 
     profile of the average (or total) value of the z field in bins of the 
     x and y fields.
-    
+
     Parameters
     ----------
-    data_source : AMR3DData Object
+    data_source : YTSelectionContainer Object
         The data object to be profiled, such as all_data, region, or 
         sphere.
     x_field : str
@@ -673,7 +706,7 @@ class PhasePlot(ImagePlotContainer):
     >>> plot.set_cmap("cell_mass", "jet")
     >>> plot.set_zlim("cell_mass", 1e8, 1e13)
     >>> plot.set_title("cell_mass", "This is a phase plot")
-    
+
     """
     x_log = None
     y_log = None
@@ -681,7 +714,6 @@ class PhasePlot(ImagePlotContainer):
     _plot_valid = False
     _plot_type = 'Phase'
 
-
     def __init__(self, data_source, x_field, y_field, z_fields,
                  weight_field="cell_mass", x_bins=128, y_bins=128,
                  accumulation=False, fractional=False,
@@ -744,18 +776,18 @@ class PhasePlot(ImagePlotContainer):
         if isinstance(field, tuple): field = field[1]
         if field_name is None:
             field_name = r'$\rm{'+field+r'}$'
-            field_name = r'$\rm{'+field.replace('_','\/').title()+r'}$'
+            field_name = r'$\rm{'+field.replace('_','\ ').title()+r'}$'
         elif field_name.find('$') == -1:
-            field_name = field_name.replace(' ','\/')
+            field_name = field_name.replace(' ','\ ')
             field_name = r'$\rm{'+field_name+r'}$'
         if fractional:
-            label = field_name + r'$\rm{\/Probability\/Density}$'
+            label = field_name + r'$\rm{\ Probability\ Density}$'
         elif field_unit is None or field_unit is '':
             label = field_name
         else:
-            label = field_name+r'$\/\/('+field_unit+r')$'
+            label = field_name+r'$\ \ ('+field_unit+r')$'
         return label
-        
+
     def _get_field_log(self, field_z, profile):
         ds = profile.data_source.ds
         zf, = profile.data_source._determine_fields([field_z])
@@ -776,6 +808,8 @@ class PhasePlot(ImagePlotContainer):
         return scales[x_log], scales[y_log], scales[z_log]
 
     def _setup_plots(self):
+        if self._plot_valid:
+            return
         for f, data in self.profile.items():
             fig = None
             axes = None
@@ -814,21 +848,29 @@ class PhasePlot(ImagePlotContainer):
                     self._field_transform[f] = linear_transform
                 zlim = [zmin, np.nanmax(data)]
 
-            fp = self._font_properties
+            font_size = self._font_properties.get_size()
             f = self.profile.data_source._determine_fields(f)[0]
 
+            # if this is a Particle Phase Plot AND if we using a single color,
+            # override the colorbar here.
+            splat_color = getattr(self, "splat_color", None)
+            if splat_color is not None:
+                cmap = matplotlib.colors.ListedColormap(splat_color, 'dummy')
+            else:
+                cmap = self._colormaps[f]
+
             self.plots[f] = PhasePlotMPL(self.profile.x, self.profile.y, data,
                                          x_scale, y_scale, z_scale,
-                                         self._colormaps[f], zlim,
-                                         self.figure_size, fp.get_size(),
+                                         cmap, zlim,
+                                         self.figure_size, font_size,
                                          fig, axes, cax)
 
             self.plots[f]._toggle_axes(draw_axes)
             self.plots[f]._toggle_colorbar(draw_colorbar)
 
-            self.plots[f].axes.xaxis.set_label_text(x_title, fontproperties=fp)
-            self.plots[f].axes.yaxis.set_label_text(y_title, fontproperties=fp)
-            self.plots[f].cax.yaxis.set_label_text(z_title, fontproperties=fp)
+            self.plots[f].axes.xaxis.set_label_text(x_title)
+            self.plots[f].axes.yaxis.set_label_text(y_title)
+            self.plots[f].cax.yaxis.set_label_text(z_title)
 
             if f in self._plot_text:
                 self.plots[f].axes.text(self._text_xpos[f], self._text_ypos[f],
@@ -839,15 +881,34 @@ class PhasePlot(ImagePlotContainer):
             if f in self.plot_title:
                 self.plots[f].axes.set_title(self.plot_title[f])
 
-            ax = self.plots[f].axes
-            cbax = self.plots[f].cb.ax
-            labels = ((ax.xaxis.get_ticklabels() + ax.yaxis.get_ticklabels() +
-                       cbax.yaxis.get_ticklabels()) +
-                      [ax.xaxis.label, ax.yaxis.label, cbax.yaxis.label])
-            for label in labels:
-                label.set_fontproperties(fp)
-                if self._font_color is not None:
-                    label.set_color(self._font_color)
+            # x-y axes minorticks
+            if f not in self._minorticks:
+                self._minorticks[f] = True
+            if self._minorticks[f] is True:
+                self.plots[f].axes.minorticks_on()
+            else:
+                self.plots[f].axes.minorticks_off()
+
+            # colorbar minorticks
+            if f not in self._cbar_minorticks:
+                self._cbar_minorticks[f] = True
+            if self._cbar_minorticks[f] is True:
+                if self._field_transform[f] == linear_transform:
+                    self.plots[f].cax.minorticks_on()
+                else:
+                    vmin = np.float64( self.plots[f].cb.norm.vmin )
+                    vmax = np.float64( self.plots[f].cb.norm.vmax )
+                    mticks = self.plots[f].image.norm( get_log_minorticks(vmin, vmax) )
+                    self.plots[f].cax.yaxis.set_ticks(mticks, minor=True)
+            else:
+                self.plots[f].cax.minorticks_off()
+
+        self._set_font_properties()
+
+        # if this is a particle plot with one color only, hide the cbar here
+        if hasattr(self, "use_cbar") and self.use_cbar is False:
+            self.plots[f].hide_colorbar()
+
         self._plot_valid = True
 
     @classmethod
@@ -886,14 +947,13 @@ class PhasePlot(ImagePlotContainer):
         return cls._initialize_instance(obj, data_source, profile, fontsize,
                                         figure_size)
 
-
     def annotate_text(self, xpos=0.0, ypos=0.0, text=None, **text_kwargs):
         r"""
         Allow the user to insert text onto the plot
         The x-position and y-position must be given as well as the text string. 
         Add *text* tp plot at location *xpos*, *ypos* in plot coordinates
         (see example below).
-                
+
         Parameters
         ----------
         field: str or tuple
@@ -921,7 +981,8 @@ class PhasePlot(ImagePlotContainer):
             self._text_kwargs[f] = text_kwargs
         return self
 
-    def save(self, name=None, mpl_kwargs=None):
+    @validate_plot
+    def save(self, name=None, suffix=None, mpl_kwargs=None):
         r"""
         Saves a 2d profile plot.
 
@@ -929,11 +990,14 @@ class PhasePlot(ImagePlotContainer):
         ----------
         name : str
             The output file keyword.
+        suffix : string
+           Specify the image type by its suffix. If not specified, the output
+           type will be inferred from the filename. Defaults to PNG.
         mpl_kwargs : dict
            A dict of keyword arguments to be passed to matplotlib.
 
         >>> plot.save(mpl_kwargs={'bbox_inches':'tight'})
-        
+
         """
         names = []
         if not self._plot_valid:
@@ -945,13 +1009,13 @@ class PhasePlot(ImagePlotContainer):
         name = os.path.expanduser(name)
         xfn = self.profile.x_field
         yfn = self.profile.y_field
-        if isinstance(xfn, types.TupleType):
+        if isinstance(xfn, tuple):
             xfn = xfn[1]
-        if isinstance(yfn, types.TupleType):
+        if isinstance(yfn, tuple):
             yfn = yfn[1]
         for f in self.profile.field_data:
             _f = f
-            if isinstance(f, types.TupleType):
+            if isinstance(f, tuple):
                 _f = _f[1]
             middle = "2d-Profile_%s_%s_%s" % (xfn, yfn, _f)
             splitname = os.path.split(name)
@@ -962,12 +1026,15 @@ class PhasePlot(ImagePlotContainer):
                 prefix += str(self.profile.ds)
             else:
                 prefix = name
-            suffix = get_image_suffix(name)
-            if suffix != '':
-                for k, v in self.plots.iteritems():
-                    names.append(v.save(name, mpl_kwargs))
-                return names
-            fn = "%s_%s%s" % (prefix, middle, '.png')
+            if suffix is None:
+                suffix = get_image_suffix(name)
+                if suffix != '':
+                    for k, v in iteritems(self.plots):
+                        names.append(v.save(name, mpl_kwargs))
+                    return names
+                else:
+                    suffix = "png"
+            fn = "%s_%s.%s" % (prefix, middle, suffix)
             names.append(fn)
             self.plots[f].save(fn, mpl_kwargs)
         return names
@@ -987,10 +1054,10 @@ class PhasePlot(ImagePlotContainer):
         --------
 
         >>> plot.set_title("cell_mass", "This is a phase plot")
-        
-        """
 
-        self.plot_title[field] = title
+        """
+        self.plot_title[self.data_source._determine_fields(field)[0]] = title
+        return self
 
     @invalidate_plot
     def reset_plot(self):
@@ -1054,7 +1121,7 @@ class PhasePlot(ImagePlotContainer):
 
         Parameters
         ----------
-        
+
         xmin : float or None
           The new x minimum.  Defaults to None, which leaves the xmin
           unchanged.
@@ -1084,16 +1151,30 @@ class PhasePlot(ImagePlotContainer):
         extrema = {p.x_field: ((xmin, str(p.x.units)), (xmax, str(p.x.units))),
                    p.y_field: ((p.y_bins.min(), str(p.y.units)),
                                (p.y_bins.max(), str(p.y.units)))}
+        if self.x_log is not None or self.y_log is not None:
+            logs = {}
+        else:
+            logs = None
+        if self.x_log is not None:
+            logs[p.x_field] = self.x_log
+        if self.y_log is not None:
+            logs[p.y_field] = self.y_log
+        deposition = getattr(self.profile, "deposition", None)
+        if deposition is None:
+            additional_kwargs = {'accumulation': p.accumulation,
+                                 'fractional': p.fractional}
+        else:
+            additional_kwargs = {'deposition': p.deposition}
         self.profile = create_profile(
             p.data_source,
             [p.x_field, p.y_field],
-            p.field_map.values(),
-            n_bins=[len(p.x_bins)-2, len(p.y_bins)-2],
+            list(p.field_map.values()),
+            n_bins=[len(p.x_bins)-1, len(p.y_bins)-1],
             weight_field=p.weight_field,
-            accumulation=p.accumulation,
-            fractional=p.fractional,
             units=units,
-            extrema=extrema)
+            extrema=extrema,
+            logs=logs,
+            **additional_kwargs)
         for field in zunits:
             self.profile.set_field_unit(field, zunits[field])
         return self
@@ -1134,25 +1215,34 @@ class PhasePlot(ImagePlotContainer):
         extrema = {p.x_field: ((p.x_bins.min(), str(p.x.units)),
                                (p.x_bins.max(), str(p.x.units))),
                    p.y_field: ((ymin, str(p.y.units)), (ymax, str(p.y.units)))}
+        if self.x_log is not None or self.y_log is not None:
+            logs = {}
+        else:
+            logs = None
+        if self.x_log is not None:
+            logs[p.x_field] = self.x_log
+        if self.y_log is not None:
+            logs[p.y_field] = self.y_log
+        deposition = getattr(self.profile, "deposition", None)
+        if deposition is None:
+            additional_kwargs = {'accumulation': p.accumulation,
+                                 'fractional': p.fractional}
+        else:
+            additional_kwargs = {'deposition': p.deposition}
         self.profile = create_profile(
             p.data_source,
             [p.x_field, p.y_field],
-            p.field_map.values(),
-            n_bins=[len(p.x_bins), len(p.y_bins)],
+            list(p.field_map.values()),
+            n_bins=[len(p.x_bins)-1, len(p.y_bins)-1],
             weight_field=p.weight_field,
-            accumulation=p.accumulation,
-            fractional=p.fractional,
             units=units,
-            extrema=extrema)
+            extrema=extrema,
+            logs=logs,
+            **additional_kwargs)
         for field in zunits:
             self.profile.set_field_unit(field, zunits[field])
         return self
 
-    def run_callbacks(self, *args):
-        raise NotImplementedError
-    def setup_callbacks(self, *args):
-        raise NotImplementedError
-
 
 class PhasePlotMPL(ImagePlotMPL):
     """A container for a single matplotlib figure and axes for a PhasePlot"""
@@ -1198,6 +1288,7 @@ class PhasePlotMPL(ImagePlotMPL):
                                           np.array(image_data.T),
                                           norm=norm,
                                           cmap=cmap)
+
         self.axes.set_xscale(x_scale)
         self.axes.set_yscale(y_scale)
         self.cb = self.figure.colorbar(self.image, self.cax)
