@@ -27,6 +27,7 @@ from yt.utilities.answer_testing.framework import \
 from yt.visualization.api import \
     SlicePlot, ProjectionPlot, OffAxisSlicePlot, OffAxisProjectionPlot
 from yt.units.yt_array import YTArray, YTQuantity
+from collections import OrderedDict
 
 def setup():
     """Test specific setup."""
@@ -41,21 +42,21 @@ def assert_fname(fname):
 
     with open(fname, 'rb') as fimg:
         data = fimg.read()
-    data = str(data)
     image_type = ''
 
     # see http://www.w3.org/TR/PNG/#5PNG-file-signature
-    if data.startswith('\211PNG\r\n\032\n'):
+    if data.startswith(b'\211PNG\r\n\032\n'):
         image_type = '.png'
     # see http://www.mathguide.de/info/tools/media-types/image/jpeg
-    elif data.startswith('\377\330'):
+    elif data.startswith(b'\377\330'):
         image_type = '.jpeg'
-    elif data.startswith('%!PS-Adobe'):
-        if 'EPSF' in data[:data.index('\n')]:
+    elif data.startswith(b'%!PS-Adobe'):
+        data_str = data.decode("utf-8", "ignore")
+        if 'EPSF' in data_str[:data_str.index('\n')]:
             image_type = '.eps'
         else:
             image_type = '.ps'
-    elif data.startswith('%PDF'):
+    elif data.startswith(b'%PDF'):
         image_type = '.pdf'
 
     return image_type == os.path.splitext(fname)[1]
@@ -66,6 +67,9 @@ TEST_FLNMS = [None, 'test', 'test.png', 'test.eps',
 M7 = "DD0010/moving7_0010"
 WT = "WindTunnel/windtunnel_4lev_hdf5_plt_cnt_0030"
 
+FPROPS = {'family': 'sans-serif', 'style': 'italic',
+          'weight': 'bold', 'size': 24}
+
 ATTR_ARGS = {"pan": [(((0.1, 0.1), ), {})],
              "pan_rel": [(((0.1, 0.1), ), {})],
              "set_axes_unit": [(("kpc", ), {}),
@@ -77,8 +81,8 @@ ATTR_ARGS = {"pan": [(((0.1, 0.1), ), {})],
              "set_center": [(((0.4, 0.3), ), {})],
              "set_cmap": [(('density', 'RdBu'), {}),
                           (('density', 'kamae'), {})],
-             "set_font": [(({'family': 'sans-serif', 'style': 'italic',
-                             'weight': 'bold', 'size': 24}, ), {})],
+             "set_font": [((OrderedDict(sorted(FPROPS.items(), key=lambda t: t[0])), ),
+                           {})],
              "set_log": [(('density', False), {})],
              "set_window_size": [((7.0, ), {})],
              "set_zlim": [(('density', 1e-25, 1e-23), {}),
@@ -152,6 +156,28 @@ PROJECTION_METHODS = (
     'mip'
 )
 
+def simple_contour(test_obj, plot):
+    plot.annotate_contour(test_obj.plot_field)
+
+def simple_velocity(test_obj, plot):
+    plot.annotate_velocity()
+
+def simple_streamlines(test_obj, plot):
+    ax = test_obj.plot_axis
+    xax = test_obj.ds.coordinates.x_axis[ax]
+    yax = test_obj.ds.coordinates.y_axis[ax]
+    xn = test_obj.ds.coordinates.axis_name[xax]
+    yn = test_obj.ds.coordinates.axis_name[yax]
+    plot.annotate_streamlines("velocity_%s" % xn,
+                              "velocity_%s" % yn)
+
+CALLBACK_TESTS = (
+    ("simple_contour", (simple_contour,)),
+    ("simple_velocity", (simple_velocity,)),
+    #("simple_streamlines", (simple_streamlines,)),
+    #("simple_all", (simple_contour, simple_velocity, simple_streamlines)),
+)
+
 @requires_ds(M7)
 def test_attributes():
     """Test plot member functions that aren't callbacks"""
@@ -166,6 +192,9 @@ def test_attributes():
                                                args, decimals)
                 test_attributes.__name__ = test.description
                 yield test
+                for n, r in CALLBACK_TESTS:
+                    yield PlotWindowAttributeTest(ds, plot_field, ax, attr_name,
+                        args, decimals, callback_id=n, callback_runners=r)
 
 
 @requires_ds(WT)
@@ -179,7 +208,42 @@ def test_attributes_wt():
         for args in ATTR_ARGS[attr_name]:
             yield PlotWindowAttributeTest(ds, plot_field, ax, attr_name,
                                           args, decimals)
+            for n, r in CALLBACK_TESTS:
+                yield PlotWindowAttributeTest(ds, plot_field, ax, attr_name,
+                    args, decimals, callback_id=n, callback_runners=r)
 
+class TestHideAxesColorbar(unittest.TestCase):
+
+    ds = None
+
+    def setUp(self):
+        if self.ds is None:
+            self.ds = fake_random_ds(64)
+            self.slc = SlicePlot(self.ds, 0, "density")
+        self.tmpdir = tempfile.mkdtemp()
+        self.curdir = os.getcwd()
+        os.chdir(self.tmpdir)
+
+    def tearDown(self):
+        os.chdir(self.curdir)
+        shutil.rmtree(self.tmpdir)
+
+    def test_hide_show_axes(self):
+        self.slc.hide_axes()
+        self.slc.save()
+        self.slc.show_axes()
+        self.slc.save()
+
+    def test_hide_show_colorbar(self):
+        self.slc.hide_colorbar()
+        self.slc.save()
+        self.slc.show_colorbar()
+        self.slc.save()
+
+    def test_hide_axes_colorbar(self):
+        self.slc.hide_colorbar()
+        self.slc.hide_axes()
+        self.slc.save()
 
 class TestSetWidth(unittest.TestCase):
 
@@ -323,6 +387,9 @@ class TestPlotWindowSave(unittest.TestCase):
     def test_offaxis_projection_plot(self, fname):
         assert assert_fname(self.offaxis_proj.save(fname)[0])
 
+    def test_ipython_repr(self):
+        self.slices[0]._repr_html_()
+
     @parameterized.expand(
         param.explicit((width, ))
         for width in WIDTH_SPECS)
