@@ -22,6 +22,7 @@ from .image_handling import export_rgba, import_rgba, \
 
 from .camera import Camera, PerspectiveCamera, StereoPairCamera, \
     off_axis_projection, FisheyeCamera, MosaicFisheyeCamera, \
-    HEALpixCamera, InteractiveCamera, ProjectionCamera
+    HEALpixCamera, InteractiveCamera, ProjectionCamera, \
+    SphericalCamera, StereoSphericalCamera
 
 from .transfer_function_helper import TransferFunctionHelper
