@@ -33,16 +33,14 @@ def export_rgba(image, fn, h5=True, fits=False, ):
         f.create_dataset("A", data=image[:,:,3])
         f.close()
     if fits:
-        from yt.utilities.fits_image import FITSImageBuffer
+        from yt.utilities.fits_image import FITSImageData
         data = {}
         data["r"] = image[:,:,0]
         data["g"] = image[:,:,1]
         data["b"] = image[:,:,2]
         data["a"] = image[:,:,3]
         nx, ny = data["r"].shape
-        fib = FITSImageBuffer(data, units="pixel",
-                              center=[0.5*(nx+1), 0.5*(ny+1)],
-                              scale=[1.]*2)
+        fib = FITSImageData(data)
         fib.writeto('%s.fits'%fn,clobber=True)
 
 def import_rgba(name, h5=True):
